@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqAnimationManager.cxx,v $
+   Module:    pqAnimationManager.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -36,13 +36,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkMath.h"
 #include "vtkProcessModule.h"
 #include "vtkPVServerInformation.h"
+#include "vtkRenderWindow.h"
 #include "vtkSMAnimationSceneGeometryWriter.h"
-#include "vtkSMAnimationSceneProxy.h"
 #include "vtkSmartPointer.h"
+#include "vtkSMProxy.h"
 #include "vtkSMProxyManager.h"
-#include "vtkSMServerProxyManagerReviver.h"
+#include "vtkSMSession.h"
 
-#include <QApplication>
+#include <QIntValidator>
 #include <QFileInfo>
 #include <QMap>
 #include <QMessageBox>
@@ -50,19 +51,22 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QSize>
 #include <QtDebug>
 
+
 #include "pqAnimationCue.h"
 #include "pqAnimationScene.h"
 #include "pqAnimationSceneImageWriter.h"
 #include "pqApplicationCore.h"
+#include "pqCoreUtilities.h"
 #include "pqEventDispatcher.h"
 #include "pqFileDialog.h"
 #include "pqObjectBuilder.h"
 #include "pqProgressManager.h"
 #include "pqProxy.h"
+#include "pqRenderViewBase.h"
 #include "pqServer.h"
 #include "pqServerManagerModel.h"
+#include "pqSettings.h"
 #include "pqSMAdaptor.h"
-#include "pqView.h"
 #include "pqViewManager.h"
 
 #define SEQUENCE 0
@@ -74,13 +78,14 @@ class pqAnimationManager::pqInternals
 {
 public:
   QPointer<pqServer> ActiveServer;
-  QPointer<pqViewManager> ViewWidget;
   typedef QMap<pqServer*, QPointer<pqAnimationScene> > SceneMap;
   SceneMap Scenes;
-  Ui::Dialog* AnimationSettingsDialog;
+  Ui::pqAnimationSettingsDialog* AnimationSettingsDialog;
 
   QSize OldMaxSize;
   QSize OldSize;
+
+  double AspectRatio;
 };
 
 //-----------------------------------------------------------------------------
@@ -99,20 +104,17 @@ pqAnimationManager::pqAnimationManager(QObject* _parent/*=0*/)
     this, SLOT(updateViewModules()));
   QObject::connect(smmodel, SIGNAL(viewRemoved(pqView*)),
     this, SLOT(updateViewModules()));
+  
+  this->restoreSettings();
 }
 
 //-----------------------------------------------------------------------------
 pqAnimationManager::~pqAnimationManager()
 {
+  this->saveSettings();
   delete this->Internals;
 }
 
-//-----------------------------------------------------------------------------
-void pqAnimationManager::setViewWidget(pqViewManager* w)
-{
-  this->Internals->ViewWidget = w;
-}
-
 //-----------------------------------------------------------------------------
 void pqAnimationManager::updateViewModules()
 {
@@ -134,7 +136,7 @@ void pqAnimationManager::updateViewModules()
 
   emit this->beginNonUndoableChanges();
 
-  vtkSMAnimationSceneProxy* sceneProxy = scene->getAnimationSceneProxy();
+  vtkSMProxy* sceneProxy = scene->getProxy();
   pqSMAdaptor::setProxyListProperty(sceneProxy->GetProperty("ViewModules"),
     viewList);
   sceneProxy->UpdateProperty("ViewModules");
@@ -280,6 +282,47 @@ void pqAnimationManager::updateGUI()
     }
 }
 
+//-----------------------------------------------------------------------------
+void pqAnimationManager::onWidthEdited()
+{
+  Ui::pqAnimationSettingsDialog *dialog
+    = this->Internals->AnimationSettingsDialog;
+  if (dialog->lockAspect->isChecked())
+    {
+    int width = dialog->width->text().toInt();
+    int height = static_cast<int>(width/this->Internals->AspectRatio);
+    dialog->height->setText(QString::number(height));
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqAnimationManager::onHeightEdited()
+{
+  Ui::pqAnimationSettingsDialog *dialog
+    = this->Internals->AnimationSettingsDialog;
+  if (dialog->lockAspect->isChecked())
+    {
+    int height = dialog->height->text().toInt();
+    int width = static_cast<int>(height*this->Internals->AspectRatio);
+    dialog->width->setText(QString::number(width));
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqAnimationManager::onLockAspectRatio(bool lock)
+{
+  if (lock)
+    {
+    Ui::pqAnimationSettingsDialog *dialog
+      = this->Internals->AnimationSettingsDialog;
+    int width = dialog->width->text().toInt();
+    int height = dialog->height->text().toInt();
+    this->Internals->AspectRatio
+      = static_cast<double>(width)/static_cast<double>(height);
+    }
+}
+
+//-----------------------------------------------------------------------------
 #define PADDING_COMPENSATION QSize(16, 16);
 inline void enforceMultiple4(QSize& newSize)
 {
@@ -315,24 +358,44 @@ bool pqAnimationManager::saveAnimation()
     }
   // Ensure that GUI is up-to-date so that we get correct sizes.
   pqEventDispatcher::processEventsAndWait(1);
-  vtkSMAnimationSceneProxy* sceneProxy = scene->getAnimationSceneProxy();
+  vtkSMProxy* sceneProxy = scene->getProxy();
 
   QDialog dialog;
-  Ui::Dialog dialogUI;
+  Ui::pqAnimationSettingsDialog dialogUI;
   this->Internals->AnimationSettingsDialog = &dialogUI;
   dialogUI.setupUi(&dialog);
 
+  QIntValidator *intValidator = new QIntValidator(this);
+  intValidator->setBottom(50);
+  dialogUI.width->setValidator(intValidator);
+  intValidator = new QIntValidator(this);
+  intValidator->setBottom(50);
+  dialogUI.height->setValidator(intValidator);
+
   // Cannot disconnect and save animation unless connected to a remote server.
   dialogUI.checkBoxDisconnect->setEnabled(
     this->Internals->ActiveServer->isRemote());
 
+  // Use viewManager is available.
+  pqViewManager* viewManager = qobject_cast<pqViewManager*>(
+    pqApplicationCore::instance()->manager("MULTIVIEW_MANAGER"));
+  
   // Set current size of the window.
-  QSize viewSize = this->Internals->ViewWidget->clientSize();
+  QSize viewSize = viewManager? viewManager->clientSize() : QSize(800, 600);
   // to avoid some unpredicable padding issues, I am reducing the size by a few
   // pixels.
-  viewSize -= PADDING_COMPENSATION;
-  dialogUI.spinBoxHeight->setValue(viewSize.height());
-  dialogUI.spinBoxWidth->setValue(viewSize.width());
+  QSize padding = PADDING_COMPENSATION;
+  if (viewSize.width() > viewSize.height())
+    {
+    padding.setWidth((padding.width()*viewSize.width())/viewSize.height());
+    }
+  else
+    {
+    padding.setHeight((padding.height()*viewSize.height())/viewSize.width());
+    }
+  viewSize -= padding;
+  dialogUI.height->setText(QString::number(viewSize.height()));
+  dialogUI.width->setText(QString::number(viewSize.width()));
 
   // Frames per timestep is only shown
   // when saving in SNAP_TO_TIMESTEPS mode.
@@ -393,6 +456,13 @@ bool pqAnimationManager::saveAnimation()
     dialogUI.spinBoxFramesPerTimestep, SIGNAL(valueChanged(int)),
     this, SLOT(updateGUI()));
 
+  QObject::connect(dialogUI.width, SIGNAL(editingFinished()),
+                   this, SLOT(onWidthEdited()));
+  QObject::connect(dialogUI.height, SIGNAL(editingFinished()),
+                   this, SLOT(onHeightEdited()));
+  QObject::connect(dialogUI.lockAspect, SIGNAL(toggled(bool)),
+                   this, SLOT(onLockAspectRatio(bool)));
+
   if (!dialog.exec())
     {
     this->Internals->AnimationSettingsDialog = 0;
@@ -402,13 +472,41 @@ bool pqAnimationManager::saveAnimation()
 
   bool disconnect_and_save = 
     (dialogUI.checkBoxDisconnect->checkState() == Qt::Checked);
+  int stereo = dialogUI.stereoMode->currentIndex();
+  if (stereo)
+    {
+    QString stereoMode = dialogUI.stereoMode->currentText();
+    if (stereoMode == "Red-Blue")
+      {
+      stereo = VTK_STEREO_RED_BLUE;
+      }
+    else if (stereoMode == "Interlaced")
+      {
+      stereo = VTK_STEREO_INTERLACED;
+      }
+    else if (stereoMode == "Checkerboard")
+      {
+      stereo = VTK_STEREO_CHECKERBOARD;
+      }
+    else if (stereoMode == "Left Eye Only")
+      {
+      stereo = VTK_STEREO_LEFT;
+      }
+    else if (stereoMode == "Right Eye Only")
+      {
+      stereo = VTK_STEREO_RIGHT;
+      }
+    else
+      {
+      stereo = 0;
+      }
+    }
 
   // Now obtain filename for the animation.
   vtkSmartPointer<vtkPVServerInformation> serverInfo;
   if (disconnect_and_save)
     {
-    serverInfo = vtkProcessModule::GetProcessModule()->GetServerInformation(
-      scene->getServer()->GetConnectionID());
+    serverInfo = sceneProxy->GetSession()->GetServerInformation();
     if (!serverInfo)
       {
       qWarning() << "Failed to locate server information about AVI support.";
@@ -423,6 +521,10 @@ bool pqAnimationManager::saveAnimation()
     }
 
   QString filters = "";
+  if (serverInfo && serverInfo->GetOGVSupport())
+    {
+    filters += "Ogg/Theora files (*.ogv);;";
+    }
   if (serverInfo && serverInfo->GetAVISupport())
     {
     filters += "AVI files (*.avi);;";
@@ -430,13 +532,7 @@ bool pqAnimationManager::saveAnimation()
   filters +="JPEG images (*.jpg);;TIFF images (*.tif);;PNG images (*.png);;";
   filters +="All files(*)";
 
-  QWidget* parent_window = qobject_cast<QWidget*>(this->parent());
-  if (!parent_window)
-    {
-    // QApplication::activateWindow() is set up differntly on Macs resulting in
-    // ambiguities in test playback. Hence we use it as the last resort.
-    parent_window = QApplication::activeWindow();
-    }
+  QWidget* parent_window = pqCoreUtilities::mainWidget();
 
   // Create a server dialog is disconnect-and-save is true, else create a client
   // dialog.
@@ -444,6 +540,7 @@ bool pqAnimationManager::saveAnimation()
     disconnect_and_save?  scene->getServer() : 0,
     parent_window,
     tr("Save Animation"), QString(), filters);
+  file_dialog->setRecentlyUsedExtension(this->AnimationExtension);
   file_dialog->setObjectName("FileSaveAnimationDialog");
   file_dialog->setFileMode(pqFileDialog::AnyFile);
   if (file_dialog->exec() != QDialog::Accepted)
@@ -452,7 +549,10 @@ bool pqAnimationManager::saveAnimation()
     return false;
     }
 
-  QStringList files = file_dialog->getSelectedFiles();
+  QStringList files  = file_dialog->getSelectedFiles();
+  QFileInfo fileInfo = QFileInfo( files[0] );
+  this->AnimationExtension = QString("*.") + fileInfo.suffix();
+  
   // essential to destroy file dialog, before we disconnect from the server, if
   // at all.
   delete file_dialog;
@@ -467,6 +567,11 @@ bool pqAnimationManager::saveAnimation()
   // Update Scene properties based on user options. 
   emit this->beginNonUndoableChanges();
 
+  if (stereo)
+    {
+    pqRenderViewBase::setStereo(stereo);
+    }
+
   switch (playMode)
     {
   case SEQUENCE:
@@ -493,20 +598,19 @@ bool pqAnimationManager::saveAnimation()
 
   sceneProxy->UpdateVTKObjects();
 
-  QSize newSize(dialogUI.spinBoxWidth->value(),
-    dialogUI.spinBoxHeight->value());
+  QSize newSize(dialogUI.width->text().toInt(),
+                dialogUI.height->text().toInt());
 
   // Enforce any view size conditions (such a multiple of 4). 
   ::enforceMultiple4(newSize); 
-  int magnification = this->Internals->ViewWidget->prepareForCapture(newSize);
+  int magnification = viewManager? viewManager->prepareForCapture(newSize): 1;
  
   if (disconnect_and_save)
     {
     pqServer* server = this->Internals->ActiveServer;
-    vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
+    vtkSMProxyManager* pxm = server->proxyManager();
 
     vtkSMProxy* writer = pxm->NewProxy("writers", "AnimationSceneImageWriter");
-    writer->SetConnectionID(server->GetConnectionID());
     pxm->RegisterProxy("animation", "writer", writer);
     writer->Delete();
 
@@ -521,24 +625,20 @@ bool pqAnimationManager::saveAnimation()
     // We save the animation offline.
     vtkSMProxy* cleaner = 
       pxm->NewProxy("connection_cleaners", "AnimationPlayer");
-    cleaner->SetConnectionID(server->GetConnectionID());
     pxm->RegisterProxy("animation","cleaner",cleaner);
     cleaner->Delete();
 
     pqSMAdaptor::setProxyProperty(cleaner->GetProperty("Writer"), writer);
     cleaner->UpdateVTKObjects();
 
-    vtkSMServerProxyManagerReviver* reviver = 
-      vtkSMServerProxyManagerReviver::New();
-    int status = reviver->ReviveRemoteServerManager(server->GetConnectionID());
-    reviver->Delete();
-    emit this->endNonUndoableChanges();
-    pqApplicationCore::instance()->getObjectBuilder()->removeServer(server);
-    this->Internals->ViewWidget->finishedCapture();
-    emit this->disconnectServer();
-    return status;
+    qCritical("Saving animations after disconnecting from servers "
+      "is temporarily not supported.");
+    return false;
     }
 
+  // let the world know we are writing an animation.
+  emit this->writeAnimation(filename, magnification, dialogUI.frameRate->value());
+
   vtkSMAnimationSceneImageWriter* writer = pqAnimationSceneImageWriter::New();
   writer->SetFileName(filename.toAscii().data());
   writer->SetMagnification(magnification);
@@ -585,7 +685,15 @@ bool pqAnimationManager::saveAnimation()
     break;
     }
   sceneProxy->UpdateVTKObjects();
-  this->Internals->ViewWidget->finishedCapture();
+  if (viewManager)
+    {
+    viewManager->finishedCapture();
+    }
+
+  if (stereo)
+    {
+    pqRenderViewBase::setStereo(0);
+    }
   emit this->endNonUndoableChanges();
   return status;
 }
@@ -604,8 +712,7 @@ bool pqAnimationManager::saveGeometry(const QString& filename,
     {
     return false;
     }
-  vtkSMAnimationSceneProxy* sceneProxy = scene->getAnimationSceneProxy();
-
+  vtkSMProxy* sceneProxy = scene->getProxy();
   vtkSMAnimationSceneGeometryWriter* writer = vtkSMAnimationSceneGeometryWriter::New();
   writer->SetFileName(filename.toAscii().data());
   writer->SetAnimationScene(sceneProxy);
@@ -615,6 +722,39 @@ bool pqAnimationManager::saveGeometry(const QString& filename,
   return status;
 }
 
+//-----------------------------------------------------------------------------
+void pqAnimationManager::restoreSettings()
+{
+  // Load the most recently used file extension from QSettings, if available.
+  pqSettings* settings = pqApplicationCore::instance()->settings();
+
+  if ( settings->contains("extensions/AnimationExtension") )
+    {
+    this->AnimationExtension = settings->value("extensions/AnimationExtension").toString();
+    }
+  else
+    {
+    this->AnimationExtension = QString();
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqAnimationManager::saveSettings()
+{ 
+  // Save the most recently used file extension to QSettings.
+  pqSettings* settings = pqApplicationCore::instance()->settings();
+  settings->setValue("extensions/AnimationExtension", this->AnimationExtension);
+}
+
+//-----------------------------------------------------------------------------
+void pqAnimationManager::updateApplicationSettings()
+{
+  foreach (QPointer<pqAnimationScene> scene, this->Internals->Scenes.values())
+    {
+    scene->updateApplicationSettings();
+    }
+}
+
 //-----------------------------------------------------------------------------
 void pqAnimationManager::onTick(int progress)
 {
