@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqAnimationViewWidget.cxx,v $
+   Module:    pqAnimationViewWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -31,46 +31,51 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 =========================================================================*/
 
 #include "pqAnimationViewWidget.h"
+#include "ui_pqPythonAnimationCue.h"
 
-#include <QVBoxLayout>
-#include <QPointer>
-#include <QSignalMapper>
-#include <QDialog>
 #include <QComboBox>
-#include <QLineEdit>
-#include <QSpinBox>
-#include <QLabel>
+#include <QDialog>
 #include <QDialogButtonBox>
 #include <QDoubleValidator>
 #include <QIntValidator>
+#include <QLabel>
+#include <QLineEdit>
+#include <QPointer>
+#include <QPushButton>
+#include <QSignalMapper>
+#include <QSpinBox>
+#include <QtDebug>
+#include <QVBoxLayout>
 
-#include "pqAnimationWidget.h"
-#include "pqAnimationModel.h"
-#include "pqAnimationTrack.h"
-#include "pqAnimationKeyFrame.h"
-
-#include "vtkSMProxy.h"
-#include "vtkSMProperty.h"
-#include "vtkSMAnimationSceneProxy.h"
-#include "vtkSMRenderViewProxy.h"
-
-#include "pqApplicationCore.h"
-#include "pqServerManagerModel.h"
-#include "pqAnimationScene.h"
-#include "pqAnimationCue.h"
-#include "pqSMAdaptor.h"
-#include "pqServer.h"
-#include "pqKeyFrameEditor.h"
-#include "pqPipelineTimeKeyFrameEditor.h"
-#include "pqPropertyLinks.h"
-#include "pqComboBoxDomain.h"
-#include "pqSignalAdaptors.h"
+#include "pqActiveView.h"
 #include "pqAnimatablePropertiesComboBox.h"
 #include "pqAnimatableProxyComboBox.h"
-#include "pqActiveView.h"
+#include "pqAnimationCue.h"
+#include "pqAnimationKeyFrame.h"
+#include "pqAnimationModel.h"
+#include "pqAnimationScene.h"
+#include "pqAnimationTrack.h"
+#include "pqAnimationWidget.h"
+#include "pqApplicationCore.h"
+#include "pqComboBoxDomain.h"
+#include "pqKeyFrameEditor.h"
+#include "pqOrbitCreatorDialog.h"
+#include "pqPipelineTimeKeyFrameEditor.h"
+#include "pqPropertyLinks.h"
 #include "pqRenderView.h"
+#include "pqServer.h"
+#include "pqServerManagerModel.h"
 #include "pqServerManagerSelectionModel.h"
+#include "pqSetName.h"
+#include "pqSignalAdaptors.h"
+#include "pqSMAdaptor.h"
 #include "pqUndoStack.h"
+#include "vtkCamera.h"
+#include "vtkPVConfig.h"
+#include "vtkSMProperty.h"
+#include "vtkSMPropertyHelper.h"
+#include "vtkSMProxy.h"
+#include "vtkSMRenderViewProxy.h"
 
 //-----------------------------------------------------------------------------
 class pqAnimationViewWidget::pqInternal
@@ -99,6 +104,8 @@ public:
   pqPropertyLinks DurationLink;
   pqAnimatableProxyComboBox* CreateSource;
   pqAnimatablePropertiesComboBox* CreateProperty;
+  QPushButton* LockEndTime;
+  QPushButton* LockStartTime;
 
   pqAnimationTrack* findTrack(pqAnimationCue* cue)
     {
@@ -129,6 +136,10 @@ public:
       {
       name = "Camera";
       }
+    else if (this->pythonCue(cue))
+      {
+      name = "Python";
+      }
     else
       {
       pqServerManagerModel* model = 
@@ -196,6 +207,16 @@ public:
     return false;
     }
 
+  /// returns true if the cue is a python cue.
+  bool pythonCue(pqAnimationCue* cue)
+    {
+    if(QString("PythonAnimationCue") == cue->getProxy()->GetXMLName())
+      {
+      return true;
+      }
+    return false;
+    }
+
   int numberOfTicks()
     {
     vtkSMProxy* pxy = this->Scene->getProxy();
@@ -212,24 +233,21 @@ public:
       }
     else if(mode == "Snap To TimeSteps")
       {
-      num = 
-        pqSMAdaptor::getMultipleElementProperty(
-          pxy->GetProperty("TimeSteps")).size();
+      num = this->Scene->getTimeSteps().size(); 
       }
     return num;
     }
 
-  QList<QVariant> ticks()
+  QList<double> ticks()
     {
     vtkSMProxy* pxy = this->Scene->getProxy();
     QString mode =
       pqSMAdaptor::getEnumerationProperty(pxy->GetProperty("PlayMode")).toString();
     if(mode == "Snap To TimeSteps")
       {
-      return pqSMAdaptor::getMultipleElementProperty(
-        pxy->GetProperty("TimeSteps"));
+      return this->Scene->getTimeSteps(); 
       }
-    return QList<QVariant>();
+    return QList<double>();
     }
 };
 
@@ -243,37 +261,65 @@ pqAnimationViewWidget::pqAnimationViewWidget(QWidget* _parent) : QWidget(_parent
   hboxlayout->setMargin(0);
 
   hboxlayout->addWidget(new QLabel("Mode:", this));
-  this->Internal->PlayMode = new QComboBox(this);
+  this->Internal->PlayMode = new QComboBox(this) << pqSetName("PlayMode");
   this->Internal->PlayMode->addItem("Snap to Timesteps");
   hboxlayout->addWidget(this->Internal->PlayMode);
   hboxlayout->addWidget(new QLabel("Time:", this));
-  this->Internal->Time = new QLineEdit(this);
+  this->Internal->Time = new QLineEdit(this) << pqSetName("Time");
   this->Internal->Time->setValidator(
     new QDoubleValidator(this->Internal->Time));
   hboxlayout->addWidget(this->Internal->Time);
   hboxlayout->addWidget(new QLabel("Start Time:", this));
-  this->Internal->StartTime = new QLineEdit(this);
+  this->Internal->StartTime = new QLineEdit(this) << pqSetName("StartTime");
   this->Internal->StartTime->setValidator(
     new QDoubleValidator(this->Internal->StartTime));
   hboxlayout->addWidget(this->Internal->StartTime);
+  this->Internal->LockStartTime = new QPushButton(this) <<
+    pqSetName("LockStartTime");
+  this->Internal->LockStartTime->setIcon(QIcon(":pqWidgets/Icons/pqLock24.png"));
+  this->Internal->LockStartTime->setToolTip(
+    "<html>Lock the start time to keep ParaView from changing it "
+    "as available data times change</html>");
+  this->Internal->LockStartTime->setStatusTip(
+    "<html>Lock the start time to keep ParaView from changing it "
+    "as available data times change</html>");
+  this->Internal->LockStartTime->setCheckable(true);
+  hboxlayout->addWidget(this->Internal->LockStartTime);
   hboxlayout->addWidget(new QLabel("End Time:", this));
-  this->Internal->EndTime = new QLineEdit(this);
+  this->Internal->EndTime = new QLineEdit(this) << pqSetName("EndTime");
   this->Internal->EndTime->setValidator(
     new QDoubleValidator(this->Internal->EndTime));
   hboxlayout->addWidget(this->Internal->EndTime);
+  this->Internal->LockEndTime = new QPushButton(this) <<
+    pqSetName("LockEndTime");
+  this->Internal->LockEndTime->setIcon(QIcon(":pqWidgets/Icons/pqLock24.png"));
+  this->Internal->LockEndTime->setToolTip(
+    "<html>Lock the end time to keep ParaView from changing it"
+    " as available data times change</html>");
+  this->Internal->LockEndTime->setStatusTip(
+    "<html>Lock the end time to keep ParaView from changing it"
+    " as available data times change</html>");
+  this->Internal->LockEndTime->setCheckable(true);
+  hboxlayout->addWidget(this->Internal->LockEndTime);
   this->Internal->DurationLabel = new QLabel(this);
   hboxlayout->addWidget(this->Internal->DurationLabel);
-  this->Internal->Duration = new QSpinBox(this);
+  this->Internal->Duration = new QSpinBox(this) << pqSetName("Duration");
   this->Internal->Duration->setRange(1, (int)(~0u >> 1));
   hboxlayout->addWidget(this->Internal->Duration);
   hboxlayout->addStretch();
 
-  this->Internal->AnimationWidget = new pqAnimationWidget(this);
+  this->Internal->AnimationWidget = new pqAnimationWidget(this)
+    << pqSetName("pqAnimationWidget");
   this->Internal->AnimationWidget->animationModel()->setInteractive(true);
   QWidget* w = this->Internal->AnimationWidget->createDeleteWidget();
 
-  this->Internal->CreateSource = new pqAnimatableProxyComboBox(w);
-  this->Internal->CreateProperty = new pqAnimatablePropertiesComboBox(w);
+  this->Internal->CreateSource = new pqAnimatableProxyComboBox(w)
+    << pqSetName("ProxyCombo");
+#ifdef PARAVIEW_ENABLE_PYTHON
+  this->Internal->CreateSource->addProxy(0, "Python", NULL);
+#endif
+  this->Internal->CreateProperty = new pqAnimatablePropertiesComboBox(w)
+    << pqSetName("PropertyCombo");
   this->Internal->CreateSource->setSizeAdjustPolicy(QComboBox::AdjustToContents);
   this->Internal->CreateProperty->setSizeAdjustPolicy(QComboBox::AdjustToContents);
   QHBoxLayout* l = new QHBoxLayout(w);
@@ -292,8 +338,8 @@ pqAnimationViewWidget::pqAnimationViewWidget(QWidget* _parent) : QWidget(_parent
                    SIGNAL(deleteTrackClicked(pqAnimationTrack*)),
                    this, SLOT(deleteTrack(pqAnimationTrack*)));
   QObject::connect(this->Internal->AnimationWidget,
-                   SIGNAL(deleteTrackClicked(pqAnimationTrack*)),
-                   this, SLOT(deleteTrack(pqAnimationTrack*)));
+                   SIGNAL(enableTrackClicked(pqAnimationTrack*)),
+                   this, SLOT(toggleTrackEnabled(pqAnimationTrack*)));
   QObject::connect(this->Internal->AnimationWidget,
                    SIGNAL(createTrackClicked()),
                    this, SLOT(createTrack()));
@@ -372,6 +418,13 @@ void pqAnimationViewWidget::setScene(pqAnimationScene* scene)
     this->Internal->Links.addPropertyLink(this->Internal->EndTime, "text",
       SIGNAL(editingFinished()), scene->getProxy(),
       scene->getProxy()->GetProperty("EndTime"));
+    // connect lock start time.
+    this->Internal->Links.addPropertyLink(this->Internal->LockStartTime,
+      "checked", SIGNAL(toggled(bool)), scene->getProxy(),
+      scene->getProxy()->GetProperty("LockStartTime"));
+    this->Internal->Links.addPropertyLink(this->Internal->LockEndTime,
+      "checked", SIGNAL(toggled(bool)), scene->getProxy(),
+      scene->getProxy()->GetProperty("LockEndTime"));
 
     QObject::connect(scene, SIGNAL(cuesChanged()), 
       this, SLOT(onSceneCuesChanged()));
@@ -415,17 +468,24 @@ void pqAnimationViewWidget::onSceneCuesChanged()
 
     if(iter == this->Internal->TrackMap.end())
       {
-      pqAnimationTrack* t = animModel->addTrack();
+      pqAnimationTrack* track = animModel->addTrack();
       if(completeName.startsWith("TimeKeeper"))
         {
-        t->setDeletable(false);
+        track->setDeletable(false);
         }
-      this->Internal->TrackMap.insert(cue, t);
-      t->setProperty(completeName);
+      this->Internal->TrackMap.insert(cue, track);
+      track->setProperty(completeName);
       this->Internal->KeyFramesChanged.setMapping(cue, cue);
       QObject::connect(cue, SIGNAL(keyframesModified()),
         &this->Internal->KeyFramesChanged,
         SLOT(map()));
+      QObject::connect(cue, SIGNAL(enabled(bool)),
+        track, SLOT(setEnabled(bool)));
+      track->setEnabled(cue->isEnabled());
+
+      // this ensures that the already present keyframes are loaded currently
+      // (which happens when loading state files).
+      this->keyFramesChanged(cue);
       }
     else
       {
@@ -448,6 +508,7 @@ void pqAnimationViewWidget::onSceneCuesChanged()
 
 }
   
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::keyFramesChanged(QObject* cueObject)
 {
   pqAnimationCue* cue = qobject_cast<pqAnimationCue*>(cueObject);
@@ -501,6 +562,7 @@ void pqAnimationViewWidget::keyFramesChanged(QObject* cueObject)
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::updateSceneTimeRange()
 {
   pqAnimationModel* animModel =
@@ -510,6 +572,7 @@ void pqAnimationViewWidget::updateSceneTimeRange()
   animModel->setEndTime(timeRange.second);
 }
 
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::updateSceneTime()
 {
   double time =
@@ -520,11 +583,15 @@ void pqAnimationViewWidget::updateSceneTime()
   animModel->setCurrentTime(time);
 }
 
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::setCurrentTime(double t)
 {
-  this->Internal->Scene->getAnimationSceneProxy()->SetAnimationTime(t);
+  vtkSMPropertyHelper(
+    this->Internal->Scene->getProxy(), "AnimationTime").Set(t);
+  this->Internal->Scene->getProxy()->UpdateVTKObjects();
 }
   
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::setKeyFrameTime(pqAnimationTrack* track, 
                                             pqAnimationKeyFrame* kf, 
                                             int edge, double time)
@@ -558,6 +625,7 @@ void pqAnimationViewWidget::setKeyFrameTime(pqAnimationTrack* track,
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::trackSelected(pqAnimationTrack* track)
 {
   pqAnimationCue* cue = this->Internal->findCue(track);
@@ -577,6 +645,21 @@ void pqAnimationViewWidget::trackSelected(pqAnimationTrack* track)
     this->Internal->Editor = 
       new pqPipelineTimeKeyFrameEditor(this->Internal->Scene, cue, NULL);
     }
+  else if (this->Internal->pythonCue(cue))
+    {
+    QDialog dialog(this);
+    Ui::PythonAnimationCue ui;
+    ui.setupUi(&dialog);
+    ui.script->setPlainText(
+      vtkSMPropertyHelper(cue->getProxy(), "Script").GetAsString());
+    if (dialog.exec() == QDialog::Accepted)
+      {
+      vtkSMPropertyHelper(cue->getProxy(), "Script").Set(
+        ui.script->toPlainText().toAscii().data());
+      cue->getProxy()->UpdateVTKObjects();
+      }
+    return;
+    }
   else
     {
     this->Internal->Editor = new QDialog;
@@ -607,6 +690,7 @@ void pqAnimationViewWidget::trackSelected(pqAnimationTrack* track)
   this->Internal->Editor->show();
 }
   
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::updatePlayMode()
 {
   pqAnimationModel* animModel =
@@ -662,17 +746,18 @@ void pqAnimationViewWidget::updatePlayMode()
 
 }
   
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::updateTicks()
 {
   pqAnimationModel* animModel =
     this->Internal->AnimationWidget->animationModel();
   if (animModel->mode() == pqAnimationModel::Custom)
     {
-    QList<QVariant> ticks = this->Internal->ticks();
+    QList<double> ticks = this->Internal->ticks();
     double *dticks = new double[ticks.size()+1];
     for (int cc=0; cc < ticks.size(); cc++)
       {
-      dticks[cc] = ticks[cc].toDouble();
+      dticks[cc] = ticks[cc];
       }
     animModel->setTickMarks(ticks.size(), dticks);
     delete [] dticks;
@@ -683,6 +768,20 @@ void pqAnimationViewWidget::updateTicks()
     }
 }
 
+//-----------------------------------------------------------------------------
+void pqAnimationViewWidget::toggleTrackEnabled(pqAnimationTrack* track)
+{
+  pqAnimationCue* cue = this->Internal->findCue(track);
+  if(!cue)
+    {
+    return;
+    }
+  BEGIN_UNDO_SET("Toggle Animation Track");
+  cue->setEnabled(!track->isEnabled());
+  END_UNDO_SET();
+}
+
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::deleteTrack(pqAnimationTrack* track)
 {
   pqAnimationCue* cue = this->Internal->findCue(track);
@@ -690,32 +789,23 @@ void pqAnimationViewWidget::deleteTrack(pqAnimationTrack* track)
     {
     return;
     }
-  pqUndoStack* undo = pqApplicationCore::instance()->getUndoStack();
-  if(undo)
-    {
-    undo->beginUndoSet("Remove Animation Track");
-    }
+  BEGIN_UNDO_SET("Remove Animation Track");
   this->Internal->Scene->removeCue(cue);
-  if(undo)
-    {
-    undo->endUndoSet();
-    }
+  END_UNDO_SET();
 }
 
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::setActiveView(pqView* view)
 {
   pqRenderView* rview = qobject_cast<pqRenderView*>(view);
-
+  this->Internal->CreateSource->removeProxy("Camera");
   if (rview && this->Internal->CreateSource->findText("Camera") == -1)
     {
     this->Internal->CreateSource->addProxy(0, "Camera", rview->getProxy());
     }
-  if (!rview)
-    {
-    this->Internal->CreateSource->removeProxy("Camera");
-    }
 }
 
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::setCurrentSelection(pqServerManagerModelItem* item)
 {
   pqProxy* pxy = qobject_cast<pqProxy*>(item);
@@ -729,11 +819,20 @@ void pqAnimationViewWidget::setCurrentSelection(pqServerManagerModelItem* item)
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::setCurrentProxy(vtkSMProxy* pxy)
 {
   if(vtkSMRenderViewProxy::SafeDownCast(pxy))
     {
-    this->Internal->CreateProperty->setSource(NULL);
+    this->Internal->CreateProperty->setSourceWithoutProperties(pxy);
+    // add camera animation modes as properties for creating the camera
+    // animation track.
+    this->Internal->CreateProperty->addSMProperty(
+      "Orbit", "orbit", 0);
+    this->Internal->CreateProperty->addSMProperty(
+      "Follow Path", "path", 0);
+    this->Internal->CreateProperty->addSMProperty(
+      "Interpolate camera locations", "camera", 0);
     }
   else
     {
@@ -741,44 +840,45 @@ void pqAnimationViewWidget::setCurrentProxy(vtkSMProxy* pxy)
     }
 }
 
-static void pqAnimationPanelResetCameraKeyFrameToCurrent(vtkSMRenderViewProxy* ren,
-                                                         vtkSMProxy* dest)
-{
-  ren->SynchronizeCameraProperties();
-
-  const char* names[] = { "Position", "FocalPoint", "ViewUp", "ViewAngle",  0 };
-  const char* snames[] = { "CameraPositionInfo", "CameraFocalPointInfo", 
-    "CameraViewUpInfo",  "CameraViewAngle", 0 };
-  for (int cc=0; names[cc] && snames[cc]; cc++)
-    {
-    QList<QVariant> p =
-      pqSMAdaptor::getMultipleElementProperty(ren->GetProperty(snames[cc]));
-    pqSMAdaptor::setMultipleElementProperty(dest->GetProperty(names[cc]),p);
-    }
-}
-  
+//-----------------------------------------------------------------------------
 void pqAnimationViewWidget::createTrack()
 {
-  pqUndoStack* undo = pqApplicationCore::instance()->getUndoStack();
-  if(undo)
-    {
-    undo->beginUndoSet("Add Animation Track");
-    }
-
   vtkSMRenderViewProxy* ren =
     vtkSMRenderViewProxy::SafeDownCast(this->Internal->CreateSource->getCurrentProxy());
   // Need to create new cue for this property.
   vtkSMProxy* curProxy = this->Internal->CreateProperty->getCurrentProxy();
-  if(ren)
-    {
-    curProxy = ren;
-    }
   QString pname = this->Internal->CreateProperty->getCurrentPropertyName();
   int pindex = this->Internal->CreateProperty->getCurrentIndex();
 
+  // used for camera tracks.
+  QString mode = this->Internal->CreateProperty->getCurrentPropertyName();
+
+  if (ren)
+    {
+    curProxy = ren;
+    pname = QString();
+    pindex = 0;
+    }
+
+  if(!curProxy)
+    {
+    // curProxy == NULL is only used for "Python" track for now. Of course,
+    // we only support that when python is enabled.
+    // we allow creating as many python tracks as needed, hence we don't check
+    // if there exists a track already (which is the case with others).
+#ifdef PARAVIEW_ENABLE_PYTHON
+    this->createPythonTrack();
+#endif
+    return;
+    }
+
   // check that we don't already have one
   foreach(pqAnimationCue* cue, this->Internal->TrackMap.keys())
     {
+    if (cue->getAnimatedProxy() == NULL)
+      {
+      continue; // skip Python tracks.
+      }
     if(cue->getAnimatedProxy() == curProxy &&
        cue->getAnimatedProxy()->GetPropertyName(cue->getAnimatedProperty()) == pname &&
        cue->getAnimatedPropertyIndex() == pindex)
@@ -787,90 +887,74 @@ void pqAnimationViewWidget::createTrack()
       }
     }
   
-  if(!curProxy)
-    {
-    // hmm something went wrong
-    return;
-    }
-  
-  
-  // Check is we are creating a camera cue or a regular cue.
-  if(ren)
-    {
-    pqAnimationCue* cue = this->Internal->Scene->createCue(curProxy,
-      pname.toAscii().data(), pindex, "CameraAnimationCue");
-    cue->setKeyFrameType("CameraKeyFrame");
-    vtkSMProxy* kf = cue->insertKeyFrame(0);
-    pqAnimationPanelResetCameraKeyFrameToCurrent(ren, kf);
-    kf->UpdateVTKObjects();
-    kf = cue->insertKeyFrame(1);
-    pqAnimationPanelResetCameraKeyFrameToCurrent(ren, kf);
-    kf->UpdateVTKObjects();
-    }
-  else
-    {
-    pqAnimationCue* cue = this->Internal->Scene->createCue(curProxy, 
-      pname.toAscii().data(), pindex, "KeyFrameAnimationCue");
-    vtkSMProxy* kf1 = cue->insertKeyFrame(0);
-    vtkSMProxy* kf2 = cue->insertKeyFrame(1);
+  pqOrbitCreatorDialog creator(this);
 
-    vtkSMProperty* prop = curProxy->GetProperty(pname.toAscii().data());
-    QList<QVariant> mins;
-    QList<QVariant> maxs;
-    if(pindex == -1 && prop)
+  // if mode=="orbit" show up a dialog allowing the user to customize the
+  // orbit.
+  if (ren && mode == "orbit")
+    {
+    creator.setNormal( ren->GetActiveCamera()->GetViewUp() );
+    creator.setOrigin( ren->GetActiveCamera()->GetPosition() );
+    if (creator.exec() != QDialog::Accepted)
       {
-      QList<QList<QVariant> > domains =
-        pqSMAdaptor::getMultipleElementPropertyDomain(prop);
-      QList<QVariant> currents = pqSMAdaptor::getMultipleElementProperty(prop);
-      for(int i=0; i<currents.size(); i++)
-        {
-        if(domains.size() > i && domains[i].size())
-          {
-          mins.append(domains[i][0].isValid() ? domains[i][0] : currents[i]);
-          maxs.append(domains[i][1].isValid() ? domains[i][1] : currents[i]);
-          }
-        else
-          {
-          mins.append(currents[i]);
-          maxs.append(currents[i]);
-          }
-        }
+      return;
+      }
+    }
+
+  BEGIN_UNDO_SET("Add Animation Track");
+
+  // This will create the cue and initialize it with default keyframes.
+  pqAnimationCue* cue = this->Internal->Scene->createCue(curProxy,
+    pname.toAscii().data(), pindex, ren? "CameraAnimationCue" :
+    "KeyFrameAnimationCue");
+
+  if (ren)
+    {
+    if (mode=="path" || mode =="orbit")
+      {
+      // Setup default animation to revolve around the selected objects (if any)
+      // in a plane normal to the current view-up vector.
+      pqSMAdaptor::setElementProperty(
+        cue->getProxy()->GetProperty("Mode"), 1); // PATH-based animation.
       }
     else
       {
-      QList<QVariant> domain =
-        pqSMAdaptor::getMultipleElementPropertyDomain(prop, pindex);
-      QVariant current = pqSMAdaptor::getMultipleElementProperty(prop, pindex);
-      if(domain.size() && domain[0].isValid())
-        {
-        mins.append(domain[0]);
-        }
-      else
-        {
-        mins.append(current);
-        }
-      if(domain.size() && domain[1].isValid())
-        {
-        maxs.append(domain[1]);
-        }
-      else
-        {
-        maxs.append(current);
-        }
+      pqSMAdaptor::setElementProperty(
+        cue->getProxy()->GetProperty("Mode"), 0); // non-PATH-based animation.
       }
+    cue->getProxy()->UpdateVTKObjects();
 
-    pqSMAdaptor::setMultipleElementProperty(
-      kf1->GetProperty("KeyValues"), mins);
-    pqSMAdaptor::setMultipleElementProperty(
-      kf2->GetProperty("KeyValues"), maxs);
+    if (mode == "orbit")
+      {
+      // update key frame paramters based on the orbit points.
+      vtkSMProxy* kf = cue->getKeyFrame(0);
+      pqSMAdaptor::setMultipleElementProperty(
+        kf->GetProperty("PositionPathPoints"),
+        creator.orbitPoints(7));
+      pqSMAdaptor::setMultipleElementProperty(
+        kf->GetProperty("FocalPathPoints"),
+        creator.center());
+      pqSMAdaptor::setElementProperty(
+        kf->GetProperty("ClosedPositionPath"), 1);
+      kf->UpdateVTKObjects();
+      }
+    }
 
-    kf1->UpdateVTKObjects();
-    kf2->UpdateVTKObjects();
-    }
-  
-  if(undo)
-    {
-    undo->endUndoSet();
-    }
+  END_UNDO_SET();
 }
 
+//-----------------------------------------------------------------------------
+void pqAnimationViewWidget::createPythonTrack()
+{
+#ifdef PARAVIEW_ENABLE_PYTHON
+  BEGIN_UNDO_SET("Add Animation Track");
+
+  pqAnimationCue* cue = this->Internal->Scene->createCue("PythonAnimationCue");
+  Q_ASSERT(cue != NULL);
+  (void)cue;
+  END_UNDO_SET();
+#else
+  qCritical() << "Python support not enabled. Please recompile ParaView "
+    "with Python enabled.";
+#endif
+}
