@@ -36,6 +36,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqUndoStack.h"
 #include "vtkEventQtSlotConnect.h"
+#include "vtkPVComparativeAnimationCue.h"
 #include "vtkSMComparativeAnimationCueProxy.h"
 #include "vtkSMPropertyHelper.h"
 
@@ -127,7 +128,7 @@ vtkSMComparativeAnimationCueProxy* pqComparativeCueWidget::cue() const
 //-----------------------------------------------------------------------------
 bool pqComparativeCueWidget::acceptsMultipleValues() const
 {
-  return (this->Cue && this->Cue->GetAnimatedElement() == -1);
+  return (this->Cue && this->Cue->GetCue()->GetAnimatedElement() == -1);
 }
 
 //-----------------------------------------------------------------------------
@@ -158,11 +159,12 @@ void pqComparativeCueWidget::updateGUI()
     }
   this->setHorizontalHeaderLabels(hlabels);
 
-  vtkSMComparativeAnimationCueProxy* acue = this->cue();
-  if (!acue)
+  vtkSMComparativeAnimationCueProxy* acueProxy = this->cue();
+  if (!acueProxy)
     {
     return;
     }
+  vtkPVComparativeAnimationCue* acue = acueProxy->GetCue();
 
   for (int colno=0; colno < cols; colno++)
     {
@@ -211,14 +213,14 @@ void pqComparativeCueWidget::onCellChanged(int rowno, int colno)
         *ptr = QVariant(part).toDouble();
         ptr++;
         }
-      this->cue()->UpdateValue(colno, rowno, newvalues,
+      this->cue()->GetCue()->UpdateValue(colno, rowno, newvalues,
         static_cast<unsigned int>(parts.size()));
       }
     }
   else
     {
     double item_data = QVariant(text).toDouble();
-    this->cue()->UpdateValue(colno, rowno, item_data);
+    this->cue()->GetCue()->UpdateValue(colno, rowno, item_data);
     }
   END_UNDO_SET();
 
@@ -283,18 +285,20 @@ void pqComparativeCueWidget::editRange()
 
   BEGIN_UNDO_SET("Update Parameter Values");
 
+  vtkPVComparativeAnimationCue* acue = this->cue()->GetCue();
+
   if (range.rowCount() == 1 &&
     range.columnCount() == this->size().width())
     {
     // user set an x-range.
-    this->cue()->UpdateXRange(range.topRow(), &minvalues[0], &maxvalues[0],
+    acue->UpdateXRange(range.topRow(), &minvalues[0], &maxvalues[0],
       numvalues);
     }
   else if (range.columnCount() == 1 &&
     range.rowCount() == this->size().height())
     {
     // user set a y-range.
-    this->cue()->UpdateYRange(range.leftColumn(), 
+    acue->UpdateYRange(range.leftColumn(),
       &minvalues[0], &maxvalues[0], numvalues);
     }
   else if (range.columnCount() == this->size().width() &&
@@ -305,20 +309,20 @@ void pqComparativeCueWidget::editRange()
       {
     case HORZ_FIRST:
       // user set a t-range.
-      this->cue()->UpdateWholeRange(&minvalues[0], &maxvalues[0], numvalues);
+      acue->UpdateWholeRange(&minvalues[0], &maxvalues[0], numvalues);
       break;
 
     case VERT_FIRST:
-      this->cue()->UpdateWholeRange(&minvalues[0], &maxvalues[0], numvalues,
+      acue->UpdateWholeRange(&minvalues[0], &maxvalues[0], numvalues,
         true);
       break;
 
     case HORZ_ONLY:
-      this->cue()->UpdateXRange(-1, &minvalues[0], &maxvalues[0], numvalues);
+      acue->UpdateXRange(-1, &minvalues[0], &maxvalues[0], numvalues);
       break;
 
     case VERT_ONLY:
-      this->cue()->UpdateYRange(-1, &minvalues[0], &maxvalues[0], numvalues);
+      acue->UpdateYRange(-1, &minvalues[0], &maxvalues[0], numvalues);
       break;
 
     default:
@@ -362,7 +366,7 @@ void pqComparativeCueWidget::editRange()
             }
           newvalues[cc] = minvalues[cc] + scale_factor * (maxvalues[cc] - minvalues[cc]);
           }
-        this->cue()->UpdateValue(xx, yy, &newvalues[0], numvalues);
+        acue->UpdateValue(xx, yy, &newvalues[0], numvalues);
         }
       }
     }
