@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqMultiViewFrame.cxx,v $
+   Module:    pqMultiViewFrame.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -45,7 +45,7 @@ static int gPenWidth = 2;
 
 pqMultiViewFrame::pqMultiViewFrame(QWidget* p)
   : QWidget(p), EmptyMainWidget(new QWidget(this)), AutoHide(false), Active(false), 
-    Color(QColor("red"))
+    Color(QColor("blue"))
 {
   QVBoxLayout* boxLayout = new QVBoxLayout(this);
   boxLayout->setMargin(gPenWidth);
