@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqObjectPanel.cxx,v $
+   Module:    pqObjectPanel.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -53,16 +53,21 @@ pqProxy* pqObjectPanel::referenceProxy() const
   return this->ReferenceProxy;
 }
 
+//-----------------------------------------------------------------------------
 void pqObjectPanel::accept()
 {
   pqProxyPanel::accept();
   this->ReferenceProxy->setModifiedState(pqProxy::UNMODIFIED);
 }
 
+//-----------------------------------------------------------------------------
 void pqObjectPanel::reset()
 {
   pqProxyPanel::reset();
-  this->ReferenceProxy->setModifiedState(pqProxy::UNMODIFIED);
+  if (this->ReferenceProxy->modifiedState() != pqProxy::UNINITIALIZED)
+    {
+    this->ReferenceProxy->setModifiedState(pqProxy::UNMODIFIED);
+    }
 }
 
 //-----------------------------------------------------------------------------
