@@ -33,9 +33,11 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ui_pqQueryDialog.h"
 
 #include "pqActiveObjects.h"
+#include "pqActiveView.h"
 #include "pqApplicationCore.h"
 #include "pqDataRepresentation.h"
 #include "pqOutputPort.h"
+#include "pqOutputPortComboBox.h"
 #include "pqPipelineSource.h"
 #include "pqPropertyLinks.h"
 #include "pqQueryClauseWidget.h"
@@ -47,11 +49,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkPVArrayInformation.h"
 #include "vtkPVDataInformation.h"
 #include "vtkPVDataSetAttributesInformation.h"
+#include "vtkSmartPointer.h"
 #include "vtkSMGlobalPropertiesManager.h"
 #include "vtkSMPropertyHelper.h"
+#include "vtkSMProperty.h"
+#include "vtkSMStringVectorProperty.h"
 #include "vtkSMProxyManager.h"
 #include "vtkSMSourceProxy.h"
-#include "vtkSMSpreadSheetRepresentationProxy.h"
+#include "vtkSMViewProxy.h"
 
 #include <QList>
 
@@ -59,11 +64,18 @@ class pqQueryDialog::pqInternals : public Ui::pqQueryDialog
 {
 public:
   QList<pqQueryClauseWidget*> Clauses;
-  pqSpreadSheetViewModel DataModel;
+  pqSpreadSheetViewModel* DataModel;
   pqPropertyLinks Links;
-  
+  vtkSmartPointer<vtkSMViewProxy> ViewProxy;
+  vtkSmartPointer<vtkSMProxy> RepresentationProxy;
+
   pqPropertyLinks LabelColorLinks;
   pqSignalAdaptorColor* LabelColorAdaptor;
+  pqInternals()
+    {
+    this->DataModel = NULL;
+    this->LabelColorAdaptor = NULL;
+    }
 };
 
 //-----------------------------------------------------------------------------
@@ -72,31 +84,19 @@ pqQueryDialog::pqQueryDialog(
   QWidget* parentObject, Qt::WindowFlags _flags)
   : Superclass(parentObject, _flags)
 {
-  this->ExtractSelection = false;
-  this->ExtractSelectionOverTime = false;
-
   this->Internals = new pqInternals();
   this->Internals->setupUi(this);
 
-  this->Producer = _producer;
-  // Producer cannot be NULL.
-  Q_ASSERT(_producer != NULL);
-
-  pqPipelineSource* source = _producer->getSource();
-  QString fromLabel;
-  if (source->getNumberOfOutputPorts() > 1)
-    {
-    fromLabel = QString("from <b>%1 (%2)</b>").arg(source->getSMName()).arg(
-      _producer->getPortName());
-    }
-  else
-    {
-    fromLabel = QString("from <b>%1</b>").arg(source->getSMName());
-    }
-  this->Internals->labelFromSource->setText(fromLabel);
+  this->Producer = NULL;
 
   // Update the GUI
-  this->populateSelectionType();
+  this->Internals->source->setAutoUpdateIndex(false);
+  this->Internals->source->fillExistingPorts();
+  if(_producer != NULL)
+    {
+    this->Internals->source->setCurrentPort(_producer);
+    this->populateSelectionType();
+    }
 
   // Ensure that there's only 1 clause
   this->resetClauses();
@@ -115,25 +115,7 @@ pqQueryDialog::pqQueryDialog(
     SIGNAL(clicked()), this, SLOT(runQuery()));
 
   // Setup the spreadsheet view.
-  this->Internals->spreadsheet->setModel(&this->Internals->DataModel);
-
-  // Create representation to show the producer.
-  vtkSMSpreadSheetRepresentationProxy* repr = 
-    vtkSMSpreadSheetRepresentationProxy::SafeDownCast(
-    vtkSMProxyManager::GetProxyManager()->NewProxy("representations",
-      "SpreadSheetRepresentation"));
-  repr->SetConnectionID(_producer->getServer()->GetConnectionID());
-  vtkSMPropertyHelper(repr, "Input").Set(
-    _producer->getSource()->getProxy(), _producer->getPortNumber());
-
-  // we always want to show all the blocks in the dataset, since we don't have a
-  // block chooser widget in this dialog.
-  vtkSMPropertyHelper(repr, "CompositeDataSetIndex").Set(0);
-  vtkSMPropertyHelper(repr, "SelectionOnly").Set(1);
-  repr->UpdateVTKObjects();
-  this->Internals->DataModel.setRepresentationProxy(repr);
-  repr->Delete();
-
+  this->Internals->spreadsheet->setModel(NULL);
 
   // Link the selection color to the global selection color so that it will
   // affect all views, otherwise user may be get confused ;).
@@ -163,78 +145,92 @@ pqQueryDialog::pqQueryDialog(
   QObject::connect(this->Internals->extractSelectionOverTime,
     SIGNAL(clicked()), this, SLOT(onExtractSelectionOverTime()));
 
-  // Render all views when any selection property is modified.
-  QObject::connect(
-    &this->Internals->Links, SIGNAL(qtWidgetChanged()),
-    _producer, SLOT(renderAllViews()));
-  QObject::connect(
-    &this->Internals->LabelColorLinks, SIGNAL(qtWidgetChanged()),
-    _producer, SLOT(renderAllViews()));
+  // Make sure that when the input port selection change we are aware of that
+  QObject::connect(this->Internals->source,
+                   SIGNAL(currentIndexChanged(pqOutputPort*)),
+                   this,
+                   SLOT(onSelectionChange(pqOutputPort*)));
 
-  vtkPVDataInformation* dataInfo = this->producer()->getDataInformation();
-  if (dataInfo->GetTimeSpan()[0] >= dataInfo->GetTimeSpan()[1])
-    {
-    // don't show the extract selection over time option is there's not time!
-    this->Internals->extractSelectionOverTime->hide();
-    }
+  // Connect the view manager to the pqActiveView.
+  QObject::connect(&pqActiveView::instance(),
+    SIGNAL(changed(pqView*)),
+    this, SLOT(onActiveViewChanged(pqView*)));
 
-  QApplication::instance()->installEventFilter(this);
+  this->onSelectionChange(_producer);
 }
 
 //-----------------------------------------------------------------------------
 pqQueryDialog::~pqQueryDialog()
 {
-  QApplication::instance()->removeEventFilter(this);
-  delete this->Internals;
+  if(this->Internals)
+    {
+    this->freeSMProxy();
+    delete this->Internals;
+    }
   this->Internals = 0;
 }
 
+//-----------------------------------------------------------------------------
+void pqQueryDialog::setupSpreadSheet()
+{
+  this->Internals->spreadsheet->setModel(NULL);
+  if(this->Internals->source->currentPort() == NULL ||
+     this->Internals->source->currentPort()->getSource()->getProxy()->GetObjectsCreated() != 1)
+    {
+    return;
+    }
+  vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
+
+  vtkSMProxy* repr = pxm->NewProxy("representations", "SpreadSheetRepresentation");
+  // we always want to show all the blocks in the dataset, since we don't have a
+  // block chooser widget in this dialog.
+  vtkSMPropertyHelper(repr, "CompositeDataSetIndex").Set(0);
+  vtkSMPropertyHelper(repr, "Input").Set(
+      this->Internals->source->currentPort()->getSource()->getProxy(),
+      this->Internals->source->currentPort()->getPortNumber());
+  repr->UpdateVTKObjects();
+
+  vtkSMViewProxy* view = vtkSMViewProxy::SafeDownCast(
+    pxm->NewProxy("views", "SpreadSheetView"));
+  vtkSMPropertyHelper(view, "SelectionOnly").Set(1);
+  vtkSMPropertyHelper(view, "Representations").Set(repr);
+  vtkSMPropertyHelper(view, "ViewSize").Set(0, 1);
+  vtkSMPropertyHelper(view, "ViewSize").Set(1, 1);
+  view->UpdateVTKObjects();
+  view->StillRender();;
+
+  this->Internals->ViewProxy.TakeReference(view);
+  this->Internals->RepresentationProxy.TakeReference(repr);
+  this->Internals->DataModel = new pqSpreadSheetViewModel(view, this);
+  this->Internals->DataModel->setActiveRepresentationProxy(repr);
+  // We deliberately don't set the model on the view here. We do that
+  // after all updates have happened. This keeps any progress events from
+  // mixing with the paint events and causing random test failures on
+  // dashboards (FindDataDialog test).
+}
+
 #include <QInputEvent>
 #include <pqCoreUtilities.h>
 #include "QVTKWidget.h"
-//-----------------------------------------------------------------------------
-bool pqQueryDialog::eventFilter(QObject* obj, QEvent* evt)
-{
-  QWidget* wdg = qobject_cast<QWidget*>(obj);
-  bool is_input_event = (dynamic_cast<QInputEvent*>(evt) != NULL);
-  bool is_meant_for_dialog =
-    (wdg == NULL || this->isAncestorOf(wdg) || this == wdg);
-  bool is_render_window = (qobject_cast<QVTKWidget*>(obj));
-  bool is_magical_event =
-    (is_meant_for_dialog || !pqCoreUtilities::mainWidget()->isAncestorOf(wdg));
-  
-  if (is_input_event && !is_meant_for_dialog && !is_magical_event &&
-    !is_render_window)
-    {
-    return true;
-    }
-      
-  return this->Superclass::eventFilter(obj, evt);
-}
-
 //-----------------------------------------------------------------------------
 void pqQueryDialog::populateSelectionType()
 {
   this->Internals->selectionType->clear();
-  vtkPVDataInformation* dataInfo = this->producer()->getDataInformation();
+  vtkPVDataInformation* dataInfo =
+      this->Internals->source->currentPort()->getDataInformation();
   if (dataInfo->DataSetTypeIsA("vtkGraph"))
     {
-    this->Internals->selectionType->addItem("Vertex",
-      vtkDataObject::VERTEX);
-    this->Internals->selectionType->addItem("Edge",
-      vtkDataObject::EDGE);
+    this->Internals->selectionType->addItem("Vertex", vtkDataObject::VERTEX);
+    this->Internals->selectionType->addItem("Edge",   vtkDataObject::EDGE);
     }
   else if (dataInfo->DataSetTypeIsA("vtkTable"))
     {
-    this->Internals->selectionType->addItem("Row",
-      vtkDataObject::ROW);
+    this->Internals->selectionType->addItem("Row", vtkDataObject::ROW);
     }
   else
     {
-    this->Internals->selectionType->addItem("Cell",
-      vtkDataObject::CELL);
-    this->Internals->selectionType->addItem("Point",
-      vtkDataObject::POINT);
+    this->Internals->selectionType->addItem("Cell",  vtkDataObject::CELL);
+    this->Internals->selectionType->addItem("Point", vtkDataObject::POINT);
     }
 }
 
@@ -257,9 +253,14 @@ void pqQueryDialog::resetClauses()
 //-----------------------------------------------------------------------------
 void pqQueryDialog::addClause()
 {
+  if(this->Internals->source->currentPort() == NULL ||
+     this->Internals->source->currentPort()->getSource()->getProxy()->GetObjectsCreated() != 1)
+    {
+    return; // We can not create a Clause on nothing...
+    }
+
   pqQueryClauseWidget* clause = new pqQueryClauseWidget(this);
-  QObject::connect(clause, SIGNAL(removeClause()),
-    this, SLOT(removeClause()));
+  QObject::connect(clause, SIGNAL(removeClause()), this, SLOT(removeClause()));
   if (this->Internals->Clauses.size() == 0)
     {
     // don't allow removal of the first clause.
@@ -268,7 +269,7 @@ void pqQueryDialog::addClause()
 
   int attr_type = this->Internals->selectionType->itemData(
     this->Internals->selectionType->currentIndex()).toInt();
-  clause->setProducer(this->producer());
+  clause->setProducer(this->Internals->source->currentPort());
   clause->setAttributeType(attr_type);
   clause->initialize();
 
@@ -282,8 +283,8 @@ void pqQueryDialog::addClause()
 //-----------------------------------------------------------------------------
 void pqQueryDialog::removeClause()
 {
-  pqQueryClauseWidget* clause = qobject_cast<pqQueryClauseWidget*>(
-    this->sender());
+  pqQueryClauseWidget* clause =
+      qobject_cast<pqQueryClauseWidget*>(this->sender());
   if (clause)
     {
     this->Internals->Clauses.removeAll(clause);
@@ -304,24 +305,36 @@ void pqQueryDialog::runQuery()
     {
     return;
     }
-
   selSource->UpdateVTKObjects();
-  this->producer()->setSelectionInput(
-    vtkSMSourceProxy::SafeDownCast(selSource), 0);
+
+  this->setupSpreadSheet();
+
+  // setupSpreadSheet already does this, but I am doing this again to be
+  // extra sure :).
+  this->Internals->spreadsheet->setModel(NULL);
+
+  this->Internals->source->currentPort()->setSelectionInput(
+      vtkSMSourceProxy::SafeDownCast(selSource), 0);
   selSource->Delete();
 
-  this->producer()->renderAllViews();
+  this->Internals->source->currentPort()->renderAllViews();
 
   int attr_type = this->Internals->selectionType->itemData(
     this->Internals->selectionType->currentIndex()).toInt();
-  vtkSMSpreadSheetRepresentationProxy* repr =
-    this->Internals->DataModel.getRepresentationProxy();
+
+  vtkSMProxy* repr = this->Internals->RepresentationProxy;
   // Pass the chosen attribute type to the spreasheet so we show cells or
   // points etc. based on what was selected.
   vtkSMPropertyHelper(repr, "FieldAssociation").Set(attr_type);
   repr->UpdateVTKObjects();
-  this->Internals->DataModel.getRepresentationProxy()->Update();
-  this->Internals->DataModel.forceUpdate();
+
+  // this may cause progress events. These can result in paint-event for the table
+  // view which could prematurely start querying information from the model.
+  // To avoid that issue, we don't set the model on  the view until after this call.
+  this->Internals->ViewProxy->StillRender();
+
+  // now set the model on the view.
+  this->Internals->spreadsheet->setModel(this->Internals->DataModel);
 
   // Once a query has been made, we enable components of the GUI that use the
   // selection
@@ -329,11 +342,10 @@ void pqQueryDialog::runQuery()
   this->Internals->labels->setEnabled(true);
   this->Internals->extractSelection->setEnabled(true);
   this->Internals->extractSelectionOverTime->setEnabled(true);
-  
+
   // update the list of available labels.
   this->updateLabels();
-
-  emit this->selected(this->producer());
+  emit this->selected(this->Internals->source->currentPort());
 }
 
 //-----------------------------------------------------------------------------
@@ -387,7 +399,7 @@ void pqQueryDialog::updateLabels()
   QIcon cellDataIcon(":/pqWidgets/Icons/pqCellData16.png");
   QIcon pointDataIcon(":/pqWidgets/Icons/pqPointData16.png");
 
-  vtkPVDataInformation* dataInfo = this->producer()->getDataInformation();
+  vtkPVDataInformation* dataInfo = this->Internals->source->currentPort()->getDataInformation();
 
   // Only adding cells and points for now since our labelling code doesn't
   // support vertex or edge labels.
@@ -429,8 +441,9 @@ void pqQueryDialog::setLabel(int index)
   // disabled when not labelling.
   this->Internals->labelColor->setEnabled(index > 0); 
 
-  pqDataRepresentation* repr = this->producer()->getRepresentation(
-    pqActiveObjects::instance().activeView());
+  pqDataRepresentation* repr =
+      this->Internals->source->currentPort()->getRepresentation(
+          pqActiveObjects::instance().activeView());
   if (!repr)
     {
     return;
@@ -458,7 +471,7 @@ void pqQueryDialog::setLabel(int index)
       }
     // based on whether cell or point labels are selected, we need to link the
     // "Label Color" widget with the right property.
-    this->linkLabelColorWidet(reprProxy, "SelectionPointLabelColor");
+    this->linkLabelColorWidget(reprProxy, "SelectionPointLabelColor");
     }
   else if (item_data == -3  || item_data == vtkDataObject::CELL)
     {
@@ -477,7 +490,7 @@ void pqQueryDialog::setLabel(int index)
       }
     // based on whether cell or point labels are selected, we need to link the
     // "Label Color" widget with the right property.
-    this->linkLabelColorWidet(reprProxy, "SelectionCellLabelColor");
+    this->linkLabelColorWidget(reprProxy, "SelectionCellLabelColor");
     }
   else
     {
@@ -487,18 +500,164 @@ void pqQueryDialog::setLabel(int index)
 
   reprProxy->UpdateVTKObjects();
   END_UNDO_SET();
-  this->producer()->renderAllViews();
+  this->Internals->source->currentPort()->renderAllViews();
 }
 
 //-----------------------------------------------------------------------------
-void pqQueryDialog::linkLabelColorWidet(vtkSMProxy* proxy, 
-  const QString& propname)
+void pqQueryDialog::linkLabelColorWidget( vtkSMProxy* proxy,
+                                         const QString& propname)
 {
   this->Internals->LabelColorLinks.removeAllPropertyLinks();
   this->Internals->LabelColorLinks.addPropertyLink(
-    this->Internals->LabelColorAdaptor,
-    "color", SIGNAL(colorChanged(const QVariant&)),
-    proxy,
-    proxy->GetProperty(propname.toAscii().data()));
+      this->Internals->LabelColorAdaptor,
+      "color",
+      SIGNAL(colorChanged(const QVariant&)),
+      proxy,
+      proxy->GetProperty(propname.toAscii().data()));
 }
 
+//-----------------------------------------------------------------------------
+void pqQueryDialog::onSelectionChange(pqOutputPort* newSelectedPort)
+{
+
+  // Reset the spreadsheet view
+  this->resetClauses();
+  this->freeSMProxy();
+
+  if(this->Producer != NULL)
+    {
+    // Disconnect previous render
+    QObject::disconnect( &this->Internals->Links, SIGNAL(qtWidgetChanged()),
+                         this->Producer, SLOT(renderAllViews()));
+    QObject::disconnect( &this->Internals->LabelColorLinks, SIGNAL(qtWidgetChanged()),
+                         this->Producer, SLOT(renderAllViews()));
+    }
+
+  this->Producer = newSelectedPort;
+
+  if(this->Producer != NULL)
+    {
+    // Render all views when any selection property is modified.
+    QObject::connect( &this->Internals->Links, SIGNAL(qtWidgetChanged()),
+                      this->Producer, SLOT(renderAllViews()));
+    QObject::connect( &this->Internals->LabelColorLinks, SIGNAL(qtWidgetChanged()),
+                      this->Producer, SLOT(renderAllViews()));
+
+    vtkPVDataInformation* dataInfo = this->Internals->source->currentPort()->getDataInformation();
+    if (dataInfo->GetTimeSpan()[0] >= dataInfo->GetTimeSpan()[1])
+      {
+      // don't show the extract selection over time option is there's not time!
+      this->Internals->extractSelectionOverTime->hide();
+      }
+    else
+      {
+      this->Internals->extractSelectionOverTime->show();
+      }
+    this->updateLabels();
+    }
+  else
+    {
+    // As no more datasource is available make sure that we free the ressources
+    this->freeSMProxy();
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqQueryDialog::onActiveViewChanged(pqView* view)
+{
+  if(this->Internals->source->currentPort() == NULL)
+    {
+    // We are in the disconnect case...
+    return;
+    }
+  if(view == NULL)
+    {
+    this->Internals->labels->blockSignals(true);
+    this->Internals->labels->setCurrentIndex(0);
+    this->Internals->labels->blockSignals(false);
+    this->Internals->labelColor->setEnabled(false);
+    }
+
+  pqDataRepresentation* repr =
+      this->Internals->source->currentPort()->getRepresentation(
+          pqActiveObjects::instance().activeView());
+  if (!repr)
+    {
+    return;
+    }
+  vtkSMProxy* reprProxy = repr->getProxy();
+
+  // Value container
+  int pointLabel;
+  double pointColor[3];
+  const char* pointArrayName;
+  int cellLabel;
+  double cellColor[3];
+  const char* cellArrayName;
+
+  // Get point infos
+  vtkSMPropertyHelper(reprProxy, "SelectionPointLabelVisibility", true).Get(&pointLabel, 1);
+  vtkSMPropertyHelper(reprProxy, "SelectionPointLabelColor", true).Get(pointColor, 3);
+  pointArrayName = vtkSMStringVectorProperty::SafeDownCast(
+      reprProxy->GetProperty("SelectionPointFieldDataArrayName"))->GetElement(0);
+
+  // Get cell infos
+  vtkSMPropertyHelper(reprProxy, "SelectionCellLabelVisibility", true).Get(&cellLabel, 1);
+  vtkSMPropertyHelper(reprProxy, "SelectionCellLabelColor", true).Get(cellColor, 3);
+  cellArrayName = vtkSMStringVectorProperty::SafeDownCast(
+      reprProxy->GetProperty("SelectionCellFieldDataArrayName"))->GetElement(0);
+
+  // Set those values to the UI
+  int new_index = 0;
+  if(pointLabel == 1)
+    {
+    new_index = this->Internals->labels->findText(pointArrayName);
+    this->Internals->labelColor->blockSignals(true);
+    this->Internals->labelColor->setChosenColor(
+        QColor( static_cast<int>(pointColor[0]*255.0),
+                static_cast<int>(pointColor[1]*255.0),
+                static_cast<int>(pointColor[2]*255.0), 255 ));
+    this->Internals->labelColor->repaint();
+    this->Internals->labelColor->blockSignals(false);
+    if(new_index == -1 && strcmp(pointArrayName, "vtkOriginalPointIds") == 0)
+      {
+      new_index = this->Internals->labels->findText("Point ID");
+      }
+    this->linkLabelColorWidget(reprProxy, "SelectionPointLabelColor");
+    }
+  else if(cellLabel == 1)
+    {
+    new_index = this->Internals->labels->findText(cellArrayName);
+    this->Internals->labelColor->blockSignals(true);
+    this->Internals->labelColor->setChosenColor(
+        QColor( static_cast<int>(cellColor[0]*255.0),
+                static_cast<int>(cellColor[1]*255.0),
+                static_cast<int>(cellColor[2]*255.0), 255 ));
+    this->Internals->labelColor->repaint();
+    this->Internals->labelColor->blockSignals(false);
+    if(new_index == -1 && strcmp(cellArrayName, "vtkOriginalCellIds") == 0)
+      {
+      new_index = this->Internals->labels->findText("Cell ID");
+      }
+    this->linkLabelColorWidget(reprProxy, "SelectionCellLabelColor");
+    }
+
+  // Update combobox
+  if (new_index != -1 && this->Internals->labels->currentIndex() != new_index)
+    {
+    this->Internals->labels->blockSignals(true);
+    this->Internals->labels->setCurrentIndex(new_index);
+    this->Internals->labels->blockSignals(false);
+    this->Internals->labelColor->setEnabled(new_index > 0);
+    }
+}
+//-----------------------------------------------------------------------------
+void pqQueryDialog::freeSMProxy()
+{
+  this->Internals->DataModel = NULL;
+  this->Internals->Links.removeAllPropertyLinks();
+  this->Internals->LabelColorLinks.removeAllPropertyLinks();
+  this->Internals->ViewProxy = NULL;
+  this->Internals->RepresentationProxy = NULL;
+  this->Internals->spreadsheet->setModel(NULL);
+}
