@@ -103,7 +103,9 @@ protected slots:
   void saveState(vtkPVXMLElement* root);
 
   /// Loads the state for the view window manager.
-  bool loadState(vtkPVXMLElement* rwRoot, vtkSMProxyLocator* loader);
+  bool loadState(vtkPVXMLElement* rwRoot, vtkSMProxyLocator* locator);
+  void loadState(vtkPVXMLElement* rwRoot)
+    { this->Superclass::loadState(rwRoot); }
 
   /// Called when server disconnects, we reset the view layout.
   void onServerDisconnect();
@@ -186,11 +188,6 @@ protected:
   void connect(pqMultiViewFrame* frame, pqView* view);
   void disconnect(pqMultiViewFrame* frame, pqView* view);
 
-  /// Hiding superclasses loadState. Don't use this API
-  /// since it is not aware of the loader which gives us 
-  /// the render modules to put in the window.
-  virtual void loadState(vtkPVXMLElement* /*root*/) { }
-
   /// Updates the context menu.
   void updateConversionActions(pqMultiViewFrame* frame);
 
