@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqAnimationSceneImageWriter.h,v $
+   Module:    pqAnimationSceneImageWriter.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -41,7 +41,7 @@ class PQCORE_EXPORT pqAnimationSceneImageWriter : public vtkSMAnimationSceneImag
 {
 public:
   static pqAnimationSceneImageWriter* New();
-  vtkTypeRevisionMacro(pqAnimationSceneImageWriter, vtkSMAnimationSceneImageWriter);
+  vtkTypeMacro(pqAnimationSceneImageWriter, vtkSMAnimationSceneImageWriter);
   void PrintSelf(ostream& os, vtkIndent indent);
 
 protected:
