@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqPropertyManager.h,v $
+   Module:    pqPropertyManager.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -79,6 +79,7 @@ signals:
   void modified();
 
   /// Signal emitted when the user has accepted changes
+  void aboutToAccept();
   void accepted();
   /// Signal emitted when the user has rejected changes
   void rejected();
