@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqQVTKWidgetEventPlayer.cxx,v $
+   Module:    pqQVTKWidgetEventPlayer.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -72,7 +72,6 @@ bool pqQVTKWidgetEventPlayer::playEvent(QObject* Object,
           ((Command=="mouseMove")?  QEvent::MouseMove : QEvent::MouseButtonRelease);
         QMouseEvent e(type, QPoint(x,y), button, buttons, keym);
         QCoreApplication::sendEvent(widget, &e);
-        pqEventDispatcher::processEventsAndWait(1);
         }
       return true;
       }
