@@ -434,8 +434,7 @@ void pqScatterPlotRepresentation::colorByArray(const char* array)
   
   if (current_scalar_bar_visibility && lut_mgr && this->getLookupTable())
     {
-    lut_mgr->setScalarBarVisibility(this->getView(),
-      this->getLookupTable(),
+    lut_mgr->setScalarBarVisibility(this,
       current_scalar_bar_visibility);
     }
 }
