@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqServerManagerModel.cxx,v $
+   Module:    pqServerManagerModel.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -31,11 +31,23 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/
 #include "pqServerManagerModel.h"
 
-// Server Manager Includes.
+#include "pqApplicationCore.h"
+#include "pqHelperProxyStateLoader.h"
+#include "pqInterfaceTracker.h"
+#include "pqOutputPort.h"
+#include "pqPipelineSource.h"
+#include "pqProxy.h"
+#include "pqRepresentation.h"
+#include "pqServer.h"
+#include "pqServerManagerModelInterface.h"
+#include "pqServerManagerObserver.h"
+#include "pqView.h"
 #include "vtkProcessModule.h"
 #include "vtkSmartPointer.h"
-#include "vtkSMProxy.h"
+#include "vtkSMOutputPort.h"
 #include "vtkSMProxyManager.h"
+#include "vtkSMSession.h"
+#include "vtkSMSourceProxy.h"
 #include "vtkStringList.h"
 
 // Qt Includes.
@@ -44,17 +56,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QMap>
 #include <QtDebug>
 
-// ParaView Includes.
-#include "pqApplicationCore.h"
-#include "pqPluginManager.h"
-#include "pqProxy.h"
-#include "pqServer.h"
-#include "pqServerManagerModelInterface.h"
-#include "pqServerManagerObserver.h"
-#include "pqPipelineSource.h"
-#include "pqView.h"
-#include "pqRepresentation.h"
-
 //-----------------------------------------------------------------------------
 class pqServerManagerModel::pqInternal
 {
@@ -65,7 +66,12 @@ public:
   typedef QMap<vtkSMProxy*, QPointer<pqProxy> > ProxyMap;
   ProxyMap Proxies;
 
+  typedef QMap<vtkSMOutputPort*, QPointer<pqOutputPort> > OutputPortMap;
+  OutputPortMap OutputPorts;
+
   QList<QPointer<pqServerManagerModelItem> > ItemList;
+
+  pqServerResource ActiveResource;
 };
 
 //-----------------------------------------------------------------------------
@@ -86,6 +92,9 @@ pqServerManagerModel::pqServerManagerModel(
     this, SLOT(onConnectionCreated(vtkIdType)));
   QObject::connect(observer, SIGNAL(connectionClosed(vtkIdType)),
     this, SLOT(onConnectionClosed(vtkIdType)));
+  QObject::connect(observer,
+    SIGNAL(stateLoaded(vtkPVXMLElement*, vtkSMProxyLocator*)),
+    this, SLOT(onStateLoaded(vtkPVXMLElement*, vtkSMProxyLocator*)));
 }
 
 //-----------------------------------------------------------------------------
@@ -94,6 +103,26 @@ pqServerManagerModel::~pqServerManagerModel()
   delete this->Internal;
 }
 
+//-----------------------------------------------------------------------------
+void pqServerManagerModel::setActiveResource(const pqServerResource& resource)
+{
+  this->Internal->ActiveResource = resource;
+}
+
+//-----------------------------------------------------------------------------
+pqServer* pqServerManagerModel::findServer(vtkSession* session) const
+{
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+  return this->findServer(pm->GetSessionID(session));
+}
+
+//-----------------------------------------------------------------------------
+pqServer* pqServerManagerModel::findServer(vtkSMSession* session) const
+{
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+  return this->findServer(pm->GetSessionID(session));
+}
+
 //-----------------------------------------------------------------------------
 pqServer* pqServerManagerModel::findServer(vtkIdType cid) const
 {
@@ -120,7 +149,7 @@ pqServer* pqServerManagerModel::findServer(const pqServerResource& resource) con
 }
 
 //-----------------------------------------------------------------------------
-pqProxy* pqServerManagerModel::findItemHelper(
+pqServerManagerModelItem* pqServerManagerModel::findItemHelper(
   const pqServerManagerModel* const model, const QMetaObject& vtkNotUsed(mo), 
   vtkSMProxy* proxy)
 {
@@ -130,28 +159,49 @@ pqProxy* pqServerManagerModel::findItemHelper(
     return iter.value();
     }
 
-  return 0;
-}
-
-//-----------------------------------------------------------------------------
-pqProxy* pqServerManagerModel::findItemHelper(
-  const pqServerManagerModel* const model, const QMetaObject& mo, 
-  vtkClientServerID id)
-{
-  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
-  vtkSMProxy* proxy = vtkSMProxy::SafeDownCast(
-    pm->GetObjectFromID(id));
-  if (proxy)
+  vtkSMOutputPort* port = vtkSMOutputPort::SafeDownCast(proxy);
+  if (port)
     {
-    return pqServerManagerModel::findItemHelper(
-      model, mo, proxy);
+    pqPipelineSource* src =
+      model->findItem<pqPipelineSource*>(port->GetSourceProxy());
+    if (src)
+      {
+      for (int cc=0; cc < src->getNumberOfOutputPorts(); cc++)
+        {
+        pqOutputPort* pqport = src->getOutputPort(cc);
+        if (pqport && pqport->getOutputPortProxy() == port)
+          {
+          return pqport;
+          }
+        }
+      }
     }
 
   return 0;
 }
 
 //-----------------------------------------------------------------------------
-pqProxy* pqServerManagerModel::findItemHelper(
+pqServerManagerModelItem* pqServerManagerModel::findItemHelper(
+  const pqServerManagerModel* const model,
+  const QMetaObject& mo, vtkTypeUInt32 id)
+{
+  foreach (pqServerManagerModelItem* item, model->Internal->ItemList)
+    {
+    if (item && mo.cast(item))
+      {
+      pqProxy* proxy = qobject_cast<pqProxy*>(item);
+      if (proxy && proxy->getProxy()->GetGlobalID() == id)
+        {
+        return proxy;
+        }
+      }
+    }
+
+  return 0;
+}
+
+//-----------------------------------------------------------------------------
+pqServerManagerModelItem* pqServerManagerModel::findItemHelper(
   const pqServerManagerModel* const model, const QMetaObject& mo, 
   const QString& name)
 {
@@ -200,19 +250,26 @@ void pqServerManagerModel::findItemsHelper(const pqServerManagerModel *const mod
 void pqServerManagerModel::onProxyRegistered(const QString& group,
   const QString& name, vtkSMProxy* proxy)
 {
-  if (group.endsWith("_prototypes"))
+  if (group.endsWith("_prototypes") || proxy->GetSession() == NULL)
     {
     // Ignore prototype proxies.
     return;
     }
-
+  if (name=="StreamingOptionsInstance" || name=="AdaptiveOptionsInstance")
+    {
+    // Ignore this particular proxy
+    return;
+    }
   if (!proxy)
     {
     qCritical() << "Null proxy cannot be registered.";
     return;
     }
 
-  pqServer* server = this->findServer(proxy->GetConnectionID());
+  pqServer* server = this->findServer(proxy->GetSession());
+
+  // Warn and return if the server can't be found. This indicates some logic error in
+  // the application.
   if (!server)
     {
     qDebug() << "Failed to locate server for newly registered proxy ("
@@ -229,7 +286,7 @@ void pqServerManagerModel::onProxyRegistered(const QString& group,
   pqProxy* item = 0;
 
   QObjectList ifaces =
-    pqApplicationCore::instance()->getPluginManager()->interfaces();
+    pqApplicationCore::instance()->interfaceTracker()->interfaces();
   foreach(QObject* iface, ifaces)
     {
     pqServerManagerModelInterface* smi = 
@@ -281,7 +338,10 @@ void pqServerManagerModel::onProxyRegistered(const QString& group,
       this, SIGNAL(nameChanged(pqServerManagerModelItem*)));
     QObject::connect(
       source, SIGNAL(modifiedStateChanged(pqServerManagerModelItem*)),
-      this, SIGNAL(nameChanged(pqServerManagerModelItem*)));
+      this, SIGNAL(modifiedStateChanged(pqServerManagerModelItem*)));
+    QObject::connect(
+      source, SIGNAL(dataUpdated(pqPipelineSource*)),
+      this, SIGNAL(dataUpdated(pqPipelineSource*)));
 
     emit this->preSourceAdded(source);
     }
@@ -332,7 +392,7 @@ void pqServerManagerModel::onProxyUnRegistered(const QString& group,
   // Verify if the proxy is registered under a different name in the same group.
   // If so, we are simply renaming the proxy.
   vtkSmartPointer<vtkStringList> names = vtkSmartPointer<vtkStringList>::New();
-  vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
+  vtkSMProxyManager* pxm = proxy->GetProxyManager();
   pxm->GetProxyNames(group.toAscii().data(), proxy, names);
   for (int cc=0; cc < names->GetLength(); cc++)
     {
@@ -399,6 +459,8 @@ void pqServerManagerModel::onConnectionCreated(vtkIdType id)
 
   vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
   pqServer* server = new pqServer(id, pm->GetOptions(), this);
+  server->setResource(this->Internal->ActiveResource);
+  this->Internal->ActiveResource = pqServerResource();
 
   emit this->preItemAdded(server);
   emit this->preServerAdded(server);
@@ -437,6 +499,14 @@ void pqServerManagerModel::onConnectionClosed(vtkIdType id)
   delete server;
 }
 
+//-----------------------------------------------------------------------------
+void pqServerManagerModel::onStateLoaded(vtkPVXMLElement* root,
+  vtkSMProxyLocator* locator)
+{
+  pqHelperProxyStateLoader loader;
+  loader.loadState(root, locator);
+}
+
 //-----------------------------------------------------------------------------
 void pqServerManagerModel::beginRemoveServer(pqServer *server)
 {
@@ -449,3 +519,4 @@ void pqServerManagerModel::endRemoveServer()
   emit this->finishedRemovingServer();
 }
 
+
