@@ -34,20 +34,19 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqChartRepresentation.h"
 #include "pqComparativeRenderView.h"
+#include "pqComparativeXYBarChartView.h"
+#include "pqComparativeXYChartView.h"
+#include "pqParallelCoordinatesChartView.h"
 #include "pqRenderView.h"
+//#include "pqScatterPlotView.h"
 #include "pqServer.h"
 #include "pqSpreadSheetView.h"
-#include "pqTableView.h"
 #include "pqTextRepresentation.h"
 #include "pqTwoDRenderView.h"
-//#include "pqScatterPlotView.h"
-#include "pqXYChartView.h"
 #include "pqXYBarChartView.h"
-#include "pqComparativeXYChartView.h"
-#include "pqComparativeXYBarChartView.h"
-#include "pqParallelCoordinatesChartView.h"
-#include "vtkSMContextViewProxy.h"
+#include "pqXYChartView.h"
 #include "vtkSMComparativeViewProxy.h"
+#include "vtkSMContextViewProxy.h"
 #include "vtkSMProxyManager.h"
 #include "vtkSMRenderViewProxy.h"
 
@@ -67,7 +66,6 @@ QStringList pqStandardViewModules::viewTypes() const
   return QStringList() <<
     pqRenderView::renderViewType() <<
     pqTwoDRenderView::twoDRenderViewType() <<
-    pqTableView::tableType() <<
     pqSpreadSheetView::spreadsheetViewType() <<
 //    pqScatterPlotView::scatterPlotViewType() <<
     pqXYChartView::XYChartViewType() <<
@@ -92,10 +90,6 @@ QString pqStandardViewModules::viewTypeName(const QString& type) const
     {
     return pqRenderView::renderViewTypeName();
     }
-  else if(type == pqTableView::tableType())
-    {
-    return pqTableView::tableTypeName();
-    }
   else if (type == pqComparativeRenderView::comparativeRenderViewType())
     {
     return pqComparativeRenderView::comparativeRenderViewTypeName();
@@ -142,9 +136,9 @@ bool pqStandardViewModules::canCreateView(const QString& viewtype) const
 }
 
 vtkSMProxy* pqStandardViewModules::createViewProxy(const QString& viewtype,
-                                                   pqServer *vtkNotUsed(server))
+                                                   pqServer *server)
 {
-  vtkSMProxyManager* pxm = vtkSMProxyManager::GetProxyManager();
+  vtkSMProxyManager* pxm = server->proxyManager();
   const char* root_xmlname = 0;
   if(viewtype == pqRenderView::renderViewType())
     {
@@ -166,10 +160,6 @@ vtkSMProxy* pqStandardViewModules::createViewProxy(const QString& viewtype,
     {
     root_xmlname = "2DRenderView";
     }
-  else if(viewtype == pqTableView::tableType())
-    {
-    root_xmlname = "TableView";
-    }
   else if (viewtype == pqSpreadSheetView::spreadsheetViewType())
     {
     root_xmlname = "SpreadSheetView";
