@@ -33,6 +33,8 @@ def reset_trace_globals():
     "representations" : ["Input"],
     "animation" : ["Cues"]}
   trace_globals.proxy_ctor_hook = None
+  trace_globals.paused_for_animation = False
+  trace_globals.paused_for_execute_script = False
   reset_trace_observer()
 reset_trace_globals()
 
@@ -442,6 +444,48 @@ def trace_save_screenshot(filename, size, allViews):
       trace_globals.trace_output.append(saveStr)
   trace_globals.trace_output.append("\n")
 
+def trace_save_animation(filename, magnification, quality, frame_rate):
+  """This method is called from the paraview C++ implementation. Do not change
+     the arguments without updating the C++ code."""
+  if not trace_globals.observer_active: return
+
+  # make sure the trace is up to date
+  append_trace()
+
+  trace_globals.trace_output.append(
+    "WriteAnimation('%s', Magnification=%d, Quality=%d, FrameRate=%f)" % \
+     (filename, magnification, quality, frame_rate))
+  trace_globals.trace_output.append("\n")
+  # we pause tracing while the animation is playing.
+  stop_trace()
+  trace_globals.paused_for_animation = True
+
+def trace_save_animation_end():
+  """This method is caleld from the ParaView C++ implementation. Do not chnage
+     the arguments without updating the C++ code."""
+  if not trace_globals.paused_for_animation: return
+  trace_globals.paused_for_animation = False
+  add_observers()
+
+def trace_save_execute_script(code):
+  """This method is called from the paraview C++ implementation. Do not change
+     the arguments without updating the C++ code."""
+  if not trace_globals.observer_active: return
+
+  # make sure the trace is up to date
+  append_trace()
+
+  trace_globals.trace_output.append("exec(\"\"\"\n%s\"\"\", dict(globals()))\n\n" % code)
+  stop_trace()
+  trace_globals.paused_for_execute_script = True
+
+def trace_save_execute_script_end():
+  """This method is called from the paraview C++ implementation. Do not change
+     the arguments without updating the C++ code."""
+  if not trace_globals.paused_for_execute_script: return
+  trace_globals.paused_for_execute_script = False
+  add_observers()
+
 def property_references_untraced_proxy(propPyVariable, propValue, propInfoList):
   """Given a property pyvariable, the property value, and a list of prop_trace_info
   objects, this methods looks for the prop_trace_info in the list with the matching
@@ -567,6 +611,10 @@ def append_trace():
           ctorMethod = "CreateComparativeXYPlotView"
         elif info.Proxy.GetXMLName() == "ComparativeBarChartView":
           ctorMethod = "CreateComparativeBarChartView"
+        elif info.Proxy.GetXMLName() == "ParallelCoordinatesChartView":
+          ctorMethod = "CreateParallelCoordinatesChartView"
+        elif info.Proxy.GetXMLName() == "2DRenderView":
+          ctorMethod = "Create2DRenderView"
         else:
           ctorMethod = "CreateRenderView"
 
