@@ -83,6 +83,12 @@ vtkDataObject* vtkXdmfHeavyData::ReadData()
 {
   if (this->Domain->GetNumberOfGrids() == 1)
     {
+    // There's just 1 grid. Now in serial, this is all good. In parallel, we
+    // need to be care:
+    // 1. If the data is structured, we respect the update-extent and read
+    //    accordingly.
+    // 2. If the data is unstructrued, we read only on the root node. The user
+    //    can apply D3 or something to repartition the data.
     return this->ReadData(this->Domain->GetGrid(0));
     }
 
@@ -431,6 +437,17 @@ vtkDataObject* vtkXdmfHeavyData::ReadUnstructuredGrid(XdmfGrid* xmfGrid)
   vtkSmartPointer<vtkUnstructuredGrid> ugData =
     vtkSmartPointer<vtkUnstructuredGrid>::New();
 
+  // BUG #12527. For non-partitioned data, don't read unstructured grid on
+  // process id > 0.
+  if (this->Piece != 0 &&
+    this->Domain->GetNumberOfGrids() == 1 &&
+    this->Domain->GetVTKDataType() == VTK_UNSTRUCTURED_GRID &&
+    this->Domain->GetSetsSelection()->GetNumberOfArrays() == 0)
+    {
+    ugData->Register(NULL);
+    return ugData;
+    }
+
   XdmfTopology* xmfTopology = xmfGrid->GetTopology();
   XdmfArray* xmfConnectivity = xmfTopology->GetConnectivity();
 
