@@ -24,9 +24,10 @@
 #include "vtkStringArray.h"
 #include "vtkStdString.h"
 #include "vtksys/ios/sstream"
-
 #include "vtkObjectFactory.h"
 
+#include <algorithm>
+#include <limits>
 #include "math.h"
 
 //-----------------------------------------------------------------------------
@@ -57,11 +58,13 @@ vtkAxis::vtkAxis()
   this->TitleProperties->SetJustificationToCentered();
   this->Minimum = 0.0;
   this->Maximum = 6.66;
+  this->MinimumLimit = std::numeric_limits<double>::max() * -1.;
+  this->MaximumLimit = std::numeric_limits<double>::max();
   this->LogScale = false;
   this->GridVisible = true;
   this->LabelsVisible = true;
   this->Precision = 2;
-  this->Notation = 0; // Mixed - do the right thing...
+  this->Notation = STANDARD_NOTATION;
   this->Behavior = 0;
   this->Pen = vtkPen::New();
   this->Pen->SetColor(0, 0, 0);
@@ -368,6 +371,7 @@ bool vtkAxis::Paint(vtkContext2D *painter)
 //-----------------------------------------------------------------------------
 void vtkAxis::SetMinimum(double minimum)
 {
+  minimum = std::max(minimum, this->MinimumLimit);
   if (this->Minimum == minimum)
     {
     return;
@@ -378,9 +382,24 @@ void vtkAxis::SetMinimum(double minimum)
   this->Modified();
 }
 
+//-----------------------------------------------------------------------------
+void vtkAxis::SetMinimumLimit(double lowest)
+{
+  if (this->MinimumLimit == lowest)
+    {
+    return;
+    }
+  this->MinimumLimit = lowest;
+  if (this->Minimum < lowest )
+    {
+    this->SetMinimum(lowest);
+    }
+}
+
 //-----------------------------------------------------------------------------
 void vtkAxis::SetMaximum(double maximum)
 {
+  maximum = std::min(maximum, this->MaximumLimit);
   if (this->Maximum == maximum)
     {
     return;
@@ -391,6 +410,20 @@ void vtkAxis::SetMaximum(double maximum)
   this->Modified();
 }
 
+//-----------------------------------------------------------------------------
+void vtkAxis::SetMaximumLimit(double highest)
+{
+  if (this->MaximumLimit == highest)
+    {
+    return;
+    }
+  this->MaximumLimit = highest;
+  if (this->Maximum > highest)
+    {
+    this->SetMaximum(highest);
+    }
+}
+
 //-----------------------------------------------------------------------------
 void vtkAxis::SetRange(double minimum, double maximum)
 {
@@ -710,6 +743,21 @@ void vtkAxis::GenerateTickLabels(double min, double max)
         {
         value = min + double(i) * mult * this->TickInterval;
         }
+      if (this->TickInterval < 1.0)
+        {
+        // For small TickInterval, increase the precision of the comparison
+        if (fabs(value) < (0.00000001 * this->TickInterval))
+          {
+          value = 0.0;
+          }
+        }
+      else
+        {
+        if (fabs(value) < 0.00000001)
+          {
+          value = 0.0;
+          }
+        }
       this->TickPositions->InsertNextValue(value);
       // Make a tick mark label for the tick
       if (this->LogScale)
@@ -723,12 +771,12 @@ void vtkAxis::GenerateTickLabels(double min, double max)
         {
         ostr.precision(this->Precision);
         }
-      if (this->Notation == 1)
+      if (this->Notation == SCIENTIFIC_NOTATION)
         {
         // Scientific notation
         ostr.setf(vtksys_ios::ios::scientific, vtksys_ios::ios::floatfield);
         }
-      else if (this->Notation == 2)
+      else if (this->Notation == FIXED_NOTATION)
         {
         ostr.setf(ios::fixed, ios::floatfield);
         }
@@ -758,12 +806,11 @@ void vtkAxis::GenerateTickLabels()
       {
       ostr.precision(this->Precision);
       }
-    if (this->Notation == 1)
+    if (this->Notation == SCIENTIFIC_NOTATION)
       {
-      // Scientific notation
       ostr.setf(vtksys_ios::ios::scientific, vtksys_ios::ios::floatfield);
       }
-    else if (this->Notation == 2)
+    else if (this->Notation == FIXED_NOTATION)
       {
       ostr.setf(ios::fixed, ios::floatfield);
       }
@@ -1015,12 +1062,11 @@ void vtkAxis::GenerateLogScaleTickMarks(int order,
       {
       ostr.precision(this->Precision);
       }
-    if (this->Notation == 1)
+    if (this->Notation == SCIENTIFIC_NOTATION)
       {
-      // Scientific notation
       ostr.setf(vtksys_ios::ios::scientific, vtksys_ios::ios::floatfield);
       }
-    else if (this->Notation == 2)
+    else if (this->Notation == FIXED_NOTATION)
       {
       ostr.setf(ios::fixed, ios::floatfield);
       }
