@@ -1206,7 +1206,6 @@ void vtkCellLocator::BuildLocatorInternal()
   int parentOffset;
   vtkIdList *octant;
   int numCellsPerBucket = this->NumberOfCellsPerNode;
-  typedef vtkIdList *vtkIdListPtr;
   int prod, numOctants;
   double hTol[3];
   
@@ -1391,7 +1390,7 @@ void vtkCellLocator::GenerateRepresentation(int level, vtkPolyData *pd)
   vtkCellArray *polys;
   int l, i, j, k, ii, boundary[3];
   vtkIdType idx = 0;
-  vtkIdList *inside, *Inside[3];
+  vtkIdList *inside, *Inside[3] = {0, 0, 0};
   int numDivs=1;
 
   this->BuildLocatorIfNeeded();
