@@ -29,6 +29,7 @@
 #include "vtkInformationIntegerVectorKey.h"
 #include "vtkInformationObjectBaseKey.h"
 #include "vtkInformationRequestKey.h"
+#include "vtkInformationUnsignedLongKey.h"
 #include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkSmartPointer.h"
@@ -39,6 +40,7 @@ vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, CONTINUE_EXECUTING, Int
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, EXACT_EXTENT, Integer);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, REQUEST_UPDATE_EXTENT, Request);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, REQUEST_UPDATE_EXTENT_INFORMATION, Request);
+vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, REQUEST_MANAGE_INFORMATION, Request);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, REQUEST_RESOLUTION_PROPAGATE, Request);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, MAXIMUM_NUMBER_OF_PIECES, Integer);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, UPDATE_EXTENT_INITIALIZED, Integer);
@@ -48,6 +50,7 @@ vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, UPDATE_NUMBER_OF_GHOST_
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, UPDATE_EXTENT_TRANSLATED, Integer);
 vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline, WHOLE_EXTENT, IntegerVector, 6);
 vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline, UPDATE_EXTENT, IntegerVector, 6);
+vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline, COMBINED_UPDATE_EXTENT, IntegerVector, 6);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, UNRESTRICTED_UPDATE_EXTENT, Integer);
 vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline,
                                  EXTENT_TRANSLATOR, ObjectBase,
@@ -59,7 +62,9 @@ vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, PREVIOUS_UPDATE_TIME_ST
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, TIME_RANGE, DoubleVector);
 
 vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline, PIECE_BOUNDING_BOX, DoubleVector, 6);
+vtkInformationKeyRestrictedMacro(vtkStreamingDemandDrivenPipeline, PIECE_NORMAL, DoubleVector, 3);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, PRIORITY, Double);
+vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, ORIGINAL_NUMBER_OF_CELLS, UnsignedLong);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, UPDATE_RESOLUTION, Double);
 vtkInformationKeyMacro(vtkStreamingDemandDrivenPipeline, REMOVE_ATTRIBUTE_INFORMATION, Integer);
 
@@ -133,15 +138,73 @@ int vtkStreamingDemandDrivenPipeline
       return 0;
       }
 
+    // Get the output info
+    vtkInformation* outInfo = 0;
+    if (outputPort > -1)
+      {
+      outInfo = outInfoVec->GetInformationObject(outputPort);
+      }
+
+    // Combine the requested extent into COMBINED_UPDATE_EXTENT,
+    // but only do so if the UPDATE_EXTENT key exists and if the
+    // UPDATE_EXTENT is not an empty extent
+    int *updateExtent = 0;
+    if (outInfo &&
+        ((updateExtent = outInfo->Get(UPDATE_EXTENT())) != 0) &&
+        !outInfo->Has(UPDATE_RESOLUTION()))
+      {
+      // Downstream algorithms can set UPDATE_EXTENT_INITIALIZED to
+      // REPLACE if they do not want to combine with previous extents
+      if (outInfo->Get(UPDATE_EXTENT_INITIALIZED()) !=
+          VTK_UPDATE_EXTENT_REPLACE)
+        {
+        int *combinedExtent = outInfo->Get(COMBINED_UPDATE_EXTENT());
+        if (combinedExtent &&
+            combinedExtent[0] <= combinedExtent[1] &&
+            combinedExtent[2] <= combinedExtent[3] &&
+            combinedExtent[4] <= combinedExtent[5])
+          {
+          if (updateExtent[0] <= updateExtent[1] &&
+              updateExtent[2] <= updateExtent[3] &&
+              updateExtent[4] <= updateExtent[5])
+            {
+            int newExtent[6];
+            for (int ii = 0; ii < 6; ii += 2)
+              {
+              newExtent[ii] = combinedExtent[ii];
+              if (updateExtent[ii] < newExtent[ii])
+                {
+                newExtent[ii] = updateExtent[ii];
+                }
+              newExtent[ii+1] = combinedExtent[ii+1];
+              if (updateExtent[ii+1] > newExtent[ii+1])
+                {
+                newExtent[ii+1] = updateExtent[ii+1];
+                }
+              }
+            outInfo->Set(COMBINED_UPDATE_EXTENT(), newExtent, 6);
+            outInfo->Set(UPDATE_EXTENT(), newExtent, 6);
+            }
+          else
+            {
+            outInfo->Set(UPDATE_EXTENT(), combinedExtent, 6);
+            }
+          }
+        else
+          {
+          outInfo->Set(COMBINED_UPDATE_EXTENT(), updateExtent, 6);
+          }
+        }
+      }
+
     // If we need to execute, propagate the update extent.
     int result = 1;
     int N2E = this->NeedToExecuteData(outputPort,inInfoVec,outInfoVec);
     if (!N2E && 
-        outputPort>-1 && 
+        outInfo &&
         this->GetNumberOfInputPorts() &&
         inInfoVec[0]->GetNumberOfInformationObjects () > 0)
       {
-      vtkInformation* outInfo = outInfoVec->GetInformationObject(outputPort);
       vtkInformation* inInfo = inInfoVec[0]->GetInformationObject(0);
       int outNumberOfPieces = outInfo->Get(UPDATE_NUMBER_OF_PIECES());
       int inNumberOfPieces = inInfo->Get(UPDATE_NUMBER_OF_PIECES());
@@ -176,23 +239,33 @@ int vtkStreamingDemandDrivenPipeline
       if(!this->InputCountIsValid(inInfoVec) || 
          !this->InputTypeIsValid(inInfoVec))
         {
-        return 0;
+        result = 0;
         }
-
-      // Remove update-related keys from the input information.
-      this->ResetUpdateInformation(request, inInfoVec, outInfoVec);
-
-      // Invoke the request on the algorithm.
-      this->LastPropogateUpdateExtentShortCircuited = 0;
-      result = this->CallAlgorithm(request, vtkExecutive::RequestUpstream,
-                                   inInfoVec, outInfoVec);
-      
-      // Propagate the update extent to all inputs.
-      if(result)
+      else
         {
-        result = this->ForwardUpstream(request);
+        // Remove update-related keys from the input information.
+        this->ResetUpdateInformation(request, inInfoVec, outInfoVec);
+
+        // Invoke the request on the algorithm.
+        this->LastPropogateUpdateExtentShortCircuited = 0;
+        result = this->CallAlgorithm(request, vtkExecutive::RequestUpstream,
+                                     inInfoVec, outInfoVec);
+
+        // Propagate the update extent to all inputs.
+        if(result)
+          {
+          result = this->ForwardUpstream(request);
+          }
+        result = 1;
+        }
+      }
+    if (this->LastPropogateUpdateExtentShortCircuited)
+      {
+      if(outInfo && outInfo->Has(COMBINED_UPDATE_EXTENT()))
+        {
+        static int emptyExt[6] = { 0, -1, 0, -1, 0, -1 };
+        outInfo->Set(COMBINED_UPDATE_EXTENT(), emptyExt, 6);
         }
-      result = 1;
       }
     return result;
     }
@@ -202,15 +275,21 @@ int vtkStreamingDemandDrivenPipeline
     // Let the superclass handle the request first.
     if(this->Superclass::ProcessRequest(request, inInfoVec, outInfoVec))
       {
-      // Crop the output if the exact extent flag is set.
       for(int i=0; i < outInfoVec->GetNumberOfInformationObjects(); ++i)
         {
         vtkInformation* info = outInfoVec->GetInformationObject(i);
+        // Crop the output if the exact extent flag is set.
         if(info->Has(EXACT_EXTENT()) && info->Get(EXACT_EXTENT()))
           {
           vtkDataObject* data = info->Get(vtkDataObject::DATA_OBJECT());
           vtkStreamingDemandDrivenPipelineToDataObjectFriendship::Crop(data);
           }
+        // Clear combined update extent, since the update cycle has completed
+        if (info->Has(COMBINED_UPDATE_EXTENT()))
+          {
+          static int emptyExt[6] = { 0, -1, 0, -1, 0, -1 };
+          info->Set(COMBINED_UPDATE_EXTENT(), emptyExt, 6);
+          }
         }
       return 1;
       }
@@ -572,78 +651,125 @@ vtkStreamingDemandDrivenPipeline
         {
         vtkInformation* outInfo = outInfoVec->GetInformationObject(i);
 
-        // Copy the priority result always, algorithms can modify it in RUEI if needed
+        // Copy these things across always, and let algorithms modify in RUEI
+        // if they need to.
         outInfo->CopyEntry(inInfo, PRIORITY());
+        outInfo->CopyEntry(inInfo, ORIGINAL_NUMBER_OF_CELLS());
+        outInfo->CopyEntry(inInfo, PIECE_NORMAL());
 
-        // Copy the attribute meta information when algorithm is known not to modify it
         vtkInformation *algsProps = this->GetAlgorithm()->GetInformation();
-        if (
-            algsProps->Has(vtkAlgorithm::PRESERVES_RANGES()) ||
-            algsProps->Has(vtkAlgorithm::PRESERVES_ATTRIBUTES()) ||
-            algsProps->Has(vtkAlgorithm::PRESERVES_DATASET())
-            )
+        if (algsProps->Has(vtkAlgorithm::MANAGES_METAINFORMATION()))
           {
-          if (inInfo->Has(vtkDataObject::CELL_DATA_VECTOR()))
+          //Let the algorithm decide what to do with the met informaiton
+          this->InAlgorithm = 1;
+          vtkInformation *newrequest = vtkInformation::New();
+          newrequest->Set(REQUEST_MANAGE_INFORMATION());
+          this->Algorithm->ProcessRequest(newrequest, inInfoVec, outInfoVec);
+          this->InAlgorithm = 0;
+
+          //if algorithm said to, remove the attribute range information downstream
+          if(newrequest->Has(REMOVE_ATTRIBUTE_INFORMATION()))
             {
-            outInfo->CopyEntry(inInfo, vtkDataObject::CELL_DATA_VECTOR(), 1);
-            }
-          if (inInfo->Has(vtkDataObject::POINT_DATA_VECTOR()))
-            {
-            outInfo->CopyEntry(inInfo, vtkDataObject::POINT_DATA_VECTOR(), 1);
+            request->Set(REMOVE_ATTRIBUTE_INFORMATION(), 1);
+
+            //remove the attribute range information downstream
+            vtkInformationVector *miv;
+            miv = outInfo->Get(vtkDataObject::CELL_DATA_VECTOR());
+            if (miv)
+              {
+              int nArrays = miv->GetNumberOfInformationObjects();
+              for (int n = 0; n < nArrays; n++)
+                {
+                vtkInformation *oArray = miv->GetInformationObject(n);
+                oArray->Remove(vtkDataObject::FIELD_ARRAY_NAME());
+                oArray->Remove(vtkDataObject::PIECE_FIELD_RANGE());
+                }
+              }
+            miv = outInfo->Get(vtkDataObject::POINT_DATA_VECTOR());
+            if (miv)
+              {
+              int nArrays = miv->GetNumberOfInformationObjects();
+              for (int n = 0; n < nArrays; n++)
+                {
+                vtkInformation *oArray = miv->GetInformationObject(n);
+                oArray->Remove(vtkDataObject::FIELD_ARRAY_NAME());
+                oArray->Remove(vtkDataObject::PIECE_FIELD_RANGE());
+                }
+              }
             }
+          newrequest->Delete();
           }
         else
           {
-          //RI normally passes it on always, so this flag says remove it downstream
-          request->Set(REMOVE_ATTRIBUTE_INFORMATION(), 1);
-          }
-
-        //remove the attribute range information downstream
-        if(request->Has(REMOVE_ATTRIBUTE_INFORMATION()))
-          {
-          vtkInformationVector *miv;
-          miv = outInfo->Get(vtkDataObject::CELL_DATA_VECTOR());
-          if (miv)
+          //Copy the attribute meta information when algorithm is known not to modify it
+          if (
+              algsProps->Has(vtkAlgorithm::PRESERVES_RANGES()) ||
+              algsProps->Has(vtkAlgorithm::PRESERVES_ATTRIBUTES()) ||
+              algsProps->Has(vtkAlgorithm::PRESERVES_DATASET())
+              )
             {
-            int nArrays = miv->GetNumberOfInformationObjects();
-            for (int n = 0; n < nArrays; n++)
+            if (inInfo->Has(vtkDataObject::CELL_DATA_VECTOR()))
               {
-              vtkInformation *oArray = miv->GetInformationObject(n);
-              oArray->Remove(vtkDataObject::FIELD_ARRAY_NAME());
-              oArray->Remove(vtkDataObject::PIECE_FIELD_RANGE());
+              outInfo->CopyEntry(inInfo, vtkDataObject::CELL_DATA_VECTOR(), 1);
+              }
+            if (inInfo->Has(vtkDataObject::POINT_DATA_VECTOR()))
+              {
+              outInfo->CopyEntry(inInfo, vtkDataObject::POINT_DATA_VECTOR(), 1);
               }
             }
-          miv = outInfo->Get(vtkDataObject::POINT_DATA_VECTOR());
-          if (miv)
+          else
             {
-            int nArrays = miv->GetNumberOfInformationObjects();
-            for (int n = 0; n < nArrays; n++)
+            //RI normally passes it on always, so this flag says remove it downstream
+            request->Set(REMOVE_ATTRIBUTE_INFORMATION(), 1);
+            }
+
+          //remove the attribute range information downstream
+          if(request->Has(REMOVE_ATTRIBUTE_INFORMATION()))
+            {
+            vtkInformationVector *miv;
+            miv = outInfo->Get(vtkDataObject::CELL_DATA_VECTOR());
+            if (miv)
               {
-              vtkInformation *oArray = miv->GetInformationObject(n);
-              oArray->Remove(vtkDataObject::FIELD_ARRAY_NAME());
-              oArray->Remove(vtkDataObject::PIECE_FIELD_RANGE());
+              int nArrays = miv->GetNumberOfInformationObjects();
+              for (int n = 0; n < nArrays; n++)
+                {
+                vtkInformation *oArray = miv->GetInformationObject(n);
+                oArray->Remove(vtkDataObject::FIELD_ARRAY_NAME());
+                oArray->Remove(vtkDataObject::PIECE_FIELD_RANGE());
+                }
+              }
+            miv = outInfo->Get(vtkDataObject::POINT_DATA_VECTOR());
+            if (miv)
+              {
+              int nArrays = miv->GetNumberOfInformationObjects();
+              for (int n = 0; n < nArrays; n++)
+                {
+                vtkInformation *oArray = miv->GetInformationObject(n);
+                oArray->Remove(vtkDataObject::FIELD_ARRAY_NAME());
+                oArray->Remove(vtkDataObject::PIECE_FIELD_RANGE());
+                }
               }
             }
+
+          // Copy the geometric meta information when algorithm is known not to modify it
+          if (
+              algsProps->Has(vtkAlgorithm::PRESERVES_BOUNDS()) ||
+              algsProps->Has(vtkAlgorithm::PRESERVES_GEOMETRY()) ||
+              algsProps->Has(vtkAlgorithm::PRESERVES_DATASET())
+              )
+            {
+            outInfo->CopyEntry(inInfo, PIECE_BOUNDING_BOX());
+            }
+
+          // Copy the topological meta information when algorithm is known not to modify it
+          if (
+              algsProps->Has(vtkAlgorithm::PRESERVES_TOPOLOGY()) ||
+              algsProps->Has(vtkAlgorithm::PRESERVES_DATASET())
+              )
+            {
+            outInfo->CopyEntry(inInfo, vtkDataObject::DATA_GEOMETRY_UNMODIFIED());
+            }
           }
-
-        // Copy the geometric meta information when algorithm is known not to modify it
-        if (
-            algsProps->Has(vtkAlgorithm::PRESERVES_BOUNDS()) ||
-            algsProps->Has(vtkAlgorithm::PRESERVES_GEOMETRY()) ||
-            algsProps->Has(vtkAlgorithm::PRESERVES_DATASET())
-            )
-          {
-          outInfo->CopyEntry(inInfo, PIECE_BOUNDING_BOX());          
-          }        
-
-        // Copy the topological meta information when algorithm is known not to modify it
-        if (
-            algsProps->Has(vtkAlgorithm::PRESERVES_TOPOLOGY()) ||
-            algsProps->Has(vtkAlgorithm::PRESERVES_DATASET())
-            )
-          {
-          outInfo->CopyEntry(inInfo, vtkDataObject::DATA_GEOMETRY_UNMODIFIED());
-          }        
         }
       }
     }
@@ -865,23 +991,25 @@ int vtkStreamingDemandDrivenPipeline
         updateExtent[2] <= updateExtent[3] &&
         updateExtent[4] <= updateExtent[5]))
       {
-      if (!outInfo->Has(UPDATE_RESOLUTION()) &&
-          !outInfo->Has(UNRESTRICTED_UPDATE_EXTENT()))
+      if (!outInfo->Has(UNRESTRICTED_UPDATE_EXTENT()))
         {
-        // Update extent is outside the whole extent and is not empty.
-        vtkErrorMacro("The update extent specified in the "
-                      "information for output port " << outputPort
-                      << " on algorithm " << this->Algorithm->GetClassName()
-                      << "(" << this->Algorithm << ") is "
-                      << updateExtent[0] << " " << updateExtent[1] << " "
-                      << updateExtent[2] << " " << updateExtent[3] << " "
-                      << updateExtent[4] << " " << updateExtent[5]
-                      << ", which is outside the whole extent "
-                      << wholeExtent[0] << " " << wholeExtent[1] << " "
-                      << wholeExtent[2] << " " << wholeExtent[3] << " "
-                      << wholeExtent[4] << " " << wholeExtent[5] << ".");
+        if (!outInfo->Has(UPDATE_RESOLUTION()))
+          {
+          // Update extent is outside the whole extent and is not empty.
+          vtkErrorMacro("The update extent specified in the "
+                        "information for output port " << outputPort
+                        << " on algorithm " << this->Algorithm->GetClassName()
+                        << "(" << this->Algorithm << ") is "
+                        << updateExtent[0] << " " << updateExtent[1] << " "
+                        << updateExtent[2] << " " << updateExtent[3] << " "
+                        << updateExtent[4] << " " << updateExtent[5]
+                        << ", which is outside the whole extent "
+                        << wholeExtent[0] << " " << wholeExtent[1] << " "
+                        << wholeExtent[2] << " " << wholeExtent[3] << " "
+                        << wholeExtent[4] << " " << wholeExtent[5] << ".");
+          }
+        return 0;
         }
-      return 0;
       }
     }
   if(dataInfo->Get(vtkDataObject::DATA_EXTENT_TYPE()) == VTK_TIME_EXTENT)
