@@ -43,10 +43,11 @@
 #include "vtkObject.h"
 #include "vtkCommand.h"  // for event defines
 #include "QVTKWin32Header.h"  // for export define
-#include "qobject.h"          // for version info
+#include <QtCore/QObject>          // for version info
 
 class QObject;
 class vtkQtConnections;
+class vtkQtConnection;
 
 // manage connections between VTK object events and Qt slots
 class QVTK_EXPORT vtkEventQtSlotConnect : public vtkObject
@@ -65,11 +66,7 @@ class QVTK_EXPORT vtkEventQtSlotConnect : public vtkObject
     virtual void Connect(vtkObject* vtk_obj, unsigned long event, 
                          const QObject* qt_obj, const char* slot, 
                          void* client_data=NULL, float priority=0.0
-#if QT_VERSION >= 0x040000
                          ,Qt::ConnectionType type = Qt::AutoConnection);
-#else
-                         );
-#endif
     
     // Description:
     // Disconnect a vtk object from a qt object.
@@ -84,6 +81,8 @@ class QVTK_EXPORT vtkEventQtSlotConnect : public vtkObject
 
   protected:
     vtkQtConnections* Connections;
+    friend class vtkQtConnection;
+    void RemoveConnection(vtkQtConnection*);
   
     vtkEventQtSlotConnect();
     ~vtkEventQtSlotConnect();
