@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkExodusReader.h,v $
+  Module:    vtkExodusReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -32,7 +32,6 @@
 // 
 
 
-
 #ifndef __vtkExodusReader_h
 #define __vtkExodusReader_h
 
@@ -57,7 +56,7 @@ class VTK_HYBRID_EXPORT vtkExodusReader : public vtkUnstructuredGridAlgorithm
 {
 public:
   static vtkExodusReader *New();
-  vtkTypeRevisionMacro(vtkExodusReader,vtkUnstructuredGridAlgorithm);
+  vtkTypeMacro(vtkExodusReader,vtkUnstructuredGridAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   // Description:
@@ -66,12 +65,13 @@ public:
 
   // Description:
   // Specify file name of the Exodus file.
-  vtkSetStringMacro(FileName);
+  // @deprecated Replaced by vtkExodusIIReader as of VTK 5.8.
+  VTK_LEGACY(void SetFileName(const char* fname));    
   vtkGetStringMacro(FileName);
 
   // Description:
   // Specify file name of the xml file.
-  void SetXMLFileName(const char *filename); 
+  vtkSetStringMacro(XMLFileName);
   vtkGetStringMacro(XMLFileName);
 
   // Description:
