@@ -38,8 +38,8 @@
 #include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkUnstructuredGrid.h"
 
-#include "netcdf.h"
-#include "vtkExodusII.h"
+#include "vtk_netcdf.h"
+#include "vtk_exodusII.h"
 
 #include "vtksys/SystemTools.hxx"
 
@@ -143,6 +143,8 @@ protected:
 // Instantiate object with NULL filename.
 vtkPExodusIIReader::vtkPExodusIIReader()
 {
+  this->ProcRank = 0;
+  this->ProcSize = 1;
   // NB. SetController will initialize ProcSize and ProcRank
   this->Controller = 0;
   this->SetController( vtkMultiProcessController::GetGlobalController() );
@@ -160,6 +162,7 @@ vtkPExodusIIReader::vtkPExodusIIReader()
   this->MultiFileName = new char[vtkPExodusIIReaderMAXPATHLEN];
   this->XMLFileName=NULL;
   this->LastCommonTimeStep = -1;
+  this->VariableCacheSize = 100;
 }
  
 //----------------------------------------------------------------------------
@@ -474,7 +477,7 @@ int vtkPExodusIIReader::RequestData(
       progress->SetReader( this );
       progress->SetIndex( reader_idx );
       er->AddObserver( vtkCommand::ProgressEvent, progress );
-      progress->Delete();
+      progress->Delete();      
 
       this->ReaderList.push_back( er );
       }
@@ -503,6 +506,14 @@ int vtkPExodusIIReader::RequestData(
   cout << "\n\n ************************************* Parallel master reader dump\n";
   this->Dump();
 #endif // DBG_PEXOIIRDR
+
+  //setup the cache size for each reader
+  double fractionalCacheSize = 0;
+  if (this->VariableCacheSize > 0 )
+    {
+    fractionalCacheSize = this->VariableCacheSize / static_cast<int>( this->ReaderList.size() );
+    }
+
   // This constructs the filenames
   int fast_path_reader_index = -1;
   for ( fileIndex = min, reader_idx=0; fileIndex <= max; ++fileIndex, ++reader_idx )
@@ -560,9 +571,16 @@ int vtkPExodusIIReader::RequestData(
         }
       else
         {
-        // Let the metadata know the time value so that the Metadata->RequestData call below will generate
-        // the animated mode shape properly.
-        this->SetModeShapeTime( requestedTimeSteps[0] );
+        // Let the metadata know the time value so that the
+        // Metadata->RequestData call below will generate the animated mode
+        // shape properly.
+
+        // Don't use this->SetModeShapeTime because that will cause Modified
+        // to be called.
+        //this->SetModeShapeTime( requestedTimeSteps[0] );
+        double phase = requestedTimeSteps[0] - floor(requestedTimeSteps[0]);
+        this->Metadata->ModeShapeTime = phase;
+
         this->ReaderList[reader_idx]->SetTimeStep( this->TimeStep );
         this->ReaderList[reader_idx]->SetModeShapeTime( requestedTimeSteps[0] );
         output->GetInformation()->Set( vtkDataObject::DATA_TIME_STEPS(), requestedTimeSteps, 1 );
@@ -585,8 +603,6 @@ int vtkPExodusIIReader::RequestData(
     this->ReaderList[reader_idx]->SetDisplacementMagnitude( this->GetDisplacementMagnitude() );
     this->ReaderList[reader_idx]->SetHasModeShapes( this->GetHasModeShapes() );
     this->ReaderList[reader_idx]->SetAnimateModeShapes( this->GetAnimateModeShapes() );
-    this->ReaderList[reader_idx]->SetEdgeFieldDecorations( this->GetEdgeFieldDecorations() );
-    this->ReaderList[reader_idx]->SetFaceFieldDecorations( this->GetFaceFieldDecorations() );
 
     this->ReaderList[reader_idx]->SetExodusModelMetadata( this->ExodusModelMetadata );
     // For now, this *must* come last before the UpdateInformation() call because its MTime is compared to the metadata's MTime,
@@ -657,7 +673,15 @@ int vtkPExodusIIReader::RequestData(
       this->ReaderList[reader_idx]->SetFastPathIdType(0);
       }
 
+    //set this reader to use the full amount of the cache
+    this->ReaderList[reader_idx]->SetCacheSize(this->VariableCacheSize);
+
+    //call the reader
     this->ReaderList[reader_idx]->Update();
+
+    //set the reader back to the fractional amount
+    this->ReaderList[reader_idx]->SetCacheSize(fractionalCacheSize);
+
     if (this->ReaderList[reader_idx]->GetProducedFastPathOutput())
       {
       //if (fast_path_reader_index != -1)
