@@ -13,7 +13,7 @@ PURPOSE.  See the above copyright notice for more information.
 
 =========================================================================*/
 /*-------------------------------------------------------------------------
-  Copyright 2008 Sandia Corporation.
+  Copyright 2011 Sandia Corporation.
   Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
   the U.S. Government retains certain rights in this software.
   -------------------------------------------------------------------------*/
@@ -35,17 +35,17 @@ PURPOSE.  See the above copyright notice for more information.
 #ifndef __vtkContingencyStatistics_h
 #define __vtkContingencyStatistics_h
 
-#include "vtkBivariateStatisticsAlgorithm.h"
+#include "vtkStatisticsAlgorithm.h"
 
 class vtkMultiBlockDataSet;
 class vtkStringArray;
 class vtkTable;
 class vtkVariant;
 
-class VTK_INFOVIS_EXPORT vtkContingencyStatistics : public vtkBivariateStatisticsAlgorithm
+class VTK_INFOVIS_EXPORT vtkContingencyStatistics : public vtkStatisticsAlgorithm
 {
 public:
-  vtkTypeMacro(vtkContingencyStatistics, vtkBivariateStatisticsAlgorithm);
+  vtkTypeMacro(vtkContingencyStatistics, vtkStatisticsAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
   static vtkContingencyStatistics* New();
 
@@ -53,40 +53,33 @@ public:
   // Given a collection of models, calculate aggregate model
   // NB: not implemented
   virtual void Aggregate( vtkDataObjectCollection*,
-                          vtkDataObject* ) { return; };
+                          vtkMultiBlockDataSet* ) { return; };
 
 protected:
   vtkContingencyStatistics();
   ~vtkContingencyStatistics();
 
-  // Description:
-  // This algorithm accepts and returns a multiblock dataset containing several tables for
-  // its meta input/output instead of a single vtkTable.
-  // FillInputPortInformation/FillOutputPortInformation are overridden accordingly.
-  virtual int FillInputPortInformation( int port, vtkInformation* info );
-  virtual int FillOutputPortInformation( int port, vtkInformation* info );
-
   // Description:
   // Execute the calculations required by the Learn option.
-  virtual void Learn( vtkTable* inData,
-                      vtkTable* inParameters,
-                      vtkDataObject* outMeta );
+  virtual void Learn( vtkTable*,
+                      vtkTable*,
+                      vtkMultiBlockDataSet* );
 
   // Description:
   // Execute the calculations required by the Derive option.
-  virtual void Derive( vtkDataObject* );
+  virtual void Derive( vtkMultiBlockDataSet* );
+
+  // Description:
+  // Execute the calculations required by the Test option.
+  virtual void Test( vtkTable*,
+                     vtkMultiBlockDataSet*,
+                     vtkTable* ); 
 
   // Description:
   // Execute the calculations required by the Assess option.
-  virtual void Assess( vtkTable* inData,
-                       vtkDataObject* inMeta,
-                       vtkTable* outData ); 
-  
-  // Description:
-  // Execute the calculations required by the Test option.
-  virtual void Test( vtkTable* inData,
-                     vtkDataObject* inMeta,
-                     vtkDataObject* outMeta ); 
+  virtual void Assess( vtkTable*,
+                       vtkMultiBlockDataSet*,
+                       vtkTable* );
 
 //BTX  
   // Description:
