@@ -34,6 +34,9 @@
 #include <vtkStringArray.h>
 #include <vtkStringToNumeric.h>
 
+#include <vtkTextCodec.h>
+#include <vtkTextCodecFactory.h>
+
 #include <vtkstd/algorithm>
 #include <vtkstd/iterator>
 #include <vtkstd/stdexcept>
@@ -43,9 +46,9 @@
 
 #include <ctype.h>
 
-#include <utf8.h>
+// #include <utf8.h>
 
-/////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
 // DelimitedTextIterator
 
 /// Output iterator object that parses a stream of Unicode characters into records and
@@ -53,7 +56,7 @@
 
 namespace {
 
-class DelimitedTextIterator
+class DelimitedTextIterator : public vtkTextCodec::OutputIterator
 {
 public:
   typedef vtkstd::forward_iterator_tag iterator_category;
@@ -101,8 +104,12 @@ public:
     // Ensure that all table columns have the same length ...
     for(vtkIdType i = 0; i != this->OutputTable->GetNumberOfColumns(); ++i)
       {
-      if(this->OutputTable->GetColumn(i)->GetNumberOfTuples() != this->OutputTable->GetColumn(0)->GetNumberOfTuples())
-        this->OutputTable->GetColumn(i)->Resize(this->OutputTable->GetColumn(0)->GetNumberOfTuples());
+      if(this->OutputTable->GetColumn(i)->GetNumberOfTuples() !=
+          this->OutputTable->GetColumn(0)->GetNumberOfTuples())
+        {
+        this->OutputTable->GetColumn(i)
+            ->Resize(this->OutputTable->GetColumn(0)->GetNumberOfTuples());
+        }
       }
   }
 
@@ -123,8 +130,9 @@ public:
       {
       return;
       }
-    vtkUnicodeString::value_type value = this->CurrentField[this->CurrentField.character_count()-1];
-    if(!this->RecordDelimiters.count(value)&&!this->Whitespace.count(value))
+    vtkUnicodeString::value_type value =
+        this->CurrentField[this->CurrentField.character_count()-1];
+    if(!this->RecordDelimiters.count(value) && !this->Whitespace.count(value))
       {
       this->InsertField();
       }
@@ -146,10 +154,13 @@ public:
   {
     // If we've already read our maximum number of records, we're done ...
     if(this->MaxRecords && this->CurrentRecordIndex == this->MaxRecordIndex)
+      {
       return *this;
+      }
 
     // Strip adjacent record delimiters and whitespace...
-    if(this->RecordAdjacent && (this->RecordDelimiters.count(value) || this->Whitespace.count(value)))
+    if(this->RecordAdjacent && (this->RecordDelimiters.count(value) ||
+                                this->Whitespace.count(value)))
       {
       return *this;
       }
@@ -241,14 +252,17 @@ public:
       }
 
     // Start a string ...
-    if(!this->WithinString && this->StringDelimiters.count(value) && this->UseStringDelimiter)
+    if(!this->WithinString && this->StringDelimiters.count(value) &&
+        this->UseStringDelimiter)
       {
       this->WithinString = value;
+      this->CurrentField.clear();
       return *this;
       }
     
     // End a string ...
-    if(this->WithinString && (this->WithinString == value) && this->UseStringDelimiter)
+    if(this->WithinString && (this->WithinString == value) &&
+        this->UseStringDelimiter)
       {
       this->WithinString = 0;
       return *this;
@@ -266,7 +280,8 @@ public:
 private:
   void InsertField()
   {
-    if(this->CurrentFieldIndex >= this->OutputTable->GetNumberOfColumns()&& 0 == this->CurrentRecordIndex)
+    if(this->CurrentFieldIndex >= this->OutputTable->GetNumberOfColumns() &&
+        0 == this->CurrentRecordIndex)
       {
       vtkAbstractArray* array;
       if(this->UnicodeArrayOutput)
@@ -352,66 +367,6 @@ private:
   vtkUnicodeString::value_type WithinString;
 };
 
-/////////////////////////////////////////////////////////////////////////////////////////
-// ascii_to_unicode
-
-template<typename OctetIteratorT, typename OutputIteratorT>
-void ascii_to_unicode(OctetIteratorT begin, OctetIteratorT end, OutputIteratorT output)
-{
-  while(begin != end)
-    {
-    const vtkTypeUInt32 code_point = *begin++;
-    if(code_point > 0x7f)
-      throw vtkstd::runtime_error("Detected a character that isn't valid US-ASCII.");
-
-    *output++ = code_point;
-    }
-
-   output.ReachedEndOfInput();
-}
-
-/////////////////////////////////////////////////////////////////////////////////////////
-// utf16_to_unicode
-
-template<typename OctetIteratorT, typename OutputIteratorT>
-void utf16_to_unicode(const bool big_endian, OctetIteratorT begin, OctetIteratorT end, OutputIteratorT output)
-{
-  while(begin != end)
-    {
-    const vtkTypeUInt8 first_byte = *begin++;
-    if(begin == end)
-      throw vtkstd::runtime_error("Premature end-of-sequence extracting UTF-16 code unit.");
-    const vtkTypeUInt8 second_byte = *begin++;
-
-    const vtkTypeUInt32 first_code_unit = big_endian ? first_byte << 8 | second_byte : second_byte << 8 | first_byte;
-
-    if(first_code_unit >= 0xd800 && first_code_unit <= 0xdfff)
-      {
-      if(begin == end)
-        throw vtkstd::runtime_error("Premature end-of-sequence extracting UTF-16 trail surrogate first byte.");
-      const vtkTypeUInt8 third_byte = *begin++;
-      if(begin == end)
-        throw vtkstd::runtime_error("Premature end-of-sequence extracting UTF-16 trail surrogate second byte.");
-      const vtkTypeUInt8 fourth_byte = *begin++;
-
-      const vtkTypeUInt32 second_code_unit = big_endian ? third_byte << 8 | fourth_byte : fourth_byte << 8 | third_byte;
-      if(second_code_unit >= 0xdc00 && second_code_unit <= 0xdfff)
-        {
-        *output++ = vtkTypeUInt32 (vtkTypeInt32 (first_code_unit << 10) + vtkTypeInt32 (second_code_unit) + (0x10000 - (0xd800 << 10) - 0xdc00));
-        }
-      else
-        {
-        throw vtkstd::runtime_error("Invalid UTF-16 trail surrogate.");
-        }
-      }
-    else
-      {
-      *output++ = first_code_unit;
-      }
-    }
-    output.ReachedEndOfInput();
-}
-
 } // End anonymous namespace
 
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -428,7 +383,8 @@ vtkDelimitedTextReader::vtkDelimitedTextReader() :
   UnicodeStringDelimiters(vtkUnicodeString::from_utf8("\"")),
   UnicodeWhitespace(vtkUnicodeString::from_utf8(" \t\r\n\v\f")),
   UnicodeEscapeCharacter(vtkUnicodeString::from_utf8("\\")),
-  HaveHeaders(false)
+  HaveHeaders(false),
+  ReplacementCharacter('x')
 {
   this->SetNumberOfInputPorts(0);
   this->SetNumberOfOutputPorts(1);
@@ -471,6 +427,7 @@ void vtkDelimitedTextReader::PrintSelf(ostream& os, vtkIndent indent)
      << "'" << endl;
   os << indent << "StringDelimiter: " 
      << this->StringDelimiter << endl;
+  os << indent << "ReplacementCharacter: " << this->ReplacementCharacter << endl;
   os << indent << "FieldDelimiterCharacters: "
      << (this->FieldDelimiterCharacters ? this->FieldDelimiterCharacters : "(none)") << endl;
   os << indent << "HaveHeaders: " 
@@ -585,39 +542,48 @@ int vtkDelimitedTextReader::RequestData(
       return 1;
       }
 
+    if (!this->PedigreeIdArrayName)
+      throw vtkstd::runtime_error("You must specify a pedigree id array name");
+
+    // Get the total size of the input file in bytes
+    ifstream file_stream(this->FileName, ios::binary);
+    if(!file_stream.good())
+      {
+      throw vtkstd::runtime_error(
+            "Unable to open input file " + std::string(this->FileName));
+      }
+
+    file_stream.seekg(0, ios::end);
+    //const vtkIdType total_bytes = file_stream.tellg();
+    file_stream.seekg(0, ios::beg);
+
     vtkStdString character_set;
+    vtkTextCodec* transCodec = NULL;
 
     if(this->UnicodeCharacterSet)
       {
       this->UnicodeOutputArrays = true;
       character_set = this->UnicodeCharacterSet;
+      transCodec = vtkTextCodecFactory::CodecForName(this->UnicodeCharacterSet);
       }
     else
       {
       char tstring[2];
       tstring[1] = '\0';
       tstring[0] = this->StringDelimiter;
-      this->SetUnicodeFieldDelimiters(vtkUnicodeString::from_utf8(this->FieldDelimiterCharacters));
+      this->SetUnicodeFieldDelimiters(
+            vtkUnicodeString::from_utf8(this->FieldDelimiterCharacters));
       this->SetUnicodeStringDelimiters(vtkUnicodeString::from_utf8(tstring));
       this->UnicodeOutputArrays = false;
+      transCodec = vtkTextCodecFactory::CodecToHandle(file_stream);
       }
 
-    if (!this->PedigreeIdArrayName)
-      throw vtkstd::runtime_error("You must specify a pedigree id array name");
-
-    // Get the total size of the input file in bytes ... 
-    ifstream file_stream(this->FileName, ios::binary);
-    if(!file_stream.good())
-      throw vtkstd::runtime_error("Unable to open input file " + std::string(this->FileName));
+    if (NULL == transCodec)
+      {
+      // should this use the locale instead??
+      return 1;
+      }
     
-    file_stream.seekg(0, ios::end);
-    const vtkIdType total_bytes = file_stream.tellg();
-    file_stream.seekg(0, ios::beg);
-
-    // Read the file into a buffer ...
-    vtkstd::vector<unsigned char> content(total_bytes);
-    file_stream.read(reinterpret_cast<char*>(&content[0]), total_bytes);
-
     DelimitedTextIterator iterator(
       this->MaxRecords,
       this->UnicodeRecordDelimiters,
@@ -631,67 +597,11 @@ int vtkDelimitedTextReader::RequestData(
       this->UseStringDelimiter,
       output_table);
 
-    if("US-ASCII" == character_set || character_set.empty())
-      {
-      ascii_to_unicode(content.begin(), content.end(), iterator);
-      }
-    else if("UTF-8" == character_set)
-      {
-      if(!vtk_utf8::is_valid(content.begin(), content.end()))
-        throw vtkstd::runtime_error("Detected a byte sequence that isn't valid UTF-8.");
+    vtkTextCodec::OutputIterator& outIter = iterator;
 
-      iterator = vtk_utf8::utf8to32(content.begin(),  content.end(), iterator);
-      iterator.ReachedEndOfInput();
-      }
-    else if("UTF-16" == character_set)
-      {
-      if(content.size() > 1 && static_cast<unsigned char>(content[0]) == 0xfe && static_cast<unsigned char>(content[1]) == 0xff)
-        {
-        utf16_to_unicode(true, content.begin() + 2, content.end(), iterator);
-        }
-      else if(content.size() > 1 && static_cast<unsigned char>(content[0]) == 0xff && static_cast<unsigned char>(content[1]) == 0xfe)
-        {
-        utf16_to_unicode(false, content.begin() + 2, content.end(), iterator);
-        }
-      else
-        {
-        throw vtkstd::runtime_error("Cannot detect UTF-16 endianness.  Try 'UTF-16BE' or 'UTF-16LE' instead.");
-        }
-      }
-    else if("UTF-16BE" == character_set)
-      {
-      if(content.size() > 1 && static_cast<unsigned char>(content[0]) == 0xfe && static_cast<unsigned char>(content[1]) == 0xff)
-        {
-        throw vtkstd::runtime_error("UTF-16BE file should not contain BOM.  Try 'UTF-16' instead.");
-        }
-      else if(content.size() > 1 && static_cast<unsigned char>(content[0]) == 0xff && static_cast<unsigned char>(content[1]) == 0xfe)
-        {
-        throw vtkstd::runtime_error("UTF-16BE file should not contain BOM.  Try 'UTF-16' instead.");
-        }
-      else
-        {
-        utf16_to_unicode(true, content.begin(), content.end(), iterator);
-        }
-      }
-    else if("UTF-16LE" == character_set)
-      {
-      if(content.size() > 1 && static_cast<unsigned char>(content[0]) == 0xfe && static_cast<unsigned char>(content[1]) == 0xff)
-        {
-        throw vtkstd::runtime_error("UTF-16LE file should not contain BOM.  Try 'UTF-16' instead.");
-        }
-      else if(content.size() > 1 && static_cast<unsigned char>(content[0]) == 0xff && static_cast<unsigned char>(content[1]) == 0xfe)
-        {
-        throw vtkstd::runtime_error("UTF-16LE file should not contain BOM.  Try 'UTF-16' instead.");
-        }
-      else
-        {
-        utf16_to_unicode(false, content.begin(), content.end(), iterator);
-        }
-      }
-    else
-      {
-      throw vtkstd::runtime_error("Unknown Unicode character set: " + vtkStdString(this->UnicodeCharacterSet));
-      }
+    transCodec->ToUnicode(file_stream, outIter);
+    iterator.ReachedEndOfInput();
+    transCodec->Delete();
 
     if(this->OutputPedigreeIds)
       {
@@ -718,7 +628,9 @@ int vtkDelimitedTextReader::RequestData(
           }
         else
           {
-          throw vtkstd::runtime_error("Could not find pedigree id array: " + vtkStdString(this->PedigreeIdArrayName));
+          throw vtkstd::runtime_error(
+                "Could not find pedigree id array: " +
+                vtkStdString(this->PedigreeIdArrayName));
           }
       }
     }
