@@ -133,40 +133,10 @@ void vtkMultiCorrelativeAssessFunctor::operator () ( vtkVariantArray* result, vt
   result->SetValue( 0, r );
 }
 
-// ----------------------------------------------------------------------
-int vtkMultiCorrelativeStatistics::FillInputPortInformation( int port, vtkInformation* info )
-{
-  int stat; // = this->Superclass::FillInputPortInformation( port, info );
-  if ( port == INPUT_MODEL )
-    {
-    info->Set( vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkMultiBlockDataSet" );
-    info->Set( vtkAlgorithm::INPUT_IS_OPTIONAL(), 1 );
-    stat = 1;
-    }
-  else
-    {
-    stat = this->Superclass::FillInputPortInformation( port, info );
-    }
-  return stat;
-}
-
-// ----------------------------------------------------------------------
-int vtkMultiCorrelativeStatistics::FillOutputPortInformation( int port, vtkInformation* info )
-{
-  int stat = this->Superclass::FillOutputPortInformation( port, info );
-  if ( port == OUTPUT_MODEL )
-    {
-    info->Set( vtkDataObject::DATA_TYPE_NAME(), "vtkMultiBlockDataSet" );
-    }
-  return stat;
-}
-
 // ----------------------------------------------------------------------
 void vtkMultiCorrelativeStatistics::Aggregate( vtkDataObjectCollection* inMetaColl,
-                                               vtkDataObject* outMetaDO )
+                                               vtkMultiBlockDataSet* outMeta )
 {
-  // Verify that the output model is indeed contained in a multiblock data set
-  vtkMultiBlockDataSet* outMeta = vtkMultiBlockDataSet::SafeDownCast( outMetaDO );
   if ( ! outMeta ) 
     { 
     return; 
@@ -208,7 +178,9 @@ void vtkMultiCorrelativeStatistics::Aggregate( vtkDataObjectCollection* inMetaCo
     // Verify that the current model is indeed contained in a multiblock data set
     inMeta = vtkMultiBlockDataSet::SafeDownCast( inMetaDO );
     if ( ! inMeta ) 
-      { 
+      {
+      outCov->Delete();
+
       return; 
       }
 
@@ -216,12 +188,16 @@ void vtkMultiCorrelativeStatistics::Aggregate( vtkDataObjectCollection* inMetaCo
     inCov = vtkTable::SafeDownCast( inMeta->GetBlock( 0 ) );
     if ( ! inCov )
       {
+      outCov->Delete();
+
       return;
       }
 
     if ( inCov->GetNumberOfRows() != nRow )
       {
       // Models do not match
+      outCov->Delete();
+
       return;
       }
 
@@ -239,6 +215,8 @@ void vtkMultiCorrelativeStatistics::Aggregate( vtkDataObjectCollection* inMetaCo
            || inCov->GetValueByName( r, VTK_MULTICORRELATIVE_KEYCOLUMN2 ) != outCov->GetValueByName( r, VTK_MULTICORRELATIVE_KEYCOLUMN2 ) )
         {
         // Models do not match
+        outCov->Delete();
+
         return;
         }
 
@@ -281,16 +259,18 @@ void vtkMultiCorrelativeStatistics::Aggregate( vtkDataObjectCollection* inMetaCo
 
   // Clean up
   outCov->Delete();
-
-  return;
 }
 
 // ----------------------------------------------------------------------
 void vtkMultiCorrelativeStatistics::Learn( vtkTable* inData, 
                                            vtkTable* vtkNotUsed( inParameters ),
-                                           vtkDataObject* outMetaDO )
+                                           vtkMultiBlockDataSet* outMeta )
 {
-  vtkMultiBlockDataSet* outMeta = vtkMultiBlockDataSet::SafeDownCast( outMetaDO );
+  if ( ! inData )
+    {
+    return;
+    }
+
   if ( ! outMeta )
     {
     return;
@@ -313,17 +293,6 @@ void vtkMultiCorrelativeStatistics::Learn( vtkTable* inData,
   sparseCov->AddColumn( mucov );
   mucov->Delete();
 
-  vtkIdType n = inData->GetNumberOfRows();
-  if ( n <= 0 )
-    {
-    return;
-    }
-
-  if ( inData->GetNumberOfColumns() <= 0 )
-    {
-    return;
-    }
-
   vtksys_stl::set<vtksys_stl::set<vtkStdString> >::const_iterator reqIt;
   vtksys_stl::set<vtkStdString>::const_iterator colIt;
   vtksys_stl::set<vtksys_stl::pair<vtkStdString,vtkDataArray*> > allColumns;
@@ -366,7 +335,9 @@ void vtkMultiCorrelativeStatistics::Learn( vtkTable* inData,
   // This keeps us from computing the same covariance entry multiple times if several requests
   // contain common pairs of columns.
   i = m;
-  // For each request:
+
+  // Loop over requests
+  vtkIdType nRow = inData->GetNumberOfRows();
   for ( reqIt = this->Internals->Requests.begin(); reqIt != this->Internals->Requests.end(); ++ reqIt )
     {
     // For each column in the request:
@@ -414,9 +385,9 @@ void vtkMultiCorrelativeStatistics::Learn( vtkTable* inData,
   mucov->SetNumberOfTuples( 1 + m + colPairs.size() ); // sample size followed by mean (mu) followed by covariance (cov) values
   mucov->FillComponent( 0, 0. );
   double* rv = mucov->GetPointer( 0 );
-  *rv = static_cast<double>( n );
+  *rv = static_cast<double>( nRow );
   ++ rv; // skip Cardinality entry
-  for ( i = 0; i < n; ++ i )
+  for ( i = 0; i < nRow; ++ i )
     {
     // First fetch column values
     for ( vtkIdType j = 0; j < m; ++ j )
@@ -486,15 +457,14 @@ void vtkMultiCorrelativeCholesky( vtksys_stl::vector<double*>& a, vtkIdType m )
 }
 
 // ----------------------------------------------------------------------
-void vtkMultiCorrelativeStatistics::Derive( vtkDataObject* outMetaDO )
+void vtkMultiCorrelativeStatistics::Derive( vtkMultiBlockDataSet* outMeta )
 {
-  vtkMultiBlockDataSet* outMeta = vtkMultiBlockDataSet::SafeDownCast( outMetaDO );
   vtkTable* sparseCov;
   vtkStringArray* ocol1;
   vtkStringArray* ocol2;
   vtkDoubleArray* mucov;
   if (
-    ! outMeta || outMeta->GetNumberOfBlocks() < 1 ||
+    ! outMeta ||
     ! ( sparseCov = vtkTable::SafeDownCast( outMeta->GetBlock( 0 ) ) ) ||
     ! ( ocol1 = vtkStringArray::SafeDownCast( sparseCov->GetColumnByName( VTK_MULTICORRELATIVE_KEYCOLUMN1 ) ) ) ||
     ! ( ocol2 = vtkStringArray::SafeDownCast( sparseCov->GetColumnByName( VTK_MULTICORRELATIVE_KEYCOLUMN2 ) ) ) ||
@@ -618,30 +588,24 @@ void vtkMultiCorrelativeStatistics::Derive( vtkDataObject* outMetaDO )
 
 // ----------------------------------------------------------------------
 void vtkMultiCorrelativeStatistics::Assess( vtkTable* inData, 
-                                            vtkDataObject* inMetaDO, 
+                                            vtkMultiBlockDataSet* inMeta, 
                                             vtkTable* outData )
 {
-  vtkMultiBlockDataSet* inMeta = vtkMultiBlockDataSet::SafeDownCast( inMetaDO );
-  if ( ! inMeta || ! outData )
+  if ( ! inData )
     {
     return;
     }
 
-  if ( inData->GetNumberOfColumns() <= 0 )
-    {
-    return;
-    }
-
-  vtkIdType nsamples = inData->GetNumberOfRows();
-  if ( nsamples <= 0 )
+  if ( ! inMeta )
     {
     return;
     }
 
   // For each request, add a column to the output data related to the likelihood of each input datum wrt the model in the request.
-  // Column names of the metadata and input data are assumed to match (no mapping using AssessNames or AssessParameters is done).
+  // Column names of the metadata and input data are assumed to match.
   // The output columns will be named "this->AssessNames->GetValue(0)(A,B,C)" where "A", "B", and "C" are the column names specified in the
   // per-request metadata tables.
+  vtkIdType nRow = inData->GetNumberOfRows();
   int nb = static_cast<int>( inMeta->GetNumberOfBlocks() );
   AssessFunctor* dfunc = 0;
   for ( int req = 1; req < nb; ++ req )
@@ -690,14 +654,14 @@ void vtkMultiCorrelativeStatistics::Assess( vtkTable* inData,
       vtkDoubleArray* assessValues = vtkDoubleArray::New();
       names[v] = assessColName.str().c_str(); // Storing names to be able to use SetValueByName which is faster than SetValue
       assessValues->SetName( names[v] );
-      assessValues->SetNumberOfTuples( nsamples );
+      assessValues->SetNumberOfTuples( nRow );
       outData->AddColumn( assessValues );
       assessValues->Delete();
       }
 
     // Assess each entry of the column
     vtkVariantArray* assessResult = vtkVariantArray::New();
-    for ( vtkIdType r = 0; r < nsamples; ++ r )
+    for ( vtkIdType r = 0; r < nRow; ++ r )
       {
       (*dfunc)( assessResult, r );
       for ( int v = 0; v < nv; ++ v )
@@ -799,8 +763,6 @@ void vtkMultiCorrelativeStatistics::SelectAssessFunctor( vtkTable* inData,
                                                          vtkStringArray* vtkNotUsed(rowNames), 
                                                          AssessFunctor*& dfunc )
 {
-  (void)inData;
-
   dfunc = 0;
   vtkTable* reqModel = vtkTable::SafeDownCast( inMetaDO );
   if ( ! reqModel )
