@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMPICommunicator.cxx,v $
+  Module:    vtkMPICommunicator.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,7 +17,7 @@
 
 #include "vtkImageData.h"
 #include "vtkMPIController.h"
-#include "vtkMPIGroup.h"
+//#include "vtkMPIGroup.h"
 #include "vtkProcessGroup.h"
 #include "vtkObjectFactory.h"
 #include "vtkRectilinearGrid.h"
@@ -29,22 +29,26 @@
   vtkSmartPointer<type> name = vtkSmartPointer<type>::New()
 
 #include <vtkstd/vector>
+#include <assert.h>
+
+static inline void  vtkMPICommunicatorDebugBarrier(MPI_Comm* handle)
+{
+  // If NDEBUG is defined, do nothing.
+#ifdef NDEBUG
+  (void)handle; // to avoid warning about unused parameter
+#else
+  MPI_Barrier(*handle);
+#endif
+}
 
-vtkCxxRevisionMacro(vtkMPICommunicator, "$Revision: 1.48 $");
 vtkStandardNewMacro(vtkMPICommunicator);
 
 vtkMPICommunicator* vtkMPICommunicator::WorldCommunicator = 0;
 
-//-----------------------------------------------------------------------------
-class vtkMPICommunicatorOpaqueRequest
-{
-public:
-  MPI_Request Handle;
-};
 
-vtkMPICommunicatorOpaqueComm::vtkMPICommunicatorOpaqueComm()
+vtkMPICommunicatorOpaqueComm::vtkMPICommunicatorOpaqueComm(MPI_Comm* handle)
 {
-  this->Handle = 0;
+  this->Handle = handle;
 }
 
 MPI_Comm* vtkMPICommunicatorOpaqueComm::GetHandle()
@@ -56,7 +60,8 @@ MPI_Comm* vtkMPICommunicatorOpaqueComm::GetHandle()
 // This MPI error handler basically does the same thing as the default error
 // handler, but also provides a convenient place to attache a debugger
 // breakpoint.
-void vtkMPICommunicatorMPIErrorHandler(MPI_Comm *comm, int *errorcode, ...)
+extern "C" void vtkMPICommunicatorMPIErrorHandler(MPI_Comm *comm,
+                                                  int *errorcode, ...)
 {
   char ErrorMessage[MPI_MAX_ERROR_STRING];
   int len;
@@ -237,14 +242,12 @@ int vtkMPICommunicatorSendData(const T* data, int length, int sizeoftype,
     }
 }
 //----------------------------------------------------------------------------
-template <class T>
-int vtkMPICommunicatorReceiveData(T* data, int length, int sizeoftype, 
-                                  int remoteProcessId, int tag, 
-                                  MPI_Datatype datatype, MPI_Comm *Handle, 
-                                  int useCopy, int& senderId)
+int vtkMPICommunicator::ReceiveDataInternal(
+  char* data, int length, int sizeoftype,
+  int remoteProcessId, int tag,
+  vtkMPICommunicatorReceiveDataInfo* info,
+  int useCopy, int& senderId)
 {
-  MPI_Status status; 
-  
   if (remoteProcessId == vtkMultiProcessController::ANY_SOURCE)
     {
     remoteProcessId = MPI_ANY_SOURCE;
@@ -255,23 +258,24 @@ int vtkMPICommunicatorReceiveData(T* data, int length, int sizeoftype,
   if (useCopy)
     {
     char* tmpData = vtkMPICommunicator::Allocate(length*sizeoftype);
-    retVal = MPI_Recv(tmpData, length, datatype, remoteProcessId, tag, 
-                      *(Handle), &status);
+    retVal = MPI_Recv(tmpData, length, info->DataType, remoteProcessId, tag, 
+                      *(info->Handle), &(info->Status));
     memcpy(data, tmpData, length*sizeoftype);
     vtkMPICommunicator::Free(tmpData);
     }
   else
     {
-    retVal = MPI_Recv(data, length, datatype, remoteProcessId, tag, 
-                      *(Handle), &status);
+    retVal = MPI_Recv(data, length, info->DataType, remoteProcessId, tag, 
+                      *(info->Handle), &(info->Status));
     }
 
   if (retVal == MPI_SUCCESS)
     {
-    senderId = status.MPI_SOURCE;
+    senderId = info->Status.MPI_SOURCE;
     }
   return retVal;
 }
+
 //----------------------------------------------------------------------------
 template <class T>
 int vtkMPICommunicatorNoBlockSendData(const T* data, int length, 
@@ -325,9 +329,16 @@ int vtkMPICommunicatorAllReduceData(const void *sendBuffer, void *recvBuffer,
 //-----------------------------------------------------------------------------
 // Method for converting an MPI operation to a
 // vtkMultiProcessController::Operation.
+// MPIAPI is defined in the microsoft mpi.h which decorates
+// with the __stdcall decoration.
 static vtkCommunicator::Operation *CurrentOperation;
+#ifdef MPIAPI
+extern "C" void MPIAPI vtkMPICommunicatorUserFunction(void *invec, void *inoutvec,
+                                               int *len, MPI_Datatype *datatype)
+#else
 extern "C" void vtkMPICommunicatorUserFunction(void *invec, void *inoutvec,
                                                int *len, MPI_Datatype *datatype)
+#endif
 {
   int vtkType = vtkMPICommunicatorGetVTKType(*datatype);
   CurrentOperation->Function(invec, inoutvec, *len, vtkType);
@@ -431,19 +442,19 @@ vtkMPICommunicator::~vtkMPICommunicator()
 }
 
 //-----------------------------------------------------------------------------
-#ifndef VTK_LEGACY_REMOVE
-int vtkMPICommunicator::Initialize(vtkMPICommunicator  *mpiComm,
-                                   vtkMPIGroup *deprecatedGroup)
-{
-  VTK_LEGACY_REPLACED_BODY(Initialize(vtkMPICommunicator *, vtkMPIGroup *),
-                           "5.2", Initialize(vtkProcessGroup *));
-
-  VTK_CREATE(vtkProcessGroup, group);
-  deprecatedGroup->CopyInto(group, mpiComm);
-
-  return this->Initialize(group);
-}
-#endif
+//#ifndef VTK_LEGACY_REMOVE
+//int vtkMPICommunicator::Initialize(vtkMPICommunicator  *mpiComm,
+//                                   vtkMPIGroup *deprecatedGroup)
+//{
+//  VTK_LEGACY_REPLACED_BODY(Initialize(vtkMPICommunicator *, vtkMPIGroup *),
+//                           "5.2", Initialize(vtkProcessGroup *));
+//
+//  VTK_CREATE(vtkProcessGroup, group);
+//  deprecatedGroup->CopyInto(group, mpiComm);
+//
+//  return this->Initialize(group);
+//}
+//#endif
 
 //-----------------------------------------------------------------------------
 int vtkMPICommunicator::Initialize(vtkProcessGroup *group)
@@ -551,6 +562,70 @@ int vtkMPICommunicator::Initialize(vtkProcessGroup *group)
   return 1;
 }
 
+//-----------------------------------------------------------------------------
+int vtkMPICommunicator::SplitInitialize(vtkCommunicator *oldcomm,
+                                        int color, int key)
+{
+  if (this->Initialized) return 0;
+
+  vtkMPICommunicator *mpiComm = vtkMPICommunicator::SafeDownCast(oldcomm);
+  if (!mpiComm)
+    {
+    vtkErrorMacro("Split communicator must be an MPI communicator.");
+    return 0;
+    }
+
+  // If mpiComm has been initialized, it is guaranteed (unless the MPI calls
+  // return an error somewhere) to have valid Communicator.
+  if (!mpiComm->Initialized)
+    {
+    vtkWarningMacro("The communicator passed has not been initialized!");
+    return 0;
+    }
+
+  this->KeepHandleOff();
+
+  this->MPIComm->Handle = new MPI_Comm;
+  int err;
+  if (   (err = MPI_Comm_split(*(mpiComm->MPIComm->Handle), color, key,
+                               this->MPIComm->Handle))
+      != MPI_SUCCESS )
+    {
+    delete this->MPIComm->Handle;
+    this->MPIComm->Handle = 0;
+
+    char *msg = vtkMPIController::ErrorString(err);
+    vtkErrorMacro("MPI error occured: " << msg);
+    delete[] msg;
+
+    return 0;
+    }
+
+  this->InitializeNumberOfProcesses();
+  this->Initialized = 1;
+
+  this->Modified();
+
+  return 1;
+}
+
+int vtkMPICommunicator::InitializeExternal (vtkMPICommunicatorOpaqueComm* comm)
+{
+  this->KeepHandleOn();
+
+  if (this->MPIComm->Handle) 
+    {
+    delete this->MPIComm->Handle;
+    }
+  this->MPIComm->Handle = new MPI_Comm (*(comm->GetHandle()));
+  this->InitializeNumberOfProcesses();
+  this->Initialized = 1;
+
+  this->Modified();
+
+  return 1;
+}
+
 //----------------------------------------------------------------------------
 // Start the copying process  
 void vtkMPICommunicator::InitializeCopy(vtkMPICommunicator* source)
@@ -592,12 +667,6 @@ int vtkMPICommunicator::InitializeNumberOfProcesses()
     delete[] msg;
     return 0;
     }
-
-  if (this->MaximumNumberOfProcesses > vtkMultiProcessController::MAX_PROCESSES)
-    {
-    vtkWarningMacro("Maximum of " << vtkMultiProcessController::MAX_PROCESSES);
-    this->MaximumNumberOfProcesses = vtkMultiProcessController::MAX_PROCESSES;
-    }
   
   this->NumberOfProcesses = this->MaximumNumberOfProcesses;
   
@@ -705,13 +774,18 @@ int vtkMPICommunicator::SendVoidArray(const void *data, vtkIdType length,
       break;
     }
 
-  int maxSend = VTK_INT_MAX/sizeOfType;
-  while (length > maxSend)
+  int maxSend = VTK_INT_MAX;
+  while (length >= maxSend)
     {
-    vtkMPICommunicatorSendData(byteData, maxSend, sizeOfType, remoteProcessId,
-                               tag, mpiType, this->MPIComm->Handle,
-                               vtkCommunicator::UseCopy,
-                               this->UseSsend);
+    if (CheckForMPIError(
+        vtkMPICommunicatorSendData(byteData, maxSend, sizeOfType, remoteProcessId,
+          tag, mpiType, this->MPIComm->Handle,
+          vtkCommunicator::UseCopy,
+          this->UseSsend)) == 0)
+      {
+      // Failed to send.
+      return 0;
+      }
     byteData += maxSend*sizeOfType;
     length -= maxSend;
     }
@@ -722,9 +796,16 @@ int vtkMPICommunicator::SendVoidArray(const void *data, vtkIdType length,
 }
 
 //-----------------------------------------------------------------------------
-int vtkMPICommunicator::ReceiveVoidArray(void *data, vtkIdType length, int type,
+inline vtkIdType vtkMPICommunicatorMin(vtkIdType a, vtkIdType b)
+{
+  return (a > b)? b : a;
+}
+
+//-----------------------------------------------------------------------------
+int vtkMPICommunicator::ReceiveVoidArray(void *data, vtkIdType maxlength, int type,
                                          int remoteProcessId, int tag)
 {
+  this->Count = 0;
   char *byteData = static_cast<char *>(data);
   MPI_Datatype mpiType = vtkMPICommunicatorGetMPIType(type);
   int sizeOfType;
@@ -737,24 +818,44 @@ int vtkMPICommunicator::ReceiveVoidArray(void *data, vtkIdType length, int type,
       break;
     }
 
-  int maxReceive = VTK_INT_MAX/sizeOfType;
-  while (length > maxReceive)
+  // maxReceive is the maximum size of data that can be fetched in a one atomic
+  // receive. If when sending the data-length >= maxReceive, then the sender
+  // splits it into multiple packets of at most maxReceive size each.  (Note
+  // that when the sending exactly maxReceive length message, it is split into 2
+  // packets of sizes maxReceive and 0 repectively).
+  int maxReceive = VTK_INT_MAX;
+  vtkMPICommunicatorReceiveDataInfo info;
+  info.Handle = this->MPIComm->Handle;
+  info.DataType = mpiType;
+  while (CheckForMPIError(
+      this->ReceiveDataInternal(byteData,
+        vtkMPICommunicatorMin(maxlength, maxReceive),
+        sizeOfType, remoteProcessId,
+        tag, &info,
+        vtkCommunicator::UseCopy,
+        this->LastSenderId)) != 0)
     {
-    vtkMPICommunicatorReceiveData(byteData, maxReceive, sizeOfType,
-                                  remoteProcessId,
-                                  tag, mpiType, this->MPIComm->Handle,
-                                  vtkCommunicator::UseCopy,
-                                  this->LastSenderId);
     remoteProcessId = this->LastSenderId;
-    byteData += maxReceive*sizeOfType;
-    length -= maxReceive;
+
+    int words_received=0;
+    if (CheckForMPIError(MPI_Get_count(&info.Status, mpiType, &words_received)) == 0)
+      {
+      // Failed.
+      return 0;
+      }
+    this->Count += words_received;
+    byteData += words_received*sizeOfType;
+    maxlength -= words_received;
+    if (words_received < maxReceive)
+      {
+      // words_received in this packet is exactly equal to maxReceive, then it
+      // means that the sender is sending atleast one more packet for this
+      // message. Otherwise, we have received all the packets for this message 
+      // and we no longer need to iterate.
+      return 1;
+      }
     }
-  return CheckForMPIError
-    (vtkMPICommunicatorReceiveData(byteData, length, sizeOfType,
-                                   remoteProcessId,
-                                   tag, mpiType, this->MPIComm->Handle,
-                                   vtkCommunicator::UseCopy,
-                                   this->LastSenderId));
+  return 0;
 }
 
 //----------------------------------------------------------------------------
@@ -974,6 +1075,7 @@ void vtkMPICommunicator::Barrier()
 int vtkMPICommunicator::BroadcastVoidArray(void *data, vtkIdType length,
                                            int type, int root)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   if (!vtkMPICommunicatorCheckSize(type, length)) return 0;
   return CheckForMPIError(MPI_Bcast(data, length,
                                     vtkMPICommunicatorGetMPIType(type),
@@ -986,6 +1088,7 @@ int vtkMPICommunicator::GatherVoidArray(const void *sendBuffer,
                                         vtkIdType length, int type,
                                         int destProcessId)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   int numProc;
   MPI_Comm_size(*this->MPIComm->Handle, &numProc);
   if (!vtkMPICommunicatorCheckSize(type, length*numProc)) return 0;
@@ -1004,6 +1107,7 @@ int vtkMPICommunicator::GatherVVoidArray(const void *sendBuffer,
                                          vtkIdType *offsets, int type,
                                          int destProcessId)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   if (!vtkMPICommunicatorCheckSize(type, sendLength)) return 0;
   MPI_Datatype mpiType = vtkMPICommunicatorGetMPIType(type);
   // We have to jump through several hoops to make sure vtkIdType arrays
@@ -1062,6 +1166,7 @@ int vtkMPICommunicator::ScatterVoidArray(const void *sendBuffer,
                                          vtkIdType length, int type,
                                          int srcProcessId)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   if (!vtkMPICommunicatorCheckSize(type, length)) return 0;
   MPI_Datatype mpiType = vtkMPICommunicatorGetMPIType(type);
   return CheckForMPIError(MPI_Scatter(const_cast<void *>(sendBuffer),
@@ -1078,6 +1183,7 @@ int vtkMPICommunicator::ScatterVVoidArray(const void *sendBuffer,
                                           vtkIdType recvLength, int type,
                                           int srcProcessId)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   if (!vtkMPICommunicatorCheckSize(type, recvLength)) return 0;
   MPI_Datatype mpiType = vtkMPICommunicatorGetMPIType(type);
   // We have to jump through several hoops to make sure vtkIdType arrays
@@ -1135,6 +1241,7 @@ int vtkMPICommunicator::AllGatherVoidArray(const void *sendBuffer,
                                            void *recvBuffer,
                                            vtkIdType length, int type)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   int numProc;
   MPI_Comm_size(*this->MPIComm->Handle, &numProc);
   if (!vtkMPICommunicatorCheckSize(type, length*numProc)) return 0;
@@ -1152,6 +1259,7 @@ int vtkMPICommunicator::AllGatherVVoidArray(const void *sendBuffer,
                                             vtkIdType *recvLengths,
                                             vtkIdType *offsets, int type)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   if (!vtkMPICommunicatorCheckSize(type, sendLength)) return 0;
   MPI_Datatype mpiType = vtkMPICommunicatorGetMPIType(type);
   // We have to jump through several hoops to make sure vtkIdType arrays
@@ -1213,6 +1321,7 @@ int vtkMPICommunicator::ReduceVoidArray(const void *sendBuffer,
                                         vtkIdType length, int type,
                                         int operation, int destProcessId)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   MPI_Op mpiOp;
   switch (operation)
     {
@@ -1242,6 +1351,7 @@ int vtkMPICommunicator::ReduceVoidArray(
                                 vtkIdType length, int type,
                                 Operation *operation, int destProcessId)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   MPI_Op mpiOp;
   MPI_Op_create(vtkMPICommunicatorUserFunction, operation->Commutative(),
                 &mpiOp);
@@ -1266,6 +1376,7 @@ int vtkMPICommunicator::AllReduceVoidArray(const void *sendBuffer,
                                            vtkIdType length, int type,
                                            int operation)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   MPI_Op mpiOp;
   switch (operation)
     {
@@ -1297,6 +1408,7 @@ int vtkMPICommunicator::AllReduceVoidArray(
                                 vtkIdType length, int type,
                                 Operation *operation)
 {
+  vtkMPICommunicatorDebugBarrier(this->MPIComm->Handle);
   MPI_Op mpiOp;
   MPI_Op_create(vtkMPICommunicatorUserFunction, operation->Commutative(),
                 &mpiOp);
@@ -1315,109 +1427,3 @@ int vtkMPICommunicator::AllReduceVoidArray(
 
   return res;
 }
-
-//=============================================================================
-// Deprecated reduce methods.
-#ifndef VTK_LEGACY_REMOVE
-
-int vtkMPICommunicator::ReduceMax(int *data, int *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMax, "5.2", Reduce);
-  return this->Reduce(data, to, size, MAX_OP, root);
-}
-
-int vtkMPICommunicator::ReduceMax(unsigned long *data, unsigned long *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMax, "5.2", Reduce);
-  return this->Reduce(data, to, size, MAX_OP, root);
-}
-
-int vtkMPICommunicator::ReduceMax(float *data, float *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMax, "5.2", Reduce);
-  return this->Reduce(data, to, size, MAX_OP, root);
-}
-
-int vtkMPICommunicator::ReduceMax(double *data, double *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMax, "5.2", Reduce);
-  return this->Reduce(data, to, size, MAX_OP, root);
-}
-
-int vtkMPICommunicator::ReduceMin(int *data, int *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMin, "5.2", Reduce);
-  return this->Reduce(data, to, size, MIN_OP, root);
-}
-
-int vtkMPICommunicator::ReduceMin(unsigned long *data, unsigned long *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMin, "5.2", Reduce);
-  return this->Reduce(data, to, size, MIN_OP, root);
-}
-
-int vtkMPICommunicator::ReduceMin(float *data, float *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMin, "5.2", Reduce);
-  return this->Reduce(data, to, size, MIN_OP, root);
-}
-
-int vtkMPICommunicator::ReduceMin(double *data, double *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceMin, "5.2", Reduce);
-  return this->Reduce(data, to, size, MIN_OP, root);
-}
-
-int vtkMPICommunicator::ReduceSum(int *data, int *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceSum, "5.2", Reduce);
-  return this->Reduce(data, to, size, SUM_OP, root);
-}
-
-int vtkMPICommunicator::ReduceSum(unsigned long *data, unsigned long *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceSum, "5.2", Reduce);
-  return this->Reduce(data, to, size, SUM_OP, root);
-}
-
-int vtkMPICommunicator::ReduceSum(float *data, float *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceSum, "5.2", Reduce);
-  return this->Reduce(data, to, size, SUM_OP, root);
-}
-
-int vtkMPICommunicator::ReduceSum(double *data, double *to,
-                                  int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceSum, "5.2", Reduce);
-  return this->Reduce(data, to, size, SUM_OP, root);
-}
-
-int vtkMPICommunicator::ReduceAnd(bool *data, bool *to, int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceAnd, "5.2", Reduce);
-  return this->Reduce(reinterpret_cast<char *>(data),
-                      reinterpret_cast<char *>(to), size,
-                      LOGICAL_AND_OP, root);
-}
-
-int vtkMPICommunicator::ReduceOr(bool *data, bool *to, int size, int root)
-{
-  VTK_LEGACY_REPLACED_BODY(ReduceOr, "5.2", Reduce);
-  return this->Reduce(reinterpret_cast<char *>(data),
-                      reinterpret_cast<char *>(to), size,
-                      LOGICAL_OR_OP, root);
-}
-
-#endif // VTK_LEGACY_REMOVE
