@@ -28,6 +28,9 @@
 // Needed when we don't use the vtkStandardNewMacro.
 vtkInstantiatorNewMacro(vtkCamera);
 
+vtkCxxSetObjectMacro(vtkCamera, EyeTransformMatrix, vtkMatrix4x4);
+vtkCxxSetObjectMacro(vtkCamera, ModelTransformMatrix, vtkMatrix4x4);
+
 //-----------------------------------------------------------------------------
 class vtkCameraCallbackCommand : public vtkCommand
 {
@@ -51,8 +54,8 @@ protected:
 };
 
 //----------------------------------------------------------------------------
-// Construct camera instance with its focal point at the origin, 
-// and position=(0,0,1). The view up is along the y-axis, 
+// Construct camera instance with its focal point at the origin,
+// and position=(0,0,1). The view up is along the y-axis,
 // view angle is 30 degrees, and the clipping range is (.1,1000).
 vtkCamera::vtkCamera()
 {
@@ -75,6 +78,31 @@ vtkCamera::vtkCamera()
   this->ViewAngle = 30.0;
   this->UseHorizontalViewAngle = 0;
 
+  this->UseOffAxisProjection  = 0;
+
+  this->ScreenBottomLeft[0] = -1.0;
+  this->ScreenBottomLeft[1] = -1.0;
+  this->ScreenBottomLeft[2] = -1.0;
+
+  this->ScreenBottomRight[0] =  1.0;
+  this->ScreenBottomRight[1] = -1.0;
+  this->ScreenBottomRight[2] = -1.0;
+
+  this->ScreenTopRight[0] =  1.0;
+  this->ScreenTopRight[1] =  1.0;
+  this->ScreenTopRight[2] = -1.0;
+
+  this->EyeSeparation = 0.06;
+
+  this->WorldToScreenMatrix = vtkMatrix4x4::New();
+  this->WorldToScreenMatrix->Identity();
+
+  this->EyeTransformMatrix = vtkMatrix4x4::New();
+  this->EyeTransformMatrix->Identity();
+
+  this->ModelTransformMatrix = vtkMatrix4x4::New();
+  this->ModelTransformMatrix->Identity();
+
   this->ClippingRange[0] = 0.01;
   this->ClippingRange[1] = 1000.01;
   this->Thickness = 1000.0;
@@ -88,7 +116,7 @@ vtkCamera::vtkCamera()
 
   this->WindowCenter[0] = 0.0;
   this->WindowCenter[1] = 0.0;
-  
+
   this->ViewShear[0] = 0.0;
   this->ViewShear[1] = 0.0;
   this->ViewShear[2] = 1.0;
@@ -99,6 +127,7 @@ vtkCamera::vtkCamera()
   this->ViewTransform = vtkTransform::New();
   this->ProjectionTransform = vtkPerspectiveTransform::New();
   this->CameraLightTransform = vtkTransform::New();
+  this->ModelViewTransform = vtkTransform::New();
   this->UserTransform = NULL;
   this->UserViewTransform = NULL;
   this->UserViewTransformCallbackCommand = NULL;
@@ -112,10 +141,20 @@ vtkCamera::vtkCamera()
 //----------------------------------------------------------------------------
 vtkCamera::~vtkCamera()
 {
+  this->WorldToScreenMatrix->Delete();
+  this->WorldToScreenMatrix = NULL;
+
+  this->EyeTransformMatrix->Delete();
+  this->EyeTransformMatrix = NULL;
+
+  this->ModelTransformMatrix->Delete();
+  this->ModelTransformMatrix = NULL;
+
   this->Transform->Delete();
   this->ViewTransform->Delete();
   this->ProjectionTransform->Delete();
   this->CameraLightTransform->Delete();
+  this->ModelViewTransform->Delete();
   if (this->UserTransform)
     {
     this->UserTransform->UnRegister(this);
@@ -135,7 +174,7 @@ vtkCamera::~vtkCamera()
 }
 
 //----------------------------------------------------------------------------
-// return the correct type of Camera 
+// return the correct type of Camera
 vtkCamera *vtkCamera::New()
 {
   // First try to create the object from the vtkObjectFactory
@@ -147,7 +186,7 @@ vtkCamera *vtkCamera::New()
 //----------------------------------------------------------------------------
 // The first set of methods deal exclusively with the ViewTransform, which
 // is the only transform which is set up entirely in the camera.  The
-// perspective transform must be set up by the Renderer because the 
+// perspective transform must be set up by the Renderer because the
 // Camera doesn't know the Renderer's aspect ratio.
 //----------------------------------------------------------------------------
 //----------------------------------------------------------------------------
@@ -155,18 +194,18 @@ vtkCamera *vtkCamera::New()
 //----------------------------------------------------------------------------
 void vtkCamera::SetPosition(double x, double y, double z)
 {
-  if (x == this->Position[0] && 
+  if (x == this->Position[0] &&
       y == this->Position[1] &&
       z == this->Position[2])
     {
     return;
     }
-  
+
   this->Position[0] = x;
   this->Position[1] = y;
   this->Position[2] = z;
 
-  vtkDebugMacro(<< " Position set to ( " <<  this->Position[0] << ", " 
+  vtkDebugMacro(<< " Position set to ( " <<  this->Position[0] << ", "
                 << this->Position[1] << ", " << this->Position[2] << ")");
 
   this->ComputeViewTransform();
@@ -180,11 +219,11 @@ void vtkCamera::SetPosition(double x, double y, double z)
 //----------------------------------------------------------------------------
 void vtkCamera::SetUserTransform(vtkHomogeneousTransform *transform)
 {
-  if (transform == this->UserTransform) 
-    { 
-    return; 
+  if (transform == this->UserTransform)
+    {
+    return;
     }
-  if (this->UserTransform) 
+  if (this->UserTransform)
     {
     this->UserTransform->Delete();
     this->UserTransform = NULL;
@@ -200,11 +239,11 @@ void vtkCamera::SetUserTransform(vtkHomogeneousTransform *transform)
 //----------------------------------------------------------------------------
 void vtkCamera::SetUserViewTransform(vtkHomogeneousTransform *transform)
 {
-  if (transform == this->UserViewTransform) 
-    { 
-    return; 
+  if (transform == this->UserViewTransform)
+    {
+    return;
     }
-  if (this->UserViewTransform) 
+  if (this->UserViewTransform)
     {
     this->UserViewTransform->RemoveObserver(
       this->UserViewTransformCallbackCommand);
@@ -233,15 +272,15 @@ void vtkCamera::SetUserViewTransform(vtkHomogeneousTransform *transform)
 //----------------------------------------------------------------------------
 void vtkCamera::SetFocalPoint(double x, double y, double z)
 {
-  if (x == this->FocalPoint[0] && 
-      y == this->FocalPoint[1] && 
+  if (x == this->FocalPoint[0] &&
+      y == this->FocalPoint[1] &&
       z == this->FocalPoint[2])
     {
     return;
     }
 
-  this->FocalPoint[0] = x; 
-  this->FocalPoint[1] = y; 
+  this->FocalPoint[0] = x;
+  this->FocalPoint[1] = y;
   this->FocalPoint[2] = z;
 
   vtkDebugMacro(<< " FocalPoint set to ( " <<  this->FocalPoint[0] << ", " << this->FocalPoint[1] << ", " << this->FocalPoint[2] << ")");
@@ -259,21 +298,21 @@ void vtkCamera::SetViewUp(double x, double y, double z)
 {
   // normalize ViewUp, but do _not_ orthogonalize it by default
   double norm = sqrt(x*x + y*y + z*z);
-  
-  if(norm != 0) 
+
+  if(norm != 0)
     {
-    x /= norm; 
-    y /= norm; 
+    x /= norm;
+    y /= norm;
     z /= norm;
     }
-  else 
+  else
     {
-    x = 0; 
-    y = 1; 
+    x = 0;
+    y = 1;
     z = 0;
     }
-  
-  if (x == this->ViewUp[0] && 
+
+  if (x == this->ViewUp[0] &&
       y == this->ViewUp[1] &&
       z == this->ViewUp[2])
     {
@@ -326,6 +365,152 @@ void vtkCamera::ComputeCameraLightTransform()
   t->Translate(0.0, 0.0, -1.0);
 }
 
+//----------------------------------------------------------------------------
+void vtkCamera::ComputeWorldToScreenMatrix()
+{
+  // Avoid recalculating screen orientation if we don't need to.
+  if(this->WorldToScreenMatrixMTime.GetMTime() < this->GetMTime())
+    {
+    double xAxis[3];
+    double yAxis[3];
+    double zAxis[3];
+
+    for(int i = 0; i < 3; ++i)
+      {
+      xAxis[i] = this->ScreenBottomRight[i] - this->ScreenBottomLeft[i];
+      yAxis[i] = this->ScreenTopRight[i]    - this->ScreenBottomRight[i];
+      }
+
+    vtkMath::Normalize(xAxis);
+    vtkMath::Normalize(yAxis);
+    vtkMath::Cross(xAxis, yAxis, zAxis);
+    vtkMath::Normalize(zAxis);
+
+    // Setting individual elements of the matrix.
+
+    // Make it column major and then invert it to make sure the translation is correct
+    // This is using column major (vectors are copied into the column)
+    // \Note: while the initial element assignments are made in column-major
+    // ordering, the matrix will be inverted, resulting in a row-major
+    // matrix that provides the transformation from World to Screen space.
+    this->WorldToScreenMatrix->SetElement(0, 0, xAxis[0]);
+    this->WorldToScreenMatrix->SetElement(1, 0, xAxis[1]);
+    this->WorldToScreenMatrix->SetElement(2, 0, xAxis[2]);
+
+    this->WorldToScreenMatrix->SetElement(0, 1, yAxis[0]);
+    this->WorldToScreenMatrix->SetElement(1, 1, yAxis[1]);
+    this->WorldToScreenMatrix->SetElement(2, 1, yAxis[2]);
+
+    this->WorldToScreenMatrix->SetElement(0, 2, zAxis[0]);
+    this->WorldToScreenMatrix->SetElement(1, 2, zAxis[1]);
+    this->WorldToScreenMatrix->SetElement(2, 2, zAxis[2]);
+
+    this->WorldToScreenMatrix->SetElement(0, 3, this->ScreenBottomLeft[0]);
+    this->WorldToScreenMatrix->SetElement(1, 3, this->ScreenBottomLeft[1]);
+    this->WorldToScreenMatrix->SetElement(2, 3, this->ScreenBottomLeft[2]);
+
+    this->WorldToScreenMatrix->SetElement(3, 3, 1.0);
+
+    // The reason for doing this as an Invert as the goal here is to put
+    // the translation through the rotation that we've just assigned ie.
+    // the translation has to be put into screen space too.
+    this->WorldToScreenMatrix->Invert();
+
+    this->WorldToScreenMatrixMTime.Modified();
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkCamera::ComputeOffAxisProjectionFrustum()
+{
+  this->ComputeWorldToScreenMatrix();
+
+  // \NOTE: Varibles names reflect naming convention used in
+  // "High Resolution Virtual Reality", in Proc.
+  // SIGGRAPH '92, Computer Graphics, pages 195-202, 1992.
+
+  // OffAxis calculations.
+
+
+  // vtkMatrix::MultiplyPoint expect homogeneous coordinate.
+  double E[4] = {0.0, 0.0, 0.0, 1.0};
+
+  double L[4] = {this->ScreenBottomLeft[0], this->ScreenBottomLeft[1], this->ScreenBottomLeft[2], 1.0};
+  double H[4] = {this->ScreenTopRight[0],   this->ScreenTopRight[1], this->ScreenTopRight[2], 1.0};
+
+  if(this->LeftEye)
+    {
+    E[0] -= this->EyeSeparation / 2.0;
+    }
+  else
+    {
+    E[0] += this->EyeSeparation / 2.0;
+    }
+
+  // First tranform the eye to new position.
+  this->EyeTransformMatrix->MultiplyPoint(E, E);
+
+  // Now transform the eye and screen corner points into the screen
+  // coordinate system.
+  this->WorldToScreenMatrix->MultiplyPoint(E, E);
+  this->WorldToScreenMatrix->MultiplyPoint(H, H);
+  this->WorldToScreenMatrix->MultiplyPoint(L, L);
+
+  double matrix[4][4];
+  double width  = H[0] - L[0];
+  double height = H[1] - L[1];
+
+  // Back and front are not traditional near and far.
+  // Front (aka near)
+  double F = E[2] - this->ClippingRange[1];
+  // Back (aka far)
+  double B = E[2] - this->ClippingRange[0];
+
+  double depth = B - F;
+  matrix[0][0] =  ( 2*E[2] ) / width;
+  matrix[1][0] =  0;
+  matrix[2][0] =  0;
+  matrix[3][0] =  0;
+
+  matrix[0][1] =  0;
+  matrix[1][1] =  ( 2*E[2] )/ height;
+  matrix[2][1] =  0;
+  matrix[3][1] =  0;
+
+  matrix[0][2] = ( H[0]+L[0] - 2*E[0] )/width;
+  matrix[1][2] = ( H[1]+L[1] - 2*E[1] )/height;
+  matrix[2][2] = ( B+F-2*E[2] )/depth;
+  matrix[3][2] = -1;
+
+  matrix[0][3] = ( -E[2]*( H[0]+L[0] ) )/width;
+  matrix[1][3] = ( -E[2]*( H[1]+L[1] ) )/height;
+  matrix[2][3] = B-E[2]- ( B *( B+F - 2*E[2] )/depth );
+  matrix[3][3] = E[2];
+
+  for ( int i=0; i<4; i++ )
+    {
+    for ( int j=0; j<4; j++ )
+      {
+      this->ProjectionTransform->GetMatrix()->SetElement( i,j,  matrix[i][j] ) ;
+      }
+    }
+
+  //  Now move the world into display space.
+  vtkMatrix4x4::Multiply4x4(this->ProjectionTransform->GetMatrix(), this->WorldToScreenMatrix,
+                            this->ProjectionTransform->GetMatrix());
+}
+
+//----------------------------------------------------------------------------
+void vtkCamera::ComputeModelViewMatrix()
+{
+  if(this->ModelViewTransform->GetMTime() < this->ModelTransformMatrix->GetMTime() ||
+     this->ModelViewTransform->GetMTime() < this->ViewTransform->GetMTime())
+    {
+    vtkMatrix4x4::Multiply4x4(this->ViewTransform->GetMatrix(), this->ModelTransformMatrix,
+                              this->ModelViewTransform->GetMatrix());
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkCamera::OrthogonalizeViewUp()
 {
@@ -382,7 +567,7 @@ void vtkCamera::ComputeDistance()
 
   this->Distance = sqrt(dx*dx + dy*dy + dz*dz);
 
-  if (this->Distance < 1e-20) 
+  if (this->Distance < 1e-20)
     {
     this->Distance = 1e-20;
     vtkDebugMacro(<< " Distance is set to minimum.");
@@ -828,7 +1013,7 @@ void vtkCamera::ComputeProjectionTransform(double aspect,
   // adjust Z-buffer range
   this->ProjectionTransform->AdjustZBuffer( -1, +1, nearz, farz );
 
-  if ( this->ParallelProjection)
+  if ( this->ParallelProjection )
     {
     // set up a rectangular parallelipiped
 
@@ -844,6 +1029,10 @@ void vtkCamera::ComputeProjectionTransform(double aspect,
                                       this->ClippingRange[0],
                                       this->ClippingRange[1] );
     }
+  else if(this->UseOffAxisProjection)
+    {
+    this->ComputeOffAxisProjectionFrustum();
+    }
   else
     {
     // set up a perspective frustum
@@ -872,7 +1061,7 @@ void vtkCamera::ComputeProjectionTransform(double aspect,
                                         this->ClippingRange[1] );
     }
 
-  if ( this->Stereo )
+  if ( this->Stereo && !this->UseOffAxisProjection)
     {
     // set up a shear for stereo views
     if ( this->LeftEye )
@@ -1035,18 +1224,18 @@ void vtkCamera::GetFrustumPlanes(double aspect, double planes[24])
     normals[i][1] = 0.0;
     normals[i][2] = 0.0;
     normals[i][3] = 1.0;
-    // if i is even set to -1, if odd set to +1 
+    // if i is even set to -1, if odd set to +1
     normals[i][i/2] = 1 - (i%2)*2;
     }
 
   // get the composite perspective matrix
   vtkMatrix4x4::DeepCopy(
-    *matrix, 
+    *matrix,
     this->GetCompositeProjectionTransformMatrix(aspect,-1,+1));
-  
+
   // transpose the matrix for use with normals
   vtkMatrix4x4::Transpose(*matrix,*matrix);
-  
+
   // transform the normals to world coordinates
   for (i = 0; i < 6; i++)
     {
@@ -1055,7 +1244,7 @@ void vtkCamera::GetFrustumPlanes(double aspect, double planes[24])
     f = 1.0/sqrt(normals[i][0]*normals[i][0] +
                  normals[i][1]*normals[i][1] +
                  normals[i][2]*normals[i][2]);
-    
+
     planes[4*i + 0] = normals[i][0]*f;
     planes[4*i + 1] = normals[i][1]*f;
     planes[4*i + 2] = normals[i][2]*f;
@@ -1085,9 +1274,9 @@ void vtkCamera::ShallowCopy(vtkCamera *source)
 {
   assert("pre: source_exists" && source!=0);
   assert("pre: not_this" && source!=this);
-  
+
   this->PartialCopy(source);
-  
+
   // Shallow copy of matrices:
   if(this->UserTransform!=0)
     {
@@ -1107,7 +1296,7 @@ void vtkCamera::ShallowCopy(vtkCamera *source)
     {
     this->UserViewTransform->Register(this);
     }
-  
+
   if(this->ViewTransform!=0)
     {
     this->ViewTransform->Delete();
@@ -1117,7 +1306,7 @@ void vtkCamera::ShallowCopy(vtkCamera *source)
     {
     this->ViewTransform->Register(this);
     }
-  
+
   if(this->ProjectionTransform!=0)
     {
     this->ProjectionTransform->Delete();
@@ -1127,7 +1316,7 @@ void vtkCamera::ShallowCopy(vtkCamera *source)
     {
     this->ProjectionTransform->Register(this);
     }
-  
+
   if(this->Transform!=0)
     {
     this->Transform->Delete();
@@ -1137,7 +1326,7 @@ void vtkCamera::ShallowCopy(vtkCamera *source)
     {
     this->Transform->Register(this);
     }
-  
+
   if(this->CameraLightTransform!=0)
     {
     this->CameraLightTransform->Delete();
@@ -1148,7 +1337,7 @@ void vtkCamera::ShallowCopy(vtkCamera *source)
     this->CameraLightTransform->Register(this);
     }
 }
-  
+
 // ----------------------------------------------------------------------------
 // Description:
 // Copy the properties of `source' into `this'.
@@ -1159,9 +1348,9 @@ void vtkCamera::DeepCopy(vtkCamera *source)
 {
   assert("pre: source_exists" && source!=0);
   assert("pre: not_this" && source!=this);
-  
+
   this->PartialCopy(source);
-  
+
   // Deep copy the matrices:
   if(source->UserTransform==0)
     {
@@ -1200,7 +1389,7 @@ void vtkCamera::DeepCopy(vtkCamera *source)
       }
      this->UserViewTransform->DeepCopy(source->UserViewTransform);
     }
-  
+
   if(source->ViewTransform==0)
     {
     if(this->ViewTransform!=0)
@@ -1219,7 +1408,7 @@ void vtkCamera::DeepCopy(vtkCamera *source)
       }
      this->ViewTransform->DeepCopy(source->ViewTransform);
     }
-  
+
   if(source->ProjectionTransform==0)
     {
     if(this->ProjectionTransform!=0)
@@ -1238,7 +1427,7 @@ void vtkCamera::DeepCopy(vtkCamera *source)
       }
      this->ProjectionTransform->DeepCopy(source->ProjectionTransform);
     }
-  
+
   if(source->Transform==0)
     {
     if(this->Transform!=0)
@@ -1257,7 +1446,7 @@ void vtkCamera::DeepCopy(vtkCamera *source)
       }
      this->Transform->DeepCopy(source->Transform);
     }
-  
+
   if(source->CameraLightTransform==0)
     {
     if(this->CameraLightTransform!=0)
@@ -1288,9 +1477,9 @@ void vtkCamera::PartialCopy(vtkCamera *source)
 {
   assert("pre: source_exists" && source!=0);
   assert("pre: not_this" && source!=this);
-  
+
   int i;
-  
+
   i=0;
   while(i<2)
     {
@@ -1310,7 +1499,7 @@ void vtkCamera::PartialCopy(vtkCamera *source)
     this->ViewShear[i]=source->ViewShear[i];
     ++i;
     }
-  
+
   this->ViewAngle=source->ViewAngle;
   this->EyeAngle=source->EyeAngle;
   this->ParallelProjection=source->ParallelProjection;
@@ -1377,13 +1566,178 @@ void vtkCamera::PrintSelf(ostream& os, vtkIndent indent)
      << this->ViewUp[1] << ", " << this->ViewUp[2] << ")\n";
   os << indent << "WindowCenter: (" << this->WindowCenter[0] << ", "
      << this->WindowCenter[1] << ")\n";
+
+  os << indent << "UseOffAxisProjection: (" << this->UseOffAxisProjection
+     << ")\n";
+
+  os << indent << "ScreenBottomLeft: (" << this->ScreenBottomLeft[0]
+     << ", " << this->ScreenBottomLeft[1] << ", " << this->ScreenBottomLeft[2]
+     << ")\n";
+
+  os << indent << "ScreenBottomRight: (" << this->ScreenBottomRight[0]
+     << ", " << this->ScreenBottomRight[1] << ", " << this->ScreenBottomRight[2]
+     << ")\n";
+
+  os << indent << "ScreenTopRight: (" << this->ScreenTopRight[0]
+     << ", " << this->ScreenTopRight[1] << ", " << this->ScreenTopRight[2]
+     << ")\n";
+
+  os << indent << "EyeSeparation: (" << this->EyeSeparation
+     << ")\n";
+
+  os << indent << "WorldToScreenMatrix: (" << this->WorldToScreenMatrix << "\n";
+  this->WorldToScreenMatrix->PrintSelf(os, indent.GetNextIndent());
+  os << indent << ")\n";
+
+  os << indent << "EyeTransformMatrix: (" << this->EyeTransformMatrix << "\n";
+  this->EyeTransformMatrix->PrintSelf(os, indent.GetNextIndent());
+  os << indent << ")\n";
+
+  os << indent << "ModelTransformMatrix: (" << this->ModelTransformMatrix << "\n";
+  this->ModelTransformMatrix->PrintSelf(os, indent.GetNextIndent());
+  os << indent << ")\n";
 }
 
-vtkMatrix4x4 *vtkCamera::GetViewTransformMatrix()
-{ return this->ViewTransform->GetMatrix(); }
+//-----------------------------------------------------------------------------
+void vtkCamera::SetEyePosition(double eyePosition[3])
+{
+  if(!eyePosition)
+    {
+    vtkErrorMacro(<< "ERROR: Invalid or NULL eye position\n");
+    return;
+    }
 
+  this->EyeTransformMatrix->SetElement(0, 3, eyePosition[0]);
+  this->EyeTransformMatrix->SetElement(1, 3, eyePosition[1]);
+  this->EyeTransformMatrix->SetElement(2, 3, eyePosition[2]);
+
+  this->Modified();
+}
+
+//-----------------------------------------------------------------------------
+void vtkCamera::GetEyePosition(double eyePosition[3])
+{
+  if(!eyePosition)
+    {
+    vtkErrorMacro(<< "ERROR: Invalid or NULL eye position\n");
+    return;
+    }
+
+  eyePosition[0] = this->EyeTransformMatrix->GetElement(0, 3);
+  eyePosition[1] = this->EyeTransformMatrix->GetElement(1, 3);
+  eyePosition[2] = this->EyeTransformMatrix->GetElement(2, 3);
+}
+
+//-----------------------------------------------------------------------------
+void vtkCamera::GetEyePlaneNormal(double normal[3])
+{
+  if(!normal)
+    {
+    vtkErrorMacro(<< "ERROR: Invalid or NULL normal\n");
+    return;
+    }
+
+  // Homogeneous normal.
+  double localNormal[4];
+
+  // Get the normal from the screen orientation.
+  localNormal[0] = this->WorldToScreenMatrix->GetElement(2, 0);
+  localNormal[1] = this->WorldToScreenMatrix->GetElement(2, 1);
+  localNormal[2] = this->WorldToScreenMatrix->GetElement(2, 2);
+  localNormal[3] = 0.0;
+
+  // Just to be sure.
+  vtkMath::Normalize(localNormal);
+
+  normal[0] = localNormal[0];
+  normal[1] = localNormal[1];
+  normal[2] = localNormal[2];
+}
+
+//-----------------------------------------------------------------------------
+vtkMatrix4x4 *vtkCamera::GetModelViewTransformMatrix()
+{
+  this->ComputeModelViewMatrix();
+
+  return this->ModelViewTransform->GetMatrix();
+}
+
+//-----------------------------------------------------------------------------
+vtkTransform *vtkCamera::GetModelViewTransformObject()
+{
+  this->ComputeModelViewMatrix();
+
+  return this->ModelViewTransform;
+}
+
+//-----------------------------------------------------------------------------
+vtkMatrix4x4 *vtkCamera::GetViewTransformMatrix()
+{
+  return this->GetModelViewTransformMatrix();
+}
+
+//-----------------------------------------------------------------------------
+vtkTransform *vtkCamera::GetViewTransformObject()
+{
+  return this->GetModelViewTransformObject();
+}
+
+//-----------------------------------------------------------------------------
 double *vtkCamera::GetOrientation()
 { return this->ViewTransform->GetOrientation(); };
 
+//-----------------------------------------------------------------------------
 double *vtkCamera::GetOrientationWXYZ()
 { return this->ViewTransform->GetOrientationWXYZ(); };
+
+// ----------------------------------------------------------------------------
+void vtkCamera::SetEyeTransformMatrix(const double elements[16])
+{
+  this->EyeTransformMatrix->Element[0][0] = elements[0];
+  this->EyeTransformMatrix->Element[0][1] = elements[1];
+  this->EyeTransformMatrix->Element[0][2] = elements[2];
+  this->EyeTransformMatrix->Element[0][3] = elements[3];
+
+  this->EyeTransformMatrix->Element[1][0] = elements[4];
+  this->EyeTransformMatrix->Element[1][1] = elements[5];
+  this->EyeTransformMatrix->Element[1][2] = elements[6];
+  this->EyeTransformMatrix->Element[1][3] = elements[7];
+
+  this->EyeTransformMatrix->Element[2][0] = elements[8];
+  this->EyeTransformMatrix->Element[2][1] = elements[9];
+  this->EyeTransformMatrix->Element[2][2] = elements[10];
+  this->EyeTransformMatrix->Element[2][3] = elements[11];
+
+  this->EyeTransformMatrix->Element[3][0] = elements[12];
+  this->EyeTransformMatrix->Element[3][1] = elements[13];
+  this->EyeTransformMatrix->Element[3][2] = elements[14];
+  this->EyeTransformMatrix->Element[3][3] = elements[15];
+
+  this->Modified();
+}
+
+// ----------------------------------------------------------------------------
+void vtkCamera::SetModelTransformMatrix(const double elements[16])
+{
+  this->ModelTransformMatrix->Element[0][0] = elements[0];
+  this->ModelTransformMatrix->Element[0][1] = elements[1];
+  this->ModelTransformMatrix->Element[0][2] = elements[2];
+  this->ModelTransformMatrix->Element[0][3] = elements[3];
+
+  this->ModelTransformMatrix->Element[1][0] = elements[4];
+  this->ModelTransformMatrix->Element[1][1] = elements[5];
+  this->ModelTransformMatrix->Element[1][2] = elements[6];
+  this->ModelTransformMatrix->Element[1][3] = elements[7];
+
+  this->ModelTransformMatrix->Element[2][0] = elements[8];
+  this->ModelTransformMatrix->Element[2][1] = elements[9];
+  this->ModelTransformMatrix->Element[2][2] = elements[10];
+  this->ModelTransformMatrix->Element[2][3] = elements[11];
+
+  this->ModelTransformMatrix->Element[3][0] = elements[12];
+  this->ModelTransformMatrix->Element[3][1] = elements[13];
+  this->ModelTransformMatrix->Element[3][2] = elements[14];
+  this->ModelTransformMatrix->Element[3][3] = elements[15];
+
+  this->Modified();
+}
