@@ -35,7 +35,7 @@
 #include "vtkAbstractVolumeMapper.h"
 #include "vtkVolumeProperty.h"
 #include "vtkLODProp3D.h"
-#include "vtkImageActor.h"
+#include "vtkImageMapper3D.h"
 #include "vtkRenderer.h"
 #include "vtkCamera.h"
 #include "vtkAbstractCellLocator.h"
@@ -243,8 +243,7 @@ double vtkCellPicker::IntersectWithLine(double p1[3], double p2[3],
 { 
   vtkMapper *mapper = 0;
   vtkAbstractVolumeMapper *volumeMapper = 0;
-  vtkImageActor *imageActor = 0;
-  vtkPlaneCollection *planes = 0;
+  vtkImageMapper3D *imageMapper = 0;
 
   double tMin = VTK_DOUBLE_MAX;
   double t1 = 0.0;
@@ -253,18 +252,10 @@ double vtkCellPicker::IntersectWithLine(double p1[3], double p2[3],
   // Clip the ray with the mapper's ClippingPlanes and adjust t1, t2.
   // This limits the pick search to the inside of the clipped region.
   int clippingPlaneId = -1;
-  if (m && (planes = m->GetClippingPlanes())
-      && (planes->GetNumberOfItems() > 0))
+  if (m && !this->ClipLineWithPlanes(m, this->Transform->GetMatrix(),
+                                     p1, p2, t1, t2, clippingPlaneId))
     {
-    // This is a bit ugly: need to transform back to world coordinates
-    double q1[3], q2[3];
-    this->Transform->TransformPoint(p1, q1);
-    this->Transform->TransformPoint(p2, q2);
-
-    if (!this->ClipLineWithPlanes(planes, q1, q2, t1, t2, clippingPlaneId))
-      {
-      return VTK_DOUBLE_MAX;
-      }
+    return VTK_DOUBLE_MAX;
     }
 
   // Initialize the pick position to the frontmost clipping plane
@@ -273,22 +264,22 @@ double vtkCellPicker::IntersectWithLine(double p1[3], double p2[3],
     tMin = t1;
     }
 
-  // Actor
-  else if ( (mapper = vtkMapper::SafeDownCast(m)) )
-    {
-    tMin = this->IntersectActorWithLine(p1, p2, t1, t2, tol, prop, mapper);
-    }
-
   // Volume
   else if ( (volumeMapper = vtkAbstractVolumeMapper::SafeDownCast(m)) )
     {
     tMin = this->IntersectVolumeWithLine(p1, p2, t1, t2, prop, volumeMapper);
     }
 
-  // ImageActor
-  else if ( (imageActor = vtkImageActor::SafeDownCast(prop)) )
+  // Image
+  else if ( (imageMapper = vtkImageMapper3D::SafeDownCast(m)) )
     {
-    tMin = this->IntersectImageActorWithLine(p1, p2, t1, t2, imageActor);
+    tMin = this->IntersectImageWithLine(p1, p2, t1, t2, prop, imageMapper);
+    }
+
+  // Actor
+  else if ( (mapper = vtkMapper::SafeDownCast(m)) )
+    {
+    tMin = this->IntersectActorWithLine(p1, p2, t1, t2, tol, prop, mapper);
     }
 
   // Unidentified Prop3D
@@ -312,39 +303,19 @@ double vtkCellPicker::IntersectWithLine(double p1[3], double p2[3],
       this->MapperPosition[1] = p1[1]*(1.0-t1) + p2[1]*t1;
       this->MapperPosition[2] = p1[2]*(1.0-t1) + p2[2]*t1;
 
-      // Use the normal from the plane: it is in world coordinates
-      planes->GetItem(clippingPlaneId)->GetNormal(this->PickNormal);
-      // We want the "out" direction
-      this->PickNormal[0] = -this->PickNormal[0];
-      this->PickNormal[1] = -this->PickNormal[1];
-      this->PickNormal[2] = -this->PickNormal[2];
-
-      // This code is a little crazy: transforming a normal involves
-      // matrix inversion and transposal, but since the normal
-      // is to be transformed from world -> mapper coords, only the
-      // transpose is needed.
-      double hvec[4];
-      hvec[0] = this->PickNormal[0];
-      hvec[1] = this->PickNormal[1];
-      hvec[2] = this->PickNormal[2];
-      hvec[3] = 0.0;
-      double matrix[16];
-      vtkMatrix4x4::DeepCopy(matrix, this->Transform->GetMatrix());
-      vtkMatrix4x4::Transpose(matrix, matrix);
-      vtkMatrix4x4::MultiplyPoint(matrix, hvec, hvec);
-      this->MapperNormal[0] = hvec[0];
-      this->MapperNormal[1] = hvec[1];
-      this->MapperNormal[2] = hvec[2];
-      }
-    else
-      {
-      // The pick position isn't on a clipping plane, so
-      // use the normal generated from the mapper's input data.
-      this->Transform->TransformNormal(this->MapperNormal, this->PickNormal);
+      double plane[4];
+      m->GetClippingPlaneInDataCoords(
+        this->Transform->GetMatrix(), clippingPlaneId, plane);
+      vtkMath::Normalize(plane);
+      // Want normal outward from the planes, not inward
+      this->MapperNormal[0] = -plane[0];
+      this->MapperNormal[1] = -plane[1];
+      this->MapperNormal[2] = -plane[2];
       }
 
     // The position comes from the data, so put it into world coordinates
     this->Transform->TransformPoint(this->MapperPosition, this->PickPosition);
+    this->Transform->TransformNormal(this->MapperNormal, this->PickNormal);
     }
 
   return tMin;
@@ -778,7 +749,7 @@ double vtkCellPicker::IntersectVolumeWithLine(const double p1[3],
         if (x[j] < extent[2*j]) { x[j] = extent[2*j]; }
         else if (x[j] > extent[2*j + 1]) { x[j] = extent[2*j+1]; }
 
-        xi[j] = int(floor(x[j]));
+        xi[j] = vtkMath::Floor(x[j]);
         pcoords[j] = x[j] - xi[j];
         }
 
@@ -805,16 +776,16 @@ double vtkCellPicker::IntersectVolumeWithLine(const double p1[3],
           double lastX = x1[k]*(1.0 - lastT) + x2[k]*lastT;
 
           // Increment to next slice boundary along dimension "k",
-          // including a tolerance value for stabilityin cases
+          // including a tolerance value for stability in cases
           // where lastX is just less than an integer value.
           double nextX = 0;
           if (x2[k] > x1[k])
             {
-            nextX = floor(lastX + VTKCELLPICKER_VOXEL_TOL) + 1;
+            nextX = vtkMath::Floor(lastX + VTKCELLPICKER_VOXEL_TOL) + 1;
             }
           else
             {
-            nextX = ceil(lastX - VTKCELLPICKER_VOXEL_TOL) - 1;
+            nextX = vtkMath::Ceil(lastX - VTKCELLPICKER_VOXEL_TOL) - 1;
             }
 
           // Compute the "t" value for this slice boundary
@@ -840,7 +811,7 @@ double vtkCellPicker::IntersectVolumeWithLine(const double p1[3],
           x[j] = x1[j]*(1.0 - t) + x2[j]*t;
           if (x[j] < extent[2*j]) { x[j] = extent[2*j]; }
           else if (x[j] > extent[2*j + 1]) { x[j] = extent[2*j+1]; }
-          xi[j] = int(floor(x[j]));
+          xi[j] = vtkMath::Floor(x[j]);
           pcoords[j] = x[j] - xi[j];
           }
         }
@@ -916,19 +887,48 @@ double vtkCellPicker::IntersectVolumeWithLine(const double p1[3],
 }
 
 //----------------------------------------------------------------------------
-double vtkCellPicker::IntersectImageActorWithLine(const double p1[3],
-                                                    const double p2[3],
-                                                    double t1, double t2,
-                                                    vtkImageActor *imageActor)
+double vtkCellPicker::IntersectImageWithLine(const double p1[3],
+                                             const double p2[3],
+                                             double t1, double t2,
+                                             vtkProp3D *prop,
+                                             vtkImageMapper3D *imageMapper)
 {
-  // Convert ray to structured coordinates
-  vtkImageData *data = imageActor->GetInput();
+  // Get the image information
+  vtkImageData *data = imageMapper->GetInput();
   double spacing[3], origin[3];
   int extent[6];
   data->GetSpacing(spacing);
   data->GetOrigin(origin);
   data->GetExtent(extent);
 
+  // Get the plane equation for the slice
+  double normal[4];
+  imageMapper->GetSlicePlaneInDataCoords(prop->GetMatrix(), normal);
+
+  // Point the normal towards camera
+  if (normal[0]*(p1[0] - p2[0]) +
+      normal[1]*(p1[1] - p2[1]) +
+      normal[2]*(p1[2] - p2[2]) < 0)
+    {
+    normal[0] = -normal[0];
+    normal[1] = -normal[1];
+    normal[2] = -normal[2];
+    normal[3] = -normal[3];
+    }
+
+  // And finally convert normal to structured coords
+  double xnormal[4];
+  xnormal[0] = normal[0]*spacing[0];
+  xnormal[1] = normal[1]*spacing[1];
+  xnormal[2] = normal[2]*spacing[2];
+  xnormal[3] = normal[3] + vtkMath::Dot(origin, normal);
+  double l = vtkMath::Norm(xnormal);
+  xnormal[0] /= l;
+  xnormal[1] /= l;
+  xnormal[2] /= l;
+  xnormal[3] /= l;
+
+  // Also convert ray to structured coords
   double x1[3], x2[3];
   for (int i = 0; i < 3; i++)
     {
@@ -936,16 +936,48 @@ double vtkCellPicker::IntersectImageActorWithLine(const double p1[3],
     x2[i] = (p2[i] - origin[i])/spacing[i];
     }
 
+  // Get the bounds to discover any cropping that has been applied
+  double bounds[6];
+  imageMapper->GetBounds(bounds);
+
+  int croppedExtent[6];
+  for (int k = 0; k < 6; k++)
+    {
+    double d = (bounds[k] - origin[k>>1])/spacing[k>>1];
+    if (d > 0)
+      {
+      croppedExtent[k] = static_cast<int>(d + VTKCELLPICKER_VOXEL_TOL);
+      }
+    else
+      {
+      croppedExtent[k] = static_cast<int>(d - VTKCELLPICKER_VOXEL_TOL);
+      }
+    }
+
   // Clip the ray with the extent
-  int planeId, displayExtent[6];
+  int planeId;
   double tMin, tMax;
-  imageActor->GetDisplayExtent(displayExtent);
-  if (!this->ClipLineWithExtent(displayExtent, x1, x2, tMin, tMax, planeId)
+  if (!this->ClipLineWithExtent(croppedExtent, x1, x2, tMin, tMax, planeId)
       || tMin < t1 || tMin > t2)
     {
     return VTK_DOUBLE_MAX;
     }
 
+  if (tMin != tMax)
+    {
+    // Intersect the ray with the slice plane
+    double w1 = vtkMath::Dot(x1, xnormal) + xnormal[3];
+    double w2 = vtkMath::Dot(x2, xnormal) + xnormal[3];
+    if (w1*w2 > VTKCELLPICKER_VOXEL_TOL)
+      {
+      return VTK_DOUBLE_MAX;
+      }
+    if (w1*w2 < 0)
+      {
+      tMin = w1/(w2 - w1);
+      }
+    }
+
   if (tMin < this->GlobalTMin)
     {
     this->ResetPickInfo();
@@ -958,13 +990,13 @@ double vtkCellPicker::IntersectImageActorWithLine(const double p1[3],
       {
       x[j] = x1[j]*(1.0 - tMin) + x2[j]*tMin;
       // Avoid out-of-bounds due to roundoff error
-      if (x[j] < displayExtent[2*j])
+      if (x[j] < croppedExtent[2*j])
         {
-        x[j] = displayExtent[2*j];
+        x[j] = croppedExtent[2*j];
         } 
-      else if (x[j] > displayExtent[2*j+1])
+      else if (x[j] > croppedExtent[2*j+1])
         {
-        x[j] = displayExtent[2*j + 1];
+        x[j] = croppedExtent[2*j + 1];
         }
       }
 
@@ -975,10 +1007,9 @@ double vtkCellPicker::IntersectImageActorWithLine(const double p1[3],
     this->MapperPosition[2] = origin[2] + x[2]*spacing[2];
 
     // Set the normal in mapper coordinates
-    this->MapperNormal[0] = 0.0;
-    this->MapperNormal[1] = 0.0;
-    this->MapperNormal[2] = 0.0;
-    this->MapperNormal[planeId/2] = 2.0*(planeId%2) - 1.0;
+    this->MapperNormal[0] = normal[0];
+    this->MapperNormal[1] = normal[1];
+    this->MapperNormal[2] = normal[2];
     }
 
   return tMin;
@@ -1004,24 +1035,24 @@ double vtkCellPicker::IntersectProp3DWithLine(const double *, const double *,
 // values between 0 and 1.  The index of the frontmost intersected plane is
 // returned in planeId.
 
-int vtkCellPicker::ClipLineWithPlanes(vtkPlaneCollection *planes,
-                                        const double p1[3], const double p2[3],
-                                        double &t1, double &t2, int& planeId)
+int vtkCellPicker::ClipLineWithPlanes(vtkAbstractMapper3D *mapper,
+                                      vtkMatrix4x4 *mat,
+                                      const double p1[3], const double p2[3],
+                                      double &t1, double &t2, int& planeId)
 {
   // The minPlaneId is the index of the plane that t1 lies on
   planeId = -1;
   t1 = 0.0;
   t2 = 1.0;
 
-  vtkCollectionSimpleIterator iter;
-  planes->InitTraversal(iter);
-  vtkPlane *plane;
-  for (int i = 0; (plane = planes->GetNextPlane(iter)); i++)
+  double plane[4];
+  int numClipPlanes = mapper->GetNumberOfClippingPlanes();
+  for (int i = 0; i < numClipPlanes; i++)
     {
-    // This uses EvaluateFunction instead of FunctionValue because,
-    // like the mapper, we want to ignore any transform on the planes.
-    double d1 = plane->EvaluateFunction(const_cast<double *>(p1));
-    double d2 = plane->EvaluateFunction(const_cast<double *>(p2));
+    mapper->GetClippingPlaneInDataCoords(mat, i, plane);
+
+    double d1 = plane[0]*p1[0] + plane[1]*p1[1] + plane[2]*p1[2] + plane[3];
+    double d2 = plane[0]*p2[0] + plane[1]*p2[1] + plane[2]*p2[2] + plane[3];
 
     // If both distances are negative, both points are outside
     if (d1 < 0 && d2 < 0)
@@ -1079,8 +1110,8 @@ int vtkCellPicker::ClipLineWithPlanes(vtkPlaneCollection *planes,
 // xmin, xmax, ymin, ymax, zmin, zmax.
 
 int vtkCellPicker::ClipLineWithExtent(const int extent[6],
-                                        const double x1[3], const double x2[3],
-                                        double &t1, double &t2, int &planeId)
+                                      const double x1[3], const double x2[3],
+                                      double &t1, double &t2, int &planeId)
 {
   double bounds[6];
   bounds[0] = extent[0]; bounds[1] = extent[1]; bounds[2] = extent[2];
@@ -1349,7 +1380,7 @@ void vtkCellPicker::SetImageDataPickInfo(const double x[3],
     if (xj < extent[2*j]) { xj = extent[2*j]; }
     if (xj > extent[2*j+1]) { xj = extent[2*j+1]; }
 
-    this->CellIJK[j] = int(floor(xj));
+    this->CellIJK[j] = vtkMath::Floor(xj);
     this->PCoords[j] = xj - this->CellIJK[j];
     // Keep the cell in-bounds if it is on the edge
     if (this->CellIJK[j] == extent[2*j+1] &&
