@@ -1,7 +1,7 @@
 /*=========================================================================
 
 Program:   Visualization Toolkit
-Module:    $RCSfile: vtkCocoaRenderWindow.h,v $
+Module:    vtkCocoaRenderWindow.h
 
 Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
 All rights reserved.
@@ -25,8 +25,8 @@ PURPOSE.  See the above copyright notice for more information.
 // Cocoa APIs. This class's default behaviour is to create an NSWindow and
 // a vtkCocoaGLView which are used together to draw all vtk stuff into.
 // If you already have an NSWindow and vtkCocoaGLView and you want this
-// class to use them you must call both SetWindowId() and SetDisplayId()
-// early on (before WindowInitialize() is executed).
+// class to use them you must call both SetRootWindow() and SetWindowId(),
+// respectively, early on (before WindowInitialize() is executed).
 //
 // .SECTION See Also
 // vtkOpenGLRenderWindow vtkCocoaGLView
@@ -45,7 +45,7 @@ class VTK_RENDERING_EXPORT vtkCocoaRenderWindow : public vtkOpenGLRenderWindow
 {
 public:
   static vtkCocoaRenderWindow *New();
-  vtkTypeRevisionMacro(vtkCocoaRenderWindow,vtkOpenGLRenderWindow);
+  vtkTypeMacro(vtkCocoaRenderWindow,vtkOpenGLRenderWindow);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -69,15 +69,19 @@ public:
   virtual void Initialize();
 
   // Description:
-  // Change the window to fill the entire screen.
+  // Change the window to fill the entire screen.  This is only partially
+  // implemented for the vtkCocoaRenderWindow.  It can only be called
+  // before the window has been created, and it might not work on all
+  // versions of OS X.
   virtual void SetFullScreen(int);
 
   // Description:
-  // Remap the window.
+  // Remap the window.  This is not implemented for the vtkCocoaRenderWindow.
   virtual void WindowRemap();
 
   // Description:
-  // Set the preferred window size to full screen.
+  // Set the preferred window size to full screen.  This is not implemented
+  // for the vtkCocoaRenderWindow.
   virtual void PrefFullScreen();
 
   // Description:
@@ -111,28 +115,33 @@ public:
     {
       vtkWarningMacro("SetNextWindowInfo not implemented (WindowRemap not implemented).");
     }
-  virtual void SetParentId(void *) 
-    {
-      vtkWarningMacro("Method not implemented.");
-    }
-  virtual void* GetGenericParentId()
-    {
-      vtkWarningMacro("Method not implemented.");
-      return 0;
-    }
   virtual void* GetGenericDrawable()
     {
       vtkWarningMacro("Method not implemented.");
       return 0;
     }
-  virtual void SetWindowInfo(char*)
+  virtual void SetDisplayId(void*)
     {
       vtkWarningMacro("Method not implemented.");
     }
-  virtual void SetParentInfo(char*)
+  virtual void *GetGenericDisplayId()
     {
       vtkWarningMacro("Method not implemented.");
+      return 0;
     }
+
+  // Description:
+  // Set this RenderWindow's window id to a pre-existing window.
+  // The paramater is an ASCII string of a decimal number representing
+  // a pointer to the window.
+  virtual void SetWindowInfo(char*);
+
+  // Description:
+  // See the documenation for SetParentId().  This method allows the ParentId
+  // to be set as an ASCII string of a decimal number that is the memory
+  // address of the parent NSView.
+  virtual void SetParentInfo(char*);
+
   void SetNextWindowId(void*)
     {
       vtkWarningMacro("SetNextWindowId not implemented (WindowRemap not implemented).");
@@ -154,6 +163,10 @@ public:
   // Make this windows OpenGL context the current context.
   virtual void MakeCurrent();
   
+  // Description:
+  // Tells if this window is the current OpenGL context for the calling thread.
+  virtual bool IsCurrent();
+  
   // Description:
   // Update this window's OpenGL context, e.g. when the window is resized.
   void UpdateContext();
@@ -202,8 +215,15 @@ public:
   // Description:
   // Hide or Show the mouse cursor, it is nice to be able to hide the
   // default cursor if you want VTK to display a 3D cursor instead.
+  // Set cursor position in window (note that (0,0) is the lower left 
+  // corner).
   virtual void HideCursor();
   virtual void ShowCursor();
+  virtual void SetCursorPosition(int x, int y);
+
+  // Description:
+  // Change the shape of the cursor.
+  virtual void SetCurrentCursor(int);
   
   // Description:
   // Get the WindowCreated flag. It is 1 if this object created an instance
@@ -217,35 +237,50 @@ public:
   virtual void *GetGenericContext()   {return this->GetContextId();}
 
   // Description:
-  // Sets the NSWindow* associated with this vtkRenderWindow. This class' default
-  // behaviour, that is, if you never call this SetDisplayId()/SetWindowId() is
-  // to create an NSWindow and a vtkCocoaGLView (NSView subclass) which are used
-  // together to draw all vtk stuff into. If you already have an NSWindow and
-  // NSView and you want this class to use them you must call both SetWindowId()
-  // and SetDisplayId() early on (before WindowInitialize() is executed). In the
-  // case of Java, you should call only SetDisplayId().
-  virtual void SetWindowId(void *);
+  // Sets the NSWindow* associated with this vtkRenderWindow.
+  // This class' default behaviour, that is, if you never call
+  // SetWindowId()/SetRootWindow() is to create an NSWindow and a
+  // vtkCocoaGLView (NSView subclass) which are used together to draw
+  // all vtk stuff into. If you already have an NSWindow and NSView and
+  // you want this class to use them you must call both SetRootWindow()
+  // and SetWindowId(), respectively, early on (before WindowInitialize()
+  // is executed). In the case of Java, you should call only SetWindowId().
+  virtual void SetRootWindow(void *);
   
   // Description:
   // Returns the NSWindow* associated with this vtkRenderWindow.
-  virtual void *GetWindowId();
-  virtual void *GetGenericWindowId()  {return this->GetWindowId();}
+  virtual void *GetRootWindow();
 
   // Description:
-  // Sets the NSView* associated with this vtkRenderWindow. This class' default
-  // behaviour, that is, if you never call this SetDisplayId()/SetWindowId() is
-  // to create an NSWindow and a vtkCocoaGLView (NSView subclass) which are used
-  // together to draw all vtk stuff into. If you already have an NSWindow and
-  // NSView and you want this class to use them you must call both SetWindowId()
-  // and SetDisplayId() early on (before WindowInitialize() is executed). In the
-  // case of Java, you should call only SetDisplayId().
-  virtual void SetDisplayId(void *);
+  // Sets the NSView* associated with this vtkRenderWindow.
+  // This class' default behaviour, that is, if you never call this
+  // SetWindowId()/SetRootWindow() is to create an NSWindow and a
+  // vtkCocoaGLView (NSView subclass) which are used together to draw all
+  // vtk stuff into. If you already have an NSWindow and NSView and you
+  // want this class to use them you must call both SetRootWindow()
+  // and SetWindowId(), respectively, early on (before WindowInitialize()
+  // is executed). In the case of Java, you should call only SetWindowId().
+  virtual void SetWindowId(void *);
 
   // Description:
   // Returns the NSView* associated with this vtkRenderWindow.
-  virtual void *GetDisplayId();
-  virtual void *GetGenericDisplayId() {return this->GetDisplayId();}
+  virtual void *GetWindowId();
+  virtual void *GetGenericWindowId() {return this->GetWindowId();}
   
+  // Description:
+  // Set the NSView* for the vtkRenderWindow to be parented within.  The
+  // Position and Size of the RenderWindow will set the rectangle of the
+  // NSView that the vtkRenderWindow will create within this parent.
+  // If you set the WindowId, then this ParentId will be ignored.
+  virtual void SetParentId(void *nsview);
+
+  // Description:
+  // Get the parent NSView* for this vtkRenderWindow.  This method will
+  // return "NULL" if the parent was not set with SetParentId() or 
+  // SetParentInfo().
+  virtual void *GetParentId();
+  virtual void *GetGenericParentId() { return this->GetParentId(); }
+
   // Description:
   // Returns the scaling factor for 'resolution independence', to convert
   // between points and pixels.
@@ -291,7 +326,7 @@ private:
   // Important: this class cannot contain Objective-C instance
   // variables for 2 reasons:
   // 1) C++ files include this header
-  // 2) because of garbage collection
+  // 2) because of garbage collection (the GC scanner does not scan objects create by C++'s new)
   // Instead, use the CocoaManager dictionary to keep a collection
   // of what would otherwise be Objective-C instance variables.
   void     *CocoaManager; // Really an NSMutableDictionary*
