@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkRenderWindow.cxx,v $
+  Module:    vtkRenderWindow.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -21,17 +21,17 @@
 #include "vtkPainterDeviceAdapter.h"
 #include "vtkRenderWindowInteractor.h"
 #include "vtkRendererCollection.h"
+#include "vtkTimerLog.h"
 #include "vtkTransform.h"
 
-vtkCxxRevisionMacro(vtkRenderWindow, "$Revision: 1.153 $");
 
 //----------------------------------------------------------------------------
 // Needed when we don't use the vtkStandardNewMacro.
 vtkInstantiatorNewMacro(vtkRenderWindow);
 //----------------------------------------------------------------------------
 
-// Construct an instance of  vtkRenderWindow with its screen size 
-// set to 300x300, borders turned on, positioned at (0,0), double 
+// Construct an instance of  vtkRenderWindow with its screen size
+// set to 300x300, borders turned on, positioned at (0,0), double
 // buffering turned on, stereo capable off.
 vtkRenderWindow::vtkRenderWindow()
 {
@@ -68,10 +68,12 @@ vtkRenderWindow::vtkRenderWindow()
   this->Renderers = vtkRendererCollection::New();
   this->NumberOfLayers = 1;
   this->CurrentCursor = VTK_CURSOR_DEFAULT;
-  this->AnaglyphColorSaturation = 0.65;
+  this->AnaglyphColorSaturation = 0.65f;
   this->AnaglyphColorMask[0] = 4;  // red
   this->AnaglyphColorMask[1] = 3;  // cyan
   this->PainterDeviceAdapter = vtkPainterDeviceAdapter::New();
+  this->ReportGraphicErrors=0; // false
+  this->AbortCheckTime = 0.0;
 }
 
 //----------------------------------------------------------------------------
@@ -90,6 +92,18 @@ vtkRenderWindow::~vtkRenderWindow()
     delete [] this->ResultFrame;
     this->ResultFrame = NULL;
     }
+
+  vtkCollectionSimpleIterator rsit;
+  this->Renderers->InitTraversal(rsit);
+  vtkRenderer *aren;
+  while ( (aren = this->Renderers->GetNextRenderer(rsit)) )
+    {
+    if (aren->GetRenderWindow() == this)
+      {
+      vtkErrorMacro("Window destructed with renderer still associated with it!");
+      }
+    }
+
   this->Renderers->Delete();
 
   this->PainterDeviceAdapter->Delete();
@@ -101,7 +115,7 @@ vtkRenderWindow *vtkRenderWindow::New()
 {
   // First try to create the object from the vtkObjectFactory
   vtkObject* ret = vtkGraphicsFactory::CreateInstance("vtkRenderWindow");
-  return (vtkRenderWindow*)ret;
+  return static_cast<vtkRenderWindow *>(ret);
 }
 
 //----------------------------------------------------------------------------
@@ -263,7 +277,7 @@ void vtkRenderWindow::Render()
     }
 
   // CAUTION:
-  // This method uses this->GetSize() and allocates buffers using that size. 
+  // This method uses this->GetSize() and allocates buffers using that size.
   // Remember that GetSize() will returns a size scaled by the TileScale factor.
   // We should use GetActualSize() when we don't want the size to be scaled.
 
@@ -346,9 +360,15 @@ void vtkRenderWindow::Render()
         {
         for (x = 0; x < size[0]; x++)
           {
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
           }
         }
 
@@ -391,9 +411,15 @@ void vtkRenderWindow::Render()
         {
         for (x = 0; x < size[0]; x++)
           {
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
           }
         }
 
@@ -529,9 +555,15 @@ void vtkRenderWindow::DoAARender()
           {
           for (x = 0; x < size[0]; x++)
             {
-            *p1 += (float)*p2; p1++; p2++;
-            *p1 += (float)*p2; p1++; p2++;
-            *p1 += (float)*p2; p1++; p2++;
+            *p1 += static_cast<float>(*p2);
+            p1++;
+            p2++;
+            *p1 += static_cast<float>(*p2);
+            p1++;
+            p2++;
+            *p1 += static_cast<float>(*p2);
+            p1++;
+            p2++;
             }
           }
         delete [] p3;
@@ -633,9 +665,15 @@ void vtkRenderWindow::DoFDRender()
         {
         for (x = 0; x < size[0]; x++)
           {
-          *p1 += (float)*p2; p1++; p2++;
-          *p1 += (float)*p2; p1++; p2++;
-          *p1 += (float)*p2; p1++; p2++;
+          *p1 += static_cast<float>(*p2);
+          p1++;
+          p2++;
+          *p1 += static_cast<float>(*p2);
+          p1++;
+          p2++;
+          *p1 += static_cast<float>(*p2);
+          p1++;
+          p2++;
           }
         }
       delete [] p3;
@@ -737,6 +775,10 @@ void vtkRenderWindow::RemoveRenderer(vtkRenderer *ren)
 {
   // we are its parent
   this->Renderers->RemoveItem(ren);
+  if (ren->GetRenderWindow() == this)
+    {
+    ren->SetRenderWindow(NULL);
+    }
 }
 
 int vtkRenderWindow::HasRenderer(vtkRenderer *ren)
@@ -749,9 +791,14 @@ int vtkRenderWindow::CheckAbortStatus()
 {
   if (!this->InAbortCheck)
     {
-    this->InAbortCheck = 1;
-    this->InvokeEvent(vtkCommand::AbortCheckEvent,NULL);
-    this->InAbortCheck = 0;
+    // Only check for abort at most 5 times per second.
+    if (vtkTimerLog::GetUniversalTime() - this->AbortCheckTime > 0.2)
+      {
+      this->InAbortCheck = 1;
+      this->InvokeEvent(vtkCommand::AbortCheckEvent,NULL);
+      this->InAbortCheck = 0;
+      this->AbortCheckTime = vtkTimerLog::GetUniversalTime();
+      }
     }
   return this->AbortRender;
 }
@@ -812,11 +859,12 @@ void vtkRenderWindow::PrintSelf(ostream& os, vtkIndent indent)
     }
 
   os << indent << "MultiSamples: " << this->MultiSamples << "\n";
-  os << indent << "StencilCapable: " << (this->StencilCapable ? "True" : "False")
-     << endl;
+  os << indent << "StencilCapable: " <<
+    (this->StencilCapable ? "True" : "False") << endl;
+  os << indent << "ReportGraphicErrors: "
+     << (this->ReportGraphicErrors ? "On" : "Off")<< "\n";
 }
 
-
 //----------------------------------------------------------------------------
 // Update the system, if needed, due to stereo rendering. For some stereo
 // methods, subclasses might need to switch some hardware settings here.
@@ -838,6 +886,10 @@ void vtkRenderWindow::StereoUpdate(void)
         break;
       case VTK_STEREO_INTERLACED:
         this->StereoStatus = 1;
+        break;
+      case VTK_STEREO_CHECKERBOARD:
+        this->StereoStatus = 1;
+        break;
       }
     }
   else if ((!this->StereoRender) && this->StereoStatus)
@@ -856,6 +908,9 @@ void vtkRenderWindow::StereoUpdate(void)
       case VTK_STEREO_INTERLACED:
         this->StereoStatus = 0;
         break;
+      case VTK_STEREO_CHECKERBOARD:
+        this->StereoStatus = 0;
+        break;
       }
     }
 }
@@ -874,7 +929,8 @@ void vtkRenderWindow::StereoMidpoint(void)
   if ((this->StereoType == VTK_STEREO_RED_BLUE) ||
       (this->StereoType == VTK_STEREO_INTERLACED) ||
       (this->StereoType == VTK_STEREO_DRESDEN) ||
-      (this->StereoType == VTK_STEREO_ANAGLYPH))
+      (this->StereoType == VTK_STEREO_ANAGLYPH) ||
+      (this->StereoType == VTK_STEREO_CHECKERBOARD))
     {
     int *size;
     // get the size
@@ -1155,6 +1211,53 @@ void vtkRenderWindow::StereoRenderComplete(void)
       delete [] buff;
       }
       break;
+
+    case VTK_STEREO_CHECKERBOARD: {
+      unsigned char *left, *right;
+      unsigned char *sleft, *sright;
+      int *size;
+
+      // get the size
+      size = this->GetSize();
+      // get the data
+      sleft = this->StereoBuffer;
+      sright = this->GetPixelData(0, 0, size[0] - 1, size[1] - 1,
+                                  !this->DoubleBuffer);
+
+      // copy right pixels onto the left pixel buffer
+      for(int y = 0; y < size[1]; y = y + 1) {
+        // set up the pointers
+        // right starts on x = 1 on even scanlines
+        // right starts on x = 0 on odd scanlines
+        if(y % 2) {
+          left = sleft + y * 3 * size[0] + 3;
+          right = sright + y * 3 * size[0] + 3;
+        }
+        else {
+          left = sleft + y * 3 * size[0];
+          right = sright + y * 3 * size[0];
+        }
+
+        // skip every other pixel
+        for(int x = (y + 1) % 2; x < size[0]; x = x + 2) {
+          *left++ = *right++;
+          *left++ = *right++;
+          *left++ = *right++;
+
+          // skip pixel
+          left = left + 3;
+          right = right + 3;
+        }
+      }
+
+      // cleanup
+      this->ResultFrame = sleft;
+
+      this->StereoBuffer = NULL;
+      delete [] sright;
+    }
+      break;
+
     }
 }
 
@@ -1170,6 +1273,12 @@ void vtkRenderWindow::CopyResultFrame(void)
     this->SetPixelData(0,0,size[0]-1,size[1]-1,this->ResultFrame,!this->DoubleBuffer);
     }
 
+  // Just before we swap buffers (in case of double buffering), we fire the
+  // RenderEvent marking that a render call has concluded successfully. We
+  // separate this from EndEvent since some applications may want to put some
+  // more elements on the "draw-buffer" before calling the rendering complete.
+  // This event gives them that opportunity.
+  this->InvokeEvent(vtkCommand::RenderEvent);
   this->Frame();
 }
 
@@ -1221,6 +1330,8 @@ const char *vtkRenderWindow::GetStereoTypeAsString()
       return "DresdenDisplay";
     case VTK_STEREO_ANAGLYPH:
       return "Anaglyph";
+    case VTK_STEREO_CHECKERBOARD:
+      return "Checkerboard";
     default:
       return "";
   }
