@@ -1,7 +1,7 @@
 /*=========================================================================
 
 Program:   Visualization Toolkit
-Module:    $RCSfile: vtkWin32OpenGLRenderWindow.cxx,v $
+Module:    vtkWin32OpenGLRenderWindow.cxx
 
 Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
 All rights reserved.
@@ -32,7 +32,6 @@ PURPOSE.  See the above copyright notice for more information.
 
 #include "vtkOpenGL.h"
 
-vtkCxxRevisionMacro(vtkWin32OpenGLRenderWindow, "$Revision: 1.157 $");
 vtkStandardNewMacro(vtkWin32OpenGLRenderWindow);
 
 #define VTK_MAX_LIGHTS 8
@@ -61,11 +60,11 @@ vtkWin32OpenGLRenderWindow::vtkWin32OpenGLRenderWindow()
 vtkWin32OpenGLRenderWindow::~vtkWin32OpenGLRenderWindow()
 {
   this->Finalize();
-  
+
   vtkRenderer *ren;
-  vtkCollectionSimpleIterator rsit;
-  for (this->Renderers->InitTraversal(rsit); 
-       (ren = this->Renderers->GetNextRenderer(rsit));)
+  vtkCollectionSimpleIterator rit;
+  this->Renderers->InitTraversal(rit);
+  while ( (ren = this->Renderers->GetNextRenderer(rit)) )
     {
     ren->SetRenderWindow(NULL);
     }
@@ -108,7 +107,11 @@ void vtkWin32OpenGLRenderWindow::Clean()
 
     this->CleanUpRenderers();
     
-    if (wglMakeCurrent(NULL, NULL) != TRUE) 
+    // Note: wglMakeCurrent(NULL,NULL) is valid according to the documentation
+    // and works with nVidia and ATI but not with Intel. Passing an existing
+    // device context works in any case.
+    // see VTK Bug 7119.
+    if(wglMakeCurrent(this->DeviceContext,NULL)!=TRUE) 
       {
       vtkErrorMacro("wglMakeCurrent failed in Clean(), error: " << GetLastError());
       }
@@ -200,7 +203,7 @@ int vtkWin32OpenGLRenderWindow::GetEventPending()
   return 0;
 }
 
-
+// ----------------------------------------------------------------------------
 void vtkWin32OpenGLRenderWindow::MakeCurrent()
 {
   // Try to avoid doing anything (for performance).
@@ -250,7 +253,15 @@ void vtkWin32OpenGLRenderWindow::MakeCurrent()
     }
 }
 
+// ----------------------------------------------------------------------------
+// Description:
+// Tells if this window is the current OpenGL context for the calling thread.
+bool vtkWin32OpenGLRenderWindow::IsCurrent()
+{
+  return this->ContextId!=0 && this->ContextId==wglGetCurrentContext();
+}
 
+// ----------------------------------------------------------------------------
 void vtkWin32OpenGLRenderWindow::SetSize(int x, int y)
 {
   static int resizing = 0;
@@ -335,10 +346,14 @@ void vtkWin32OpenGLRenderWindow::Frame(void)
   this->MakeCurrent();
   if (!this->AbortRender && this->DoubleBuffer && this->SwapBuffers)
     {
-    // use global scope to get Win32 API SwapBuffers and not be
-    // confused with this->SwapBuffers
-    ::SwapBuffers(this->DeviceContext);
-    vtkDebugMacro(<< " SwapBuffers\n");
+    // If this check is not enforced, we crash in offscreen rendering
+    if (this->DeviceContext)
+      {
+      // use global scope to get Win32 API SwapBuffers and not be
+      // confused with this->SwapBuffers
+      ::SwapBuffers(this->DeviceContext);
+      vtkDebugMacro(<< " SwapBuffers\n");
+      }
     }
   else
     {
@@ -681,6 +696,13 @@ LRESULT vtkWin32OpenGLRenderWindow::MessageProc(HWND hWnd, UINT message,
     break;
     case WM_ERASEBKGND:
       return TRUE;
+    case WM_SETCURSOR:
+      if (HTCLIENT == LOWORD(lParam))
+        {
+        this->SetCurrentCursor(this->GetCurrentCursor());
+        return TRUE;
+        }
+      break;
     default:
       this->InvokeEvent(vtkCommand::RenderWindowMessageEvent, &message);
       break;
@@ -1464,8 +1486,9 @@ void vtkWin32OpenGLRenderWindow::SetCursorPosition(int x, int y)
     {
     SetCursorPos(point.x, point.y);
     }
-};
+}
 
+//----------------------------------------------------------------------------
 void vtkWin32OpenGLRenderWindow::SetCurrentCursor(int shape)
 {
   if ( this->InvokeEvent(vtkCommand::CursorChangedEvent,&shape) )
