@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  MetaIO
+  Copyright 2000-2010 Insight Software Consortium
 
-  Program:   MetaIO
-  Module:    $RCSfile: metaMesh.cxx,v $
-  Language:  C++
-  Date:      $Date: 2007/06/01 13:00:40 $
-  Version:   $Revision: 1.8 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) Insight Software Consortium. All rights reserved.
-  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #ifdef _MSC_VER
 #pragma warning(disable:4702)
 #pragma warning(disable:4284)
@@ -32,7 +27,7 @@ namespace METAIO_NAMESPACE {
 
 MeshPoint::
 MeshPoint(int dim)
-{ 
+{
   m_Dim = dim;
   m_X = new float[m_Dim];
   for(unsigned int i=0;i<m_Dim;i++)
@@ -43,13 +38,13 @@ MeshPoint(int dim)
 
 MeshPoint::
 ~MeshPoint()
-{ 
+{
   delete []m_X;
 }
 
 MeshCell::
 MeshCell(int dim)
-{ 
+{
   m_Dim = dim;
   m_Id = -1;
   m_PointsId = new int[m_Dim];
@@ -61,10 +56,10 @@ MeshCell(int dim)
 
 MeshCell::
 ~MeshCell()
-{ 
+{
   delete []m_PointsId;
 }
-  
+
 
 //
 // MetaMesh Constructors
@@ -75,7 +70,7 @@ MetaMesh()
 {
   if(META_DEBUG) METAIO_STREAM::cout << "MetaMesh()" << METAIO_STREAM::endl;
   m_NPoints = 0;
- 
+
   for(unsigned int i=0;i<MET_NUM_CELL_TYPES;i++)
     {
     m_CellListArray[i] = NULL;
@@ -90,7 +85,7 @@ MetaMesh(const char *_headerName)
 {
   if(META_DEBUG)  METAIO_STREAM::cout << "MetaMesh()" << METAIO_STREAM::endl;
   m_NPoints = 0;
-  
+
   for(unsigned int i=0;i<MET_NUM_CELL_TYPES;i++)
     {
     m_CellListArray[i] = NULL;
@@ -191,7 +186,7 @@ Clear(void)
   if(META_DEBUG) METAIO_STREAM::cout << "MetaMesh: Clear" << METAIO_STREAM::endl;
   MetaObject::Clear();
   if(META_DEBUG) METAIO_STREAM::cout << "MetaMesh: Clear: m_NPoints" << METAIO_STREAM::endl;
-  
+
   // Delete the list of pointers to points.
   PointListType::iterator it_pnt = m_PointList.begin();
   while(it_pnt != m_PointList.end())
@@ -260,7 +255,7 @@ Clear(void)
   m_PointDataType = MET_FLOAT;
   m_CellDataType = MET_FLOAT;
 }
-        
+
 /** Destroy tube information */
 void MetaMesh::
 M_Destroy(void)
@@ -293,7 +288,7 @@ M_SetupReadFields(void)
   MET_InitReadField(mF, "PointType", MET_STRING, true);
   mF->required = true;
   m_Fields.push_back(mF);
- 
+
   mF = new MET_FieldRecordType;
   MET_InitReadField(mF, "PointDataType", MET_STRING, true);
   mF->required = true;
@@ -317,7 +312,7 @@ M_SetupWriteFields(void)
   MetaObject::M_SetupWriteFields();
 
   MET_FieldRecordType * mF;
-  
+
   char s[255];
   mF = new MET_FieldRecordType;
   MET_TypeToString(m_PointType, s);
@@ -365,7 +360,7 @@ M_SetupWriteFields(void)
     m_Fields.push_back(mF);
   }
 
-  m_NPoints = m_PointList.size();
+  m_NPoints = (int)m_PointList.size();
   mF = new MET_FieldRecordType;
   MET_InitWriteField(mF, "NPoints", MET_INT,m_NPoints);
   m_Fields.push_back(mF);
@@ -390,7 +385,7 @@ M_Read(void)
   }
 
   if(META_DEBUG) METAIO_STREAM::cout << "MetaMesh: M_Read: Parsing Header" << METAIO_STREAM::endl;
- 
+
   MET_FieldRecordType * mF;
 
   unsigned int numberOfCellTypes =0;
@@ -399,7 +394,7 @@ M_Read(void)
   {
     numberOfCellTypes= (int)mF->value[0];
   }
- 
+
   mF = MET_GetFieldRecord("NPoints", &m_Fields);
   if(mF->defined)
   {
@@ -437,24 +432,25 @@ M_Read(void)
     int elementSize;
     MET_SizeOfType(m_PointType, &elementSize);
     int readSize = m_NPoints*(m_NDims)*elementSize+m_NPoints*sizeof(int);
-    
+
     char* _data = new char[readSize];
     m_ReadStream->read((char *)_data, readSize);
 
     int gc = m_ReadStream->gcount();
     if(gc != readSize)
     {
-      METAIO_STREAM::cout << "MetaMesh: m_Read: Points not read completely" 
+      METAIO_STREAM::cout << "MetaMesh: m_Read: Points not read completely"
                           << METAIO_STREAM::endl;
-      METAIO_STREAM::cout << "   ideal = " << readSize 
+      METAIO_STREAM::cout << "   ideal = " << readSize
                           << " : actual = " << gc << METAIO_STREAM::endl;
+      delete [] _data;
       return false;
     }
 
     int i=0;
     int d;
     int td;
-    for(j=0; j<m_NPoints; j++) 
+    for(j=0; j<m_NPoints; j++)
       {
       MeshPoint* pnt = new MeshPoint(m_NDims);
       unsigned int k;
@@ -468,7 +464,7 @@ M_Read(void)
       pnt->m_Id = td;
       i+= sizeof(int);
       delete [] num;
-      
+
       for(d=0; d<m_NDims; d++)
         {
         num = new char[elementSize];
@@ -495,7 +491,7 @@ M_Read(void)
           pnt->m_X[d] = val;
           }
         else if(m_PointType == MET_USHORT)
-          { 
+          {
           unsigned short val = (unsigned short)((unsigned short*)num)[0];
           MET_SwapByteIfSystemMSB(&val,MET_USHORT);
           pnt->m_X[d] = val;
@@ -507,31 +503,31 @@ M_Read(void)
           pnt->m_X[d] = (float)val;
           }
         else if(m_PointType == MET_UINT)
-          { 
+          {
           unsigned int val = (unsigned int)((char*)num)[0];
           MET_SwapByteIfSystemMSB(&val,MET_UINT);
           pnt->m_X[d] = (float)val;
           }
         else if(m_PointType == MET_LONG)
-          { 
+          {
           long val = (long)((long*)num)[0];
           MET_SwapByteIfSystemMSB(&val,MET_LONG);
           pnt->m_X[d] = (float)val;
           }
         else if(m_PointType == MET_ULONG)
-          { 
+          {
           unsigned long val = (unsigned long)((unsigned long*)num)[0];
           MET_SwapByteIfSystemMSB(&val,MET_ULONG);
           pnt->m_X[d] = (float)val;
           }
         else if(m_PointType == MET_FLOAT)
-          { 
+          {
           float val = (float)((float*)num)[0];
           MET_SwapByteIfSystemMSB(&val,MET_FLOAT);
           pnt->m_X[d] = val;
           }
         else if(m_PointType == MET_DOUBLE)
-          { 
+          {
           double val = (double)((double*)num)[0];
           MET_SwapByteIfSystemMSB(&val,MET_DOUBLE);
           pnt->m_X[d] = (float)val;
@@ -544,15 +540,15 @@ M_Read(void)
     }
   else
     {
-    for(j=0; j<m_NPoints; j++) 
+    for(j=0; j<m_NPoints; j++)
       {
       MeshPoint* pnt = new MeshPoint(m_NDims);
-      
+
       float v[10];
       for(int k=0; k<m_NDims+1; k++)
         {
         *m_ReadStream >> v[k];
-        m_ReadStream->get(); 
+        m_ReadStream->get();
         }
 
       int d;
@@ -560,7 +556,7 @@ M_Read(void)
       for(d=0; d<m_NDims; d++)
         {
         pnt->m_X[d] = v[d+1];
-        } 
+        }
       m_PointList.push_back(pnt);
       }
 
@@ -570,7 +566,7 @@ M_Read(void)
       c = m_ReadStream->get();// to avoid unrecognize charactere
       }
     }
-  
+
   // Now reading the cells
   for(unsigned int nCellType=0;nCellType<numberOfCellTypes;nCellType++)
     {
@@ -587,10 +583,10 @@ M_Read(void)
     MET_InitReadField(mF, "Cells", MET_NONE, true);
     mF->terminateRead = true;
     m_Fields.push_back(mF);
-  
+
    if(!MET_Read(*m_ReadStream, & m_Fields))
       {
-      METAIO_STREAM::cout << "MetaObject: Read: MET_Read Failed" 
+      METAIO_STREAM::cout << "MetaObject: Read: MET_Read Failed"
                           << METAIO_STREAM::endl;
       return false;
       }
@@ -602,7 +598,7 @@ M_Read(void)
     }
 
     MET_CellGeometry celltype = MET_VERTEX_CELL;
-  
+
     mF = MET_GetFieldRecord("CellType", &m_Fields);
     if(mF->defined)
       {
@@ -619,24 +615,25 @@ M_Read(void)
     {
       unsigned int totalcellsize = (MET_CellSize[celltype]+1)*m_NCells;
       int readSize = totalcellsize*sizeof(int);
-    
+
       char* _data = new char[readSize];
       m_ReadStream->read((char *)_data, readSize);
 
       int gc = m_ReadStream->gcount();
       if(gc != readSize)
         {
-        METAIO_STREAM::cout << "MetaMesh: m_Read: Cells not read completely" 
+        METAIO_STREAM::cout << "MetaMesh: m_Read: Cells not read completely"
                             << METAIO_STREAM::endl;
         METAIO_STREAM::cout << "   ideal = " << readSize << " : actual = " << gc
                             << METAIO_STREAM::endl;
         return false;
+        delete [] _data;
         }
 
       int i=0;
       int d;
       int td;
-      for(j=0; j<m_NCells; j++) 
+      for(j=0; j<(int)m_NCells; j++)
         {
         int n = MET_CellSize[celltype];
         MeshCell* cell = new MeshCell(n);
@@ -673,12 +670,12 @@ M_Read(void)
       }
     else
       {
-      for(j=0; j<m_NCells; j++) 
-        {     
-        int v;   
+      for(j=0; j<(int)m_NCells; j++)
+        {
+        int v;
         int n = MET_CellSize[celltype];
         MeshCell* cell = new MeshCell(MET_CellSize[celltype]);
-        
+
         *m_ReadStream >> v;
         m_ReadStream->get();
         cell->m_Id = v;
@@ -692,7 +689,7 @@ M_Read(void)
         m_CellListArray[celltype]->push_back(cell);
         }
 
-      
+
       char c = ' ';
       while( (c!='\n') && (!m_ReadStream->eof()))
       {
@@ -701,7 +698,7 @@ M_Read(void)
     }
     }
 
-  long int pos = m_ReadStream->tellg();
+  METAIO_STL::streampos pos = m_ReadStream->tellg();
 
   // Now reading the cell links
   MetaObject::ClearFields();
@@ -718,7 +715,7 @@ M_Read(void)
   MET_InitReadField(mF, "CellLinks", MET_NONE, false);
   mF->terminateRead = true;
   m_Fields.push_back(mF);
-  
+
   if(!MET_Read(*m_ReadStream, & m_Fields,'=',false,false))
     {
     METAIO_STREAM::cout << "MetaObject: Read: MET_Read Failed" << METAIO_STREAM::endl;
@@ -744,25 +741,26 @@ M_Read(void)
   if(m_BinaryData)
     {
     int readSize = totalCellLink*sizeof(int);
-    
+
     char* _data = new char[readSize];
     m_ReadStream->read((char *)_data, readSize);
 
     int gc = m_ReadStream->gcount();
     if(gc != readSize)
       {
-      METAIO_STREAM::cout << "MetaMesh: m_Read: Cell Link not read completely" 
+      METAIO_STREAM::cout << "MetaMesh: m_Read: Cell Link not read completely"
                 << METAIO_STREAM::endl;
       METAIO_STREAM::cout << "   ideal = " << readSize << " : actual = " << gc << METAIO_STREAM::endl;
+      delete [] _data;
       return false;
       }
     int i=0;
     int d;
     int td;
-    for(j=0; j<m_NCellLinks; j++) 
+    for(j=0; j<(int)m_NCellLinks; j++)
       {
       MeshCellLink* link = new MeshCellLink();
-   
+
       unsigned int k;
       char* num = new char[sizeof(int)];
       for(k=0;k<sizeof(int);k++)
@@ -772,7 +770,7 @@ M_Read(void)
       td = (int)((int*)num)[0];
       MET_SwapByteIfSystemMSB(&td,MET_INT);
       link->m_Id = (int)td;
-      
+
       i += sizeof(int);
       for(k=0;k<sizeof(int);k++)
         {
@@ -781,7 +779,7 @@ M_Read(void)
       int n = (int)((int*)num)[0];
       MET_SwapByteIfSystemMSB(&n,MET_INT);
       i += sizeof(int);
-      
+
       for(d=0; d<n; d++)
         {
         for(k=0;k<sizeof(int);k++)
@@ -800,11 +798,11 @@ M_Read(void)
     }
   else
     {
-    for(j=0; j<m_NCellLinks; j++) 
-      {     
-      int v;   
+    for(j=0; j<(int)m_NCellLinks; j++)
+      {
+      int v;
       MeshCellLink* link = new MeshCellLink();
-       
+
       *m_ReadStream >> v;
       m_ReadStream->get();
       link->m_Id = v;
@@ -821,7 +819,7 @@ M_Read(void)
         }
       m_CellLinks.push_back(link);
       }
-      
+
     if(m_NCellLinks > 0)
       {
       char c = ' ';
@@ -854,13 +852,13 @@ M_Read(void)
   MET_InitReadField(mF, "PointData", MET_NONE, false);
   mF->terminateRead = true;
   m_Fields.push_back(mF);
-  
+
   if(!MET_Read(*m_ReadStream, & m_Fields,'=',false,false))
     {
     METAIO_STREAM::cout << "MetaObject: Read: MET_Read Failed" << METAIO_STREAM::endl;
     return false;
     }
- 
+
   mF = MET_GetFieldRecord("NPointData", &m_Fields);
   if(mF->defined)
     {
@@ -880,18 +878,19 @@ M_Read(void)
   unsigned int gc = m_ReadStream->gcount();
   if(gc != pointDataSize)
     {
-    METAIO_STREAM::cout << "MetaMesh: m_Read: PointData not read completely" 
+    METAIO_STREAM::cout << "MetaMesh: m_Read: PointData not read completely"
               << METAIO_STREAM::endl;
     METAIO_STREAM::cout << "   ideal = " << pointDataSize << " : actual = " << gc << METAIO_STREAM::endl;
+    delete [] _data;
     return false;
     }
   int i=0;
   int td;
 
-  for(j=0; j<m_NPointData; j++)  
+  for(j=0; j<(int)m_NPointData; j++)
     {
     MeshDataBase* pd;
-    
+
     unsigned int k;
     char* num = new char[sizeof(int)];
     for(k=0;k<sizeof(int);k++)
@@ -906,7 +905,7 @@ M_Read(void)
 
     int elementSize;
     MET_SizeOfType(m_PointDataType, &elementSize);
-    
+
     num = new char[elementSize];
     for(k=0;k<static_cast<unsigned int>(elementSize);k++)
       {
@@ -917,8 +916,8 @@ M_Read(void)
     if(m_PointDataType == MET_CHAR)
       {
       char val = (char)((char*)num)[0];
-      pd = new MeshData<char>(); 
-      static_cast<MeshData<char>*>(pd)->m_Data = (char)val; 
+      pd = new MeshData<char>();
+      static_cast<MeshData<char>*>(pd)->m_Data = (char)val;
       }
     else if(m_PointDataType == MET_UCHAR)
       {
@@ -934,7 +933,7 @@ M_Read(void)
       static_cast<MeshData<short>*>(pd)->m_Data = (short)val;
       }
     else if(m_PointDataType == MET_USHORT)
-      { 
+      {
       unsigned short val = (unsigned short)((unsigned short*)num)[0];
       pd = new MeshData<unsigned short>();
       MET_SwapByteIfSystemMSB(&val,MET_USHORT);
@@ -948,42 +947,42 @@ M_Read(void)
       static_cast<MeshData<int>*>(pd)->m_Data = (int)val;
       }
     else if(m_PointDataType == MET_UINT)
-      { 
+      {
       unsigned int val = (unsigned int)((char*)num)[0];
       pd = new MeshData<unsigned int>();
       MET_SwapByteIfSystemMSB(&val,MET_UINT);
       static_cast<MeshData<unsigned int>*>(pd)->m_Data = (unsigned int)val;
       }
     else if(m_PointDataType == MET_LONG)
-      { 
+      {
       long val = (long)((long*)num)[0];
-      pd = new MeshData<long>(); 
+      pd = new MeshData<long>();
       MET_SwapByteIfSystemMSB(&val,MET_LONG);
       static_cast<MeshData<long>*>(pd)->m_Data = (long)val;
       }
     else if(m_PointDataType == MET_ULONG)
-      { 
+      {
       unsigned long val = (unsigned long)((unsigned long*)num)[0];
       pd = new MeshData<unsigned long>();
       MET_SwapByteIfSystemMSB(&val,MET_ULONG);
       static_cast<MeshData<unsigned long>*>(pd)->m_Data = (unsigned long)val;
       }
     else if(m_PointDataType == MET_FLOAT)
-      { 
+      {
       float val = (float)((float*)num)[0];
-      pd = new MeshData<float>();   
-      MET_SwapByteIfSystemMSB(&val,MET_FLOAT); 
+      pd = new MeshData<float>();
+      MET_SwapByteIfSystemMSB(&val,MET_FLOAT);
       static_cast<MeshData<float>*>(pd)->m_Data = (float)val;
       }
     else if(m_PointDataType == MET_DOUBLE)
-      { 
+      {
       double val = (double)((double*)num)[0];
       pd = new MeshData<double>();
       MET_SwapByteIfSystemMSB(&val,MET_DOUBLE);
       static_cast<MeshData<double>*>(pd)->m_Data = val;
       }
     else  // assume double
-      { 
+      {
       METAIO_STREAM::cerr << "Warning: Mesh point data type not known - assuming double"
                 << METAIO_STREAM::endl;
       double val = (double)((double*)num)[0];
@@ -991,7 +990,7 @@ M_Read(void)
       MET_SwapByteIfSystemMSB(&val,MET_DOUBLE);
       static_cast<MeshData<double>*>(pd)->m_Data = val;
       }
-       
+
     delete [] num;
     pd->m_Id = (int)td;
     m_PointData.push_back(pd);
@@ -1021,14 +1020,14 @@ M_Read(void)
   MET_InitReadField(mF, "CellData", MET_NONE, false);
   mF->terminateRead = true;
   m_Fields.push_back(mF);
-  
+
   if(!MET_Read(*m_ReadStream, & m_Fields,'=',false,false))
     {
     METAIO_STREAM::cout << "MetaObject: Read: MET_Read Failed" << METAIO_STREAM::endl;
     return false;
     }
 
- 
+
   mF = MET_GetFieldRecord("NCellData", &m_Fields);
   if(mF->defined)
     {
@@ -1048,14 +1047,15 @@ M_Read(void)
   unsigned int gcCell = m_ReadStream->gcount();
   if(gcCell != cellDataSize)
     {
-    METAIO_STREAM::cout << "MetaMesh: m_Read: data not read completely" 
+    METAIO_STREAM::cout << "MetaMesh: m_Read: data not read completely"
               << METAIO_STREAM::endl;
     METAIO_STREAM::cout << "   ideal = " << cellDataSize << " : actual = " << gcCell << METAIO_STREAM::endl;
+    delete [] _data;
     return false;
     }
-  
+
   i=0;
-  for(j=0; j<m_NCellData; j++)  
+  for(j=0; j<(int)m_NCellData; j++)
     {
     MeshDataBase* cd;
 
@@ -1082,16 +1082,16 @@ M_Read(void)
     i+=elementSize;
 
     if(m_CellDataType == MET_CHAR)
-      { 
+      {
       char val = (char)((char*)num)[0];
-      cd = new MeshData<char>(); 
+      cd = new MeshData<char>();
       static_cast<MeshData<char>*>(cd)->m_Data = (char)val;
       }
     else if(m_CellDataType == MET_UCHAR)
       {
       unsigned char val = (unsigned char)((unsigned char*)num)[0];
       cd = new MeshData<unsigned char>();
-      static_cast<MeshData<unsigned char>*>(cd)->m_Data = (unsigned char)val; 
+      static_cast<MeshData<unsigned char>*>(cd)->m_Data = (unsigned char)val;
       }
     else if(m_CellDataType == MET_SHORT)
       {
@@ -1101,7 +1101,7 @@ M_Read(void)
       static_cast<MeshData<short>*>(cd)->m_Data = (short)val;
       }
     else if(m_CellDataType == MET_USHORT)
-      { 
+      {
       unsigned short val = (unsigned short)((unsigned short*)num)[0];
       cd = new MeshData<unsigned short>();
       MET_SwapByteIfSystemMSB(&val,MET_USHORT);
@@ -1115,42 +1115,42 @@ M_Read(void)
       static_cast<MeshData<int>*>(cd)->m_Data = (int)val;
       }
     else if(m_CellDataType == MET_UINT)
-      { 
+      {
       unsigned int val = (unsigned int)((unsigned int*)num)[0];
       cd = new MeshData<unsigned int>();
       MET_SwapByteIfSystemMSB(&val,MET_UINT);
       static_cast<MeshData<unsigned int>*>(cd)->m_Data = (unsigned int)val;
       }
     else if(m_CellDataType == MET_LONG)
-      { 
+      {
       long val = (long)((long*)num)[0];
       cd = new MeshData<long>();
       MET_SwapByteIfSystemMSB(&val,MET_LONG);
-      static_cast<MeshData<long>*>(cd)->m_Data = (long)val; 
+      static_cast<MeshData<long>*>(cd)->m_Data = (long)val;
       }
     else if(m_CellDataType == MET_ULONG)
-      { 
+      {
       unsigned long val = (unsigned long)((unsigned long*)num)[0];
       cd = new MeshData<unsigned long>();
       MET_SwapByteIfSystemMSB(&val,MET_ULONG);
       static_cast<MeshData<unsigned long>*>(cd)->m_Data = (long)val;
       }
     else if(m_CellDataType == MET_FLOAT)
-      { 
+      {
       float val = (float)((float*)num)[0];
       cd = new MeshData<float>();
       MET_SwapByteIfSystemMSB(&val,MET_FLOAT);
       static_cast<MeshData<float>*>(cd)->m_Data = (float)val;
       }
     else if(m_CellDataType == MET_DOUBLE)
-      { 
+      {
       double val = (double)((double*)num)[0];
       cd = new MeshData<double>();
       MET_SwapByteIfSystemMSB(&val,MET_DOUBLE);
       static_cast<MeshData<double>*>(cd)->m_Data = val;
       }
     else
-      { 
+      {
       METAIO_STREAM::cerr << "Warning: Mesh point data type not known - assuming double"
                 << METAIO_STREAM::endl;
       double val = (double)((double*)num)[0];
@@ -1158,7 +1158,7 @@ M_Read(void)
       MET_SwapByteIfSystemMSB(&val,MET_DOUBLE);
       static_cast<MeshData<double>*>(cd)->m_Data = val;
       }
-       
+
     delete [] num;
 
     cd->m_Id = (int)td;
@@ -1210,7 +1210,7 @@ M_Write(void)
         MET_DoubleToValue((double)pntX,m_PointType,data,i++);
         }
       it++;
-      }  
+      }
     m_WriteStream->write((char *)data,(m_NDims+1)*m_NPoints*elementSize);
     m_WriteStream->write("\n",1);
     delete []data;
@@ -1248,7 +1248,7 @@ M_Write(void)
         m_Fields.push_back(mF);
         }
 
-      m_NCells = m_CellListArray[i]->size();
+      m_NCells = static_cast<int>(m_CellListArray[i]->size());
       mF = new MET_FieldRecordType;
       MET_InitWriteField(mF, "NCells", MET_INT,m_NCells);
       m_Fields.push_back(mF);
@@ -1267,7 +1267,7 @@ M_Write(void)
       /** Then copy all cells */
       if(m_BinaryData)
         {
-        unsigned int totalCellsSize = m_CellListArray[i]->size()*(MET_CellSize[i]+1);
+        unsigned int totalCellsSize = static_cast<unsigned int>(m_CellListArray[i]->size()*(MET_CellSize[i]+1));
         char* data = new char[totalCellsSize*sizeof(int)];
         unsigned int d;
         int j=0;
@@ -1295,7 +1295,7 @@ M_Write(void)
         {
         CellListType::const_iterator it = m_CellListArray[i]->begin();
         CellListType::const_iterator itEnd = m_CellListArray[i]->end();
-  
+
         unsigned int d;
         while(it != itEnd)
           {
@@ -1316,7 +1316,7 @@ M_Write(void)
   if(m_CellLinks.size()>0)
     {
     MetaObject::ClearFields();
-    m_NCellLinks = m_CellLinks.size();
+    m_NCellLinks = static_cast<int>(m_CellLinks.size());
     MET_FieldRecordType * mF;
     mF = new MET_FieldRecordType;
     MET_InitWriteField(mF, "NCellLinks", MET_INT,m_NCellLinks);
@@ -1324,12 +1324,12 @@ M_Write(void)
 
     int cellLinksSize = 0;
     if(m_BinaryData)
-      {    
+      {
       CellLinkListType::const_iterator it = m_CellLinks.begin();
       CellLinkListType::const_iterator itEnd = m_CellLinks.end();
       while(it != itEnd)
         {
-        cellLinksSize += 2+(*it)->m_Links.size();
+        cellLinksSize += static_cast<int>(2+(*it)->m_Links.size());
         it++;
         }
       mF = new MET_FieldRecordType;
@@ -1362,7 +1362,7 @@ M_Write(void)
         MET_SwapByteIfSystemMSB(&clId,MET_INT);
         MET_DoubleToValue((double)clId,MET_INT,data,j++);
 
-        int linkSize = (*it)->m_Links.size();
+        int linkSize = static_cast<int>((*it)->m_Links.size());
         MET_SwapByteIfSystemMSB(&linkSize,MET_INT);
         MET_DoubleToValue((double)linkSize,MET_INT,data,j++);
 
@@ -1385,7 +1385,7 @@ M_Write(void)
       {
       CellLinkListType::const_iterator it = m_CellLinks.begin();
       CellLinkListType::const_iterator itEnd = m_CellLinks.end();
-  
+
       while(it != itEnd)
         {
         *m_WriteStream << (*it)->m_Id << " ";
@@ -1408,13 +1408,13 @@ M_Write(void)
   if(m_PointData.size()>0)
     {
     MetaObject::ClearFields();
-    m_NPointData = m_PointData.size();
+    m_NPointData = static_cast<int>(m_PointData.size());
     MET_FieldRecordType * mF;
     mF = new MET_FieldRecordType;
     MET_InitWriteField(mF, "NPointData", MET_INT,m_NPointData);
     m_Fields.push_back(mF);
 
-    int pointDataSize = 0;    
+    int pointDataSize = 0;
     PointDataListType::const_iterator it = m_PointData.begin();
     PointDataListType::const_iterator itEnd = m_PointData.end();
     while(it != itEnd)
@@ -1433,12 +1433,12 @@ M_Write(void)
 
     if(!MetaObject::M_Write())
       {
-      METAIO_STREAM::cout << "MetaMesh: M_Write: Error parsing file" 
+      METAIO_STREAM::cout << "MetaMesh: M_Write: Error parsing file"
                           << METAIO_STREAM::endl;
       return false;
       }
 
-    // Then copy all Point data : 
+    // Then copy all Point data :
     // Always binary to be compatible with everything
     it = m_PointData.begin();
     itEnd = m_PointData.end();
@@ -1456,13 +1456,13 @@ M_Write(void)
   if(m_CellData.size()>0)
     {
     MetaObject::ClearFields();
-    m_NCellData = m_CellData.size();
+    m_NCellData = static_cast<int>(m_CellData.size());
     MET_FieldRecordType * mF;
     mF = new MET_FieldRecordType;
     MET_InitWriteField(mF, "NCellData", MET_INT,m_NCellData);
     m_Fields.push_back(mF);
 
-    int cellDataSize = 0;    
+    int cellDataSize = 0;
     CellDataListType::const_iterator it = m_CellData.begin();
     CellDataListType::const_iterator itEnd = m_CellData.end();
     while(it != itEnd)
@@ -1482,12 +1482,12 @@ M_Write(void)
 
     if(!MetaObject::M_Write())
       {
-      METAIO_STREAM::cout << "MetaMesh: M_Write: Error parsing file" 
+      METAIO_STREAM::cout << "MetaMesh: M_Write: Error parsing file"
                           << METAIO_STREAM::endl;
       return false;
       }
 
-    // Then copy all Cell data : 
+    // Then copy all Cell data :
     // Always binary to be compatible with everything
     it = m_CellData.begin();
     itEnd = m_CellData.end();
