@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  MetaIO
+  Copyright 2000-2010 Insight Software Consortium
 
-  Program:   MetaIO
-  Module:    metaTube.cxx
-  Language:  C++
-  Date:      $Date$
-  Version:   $Revision$
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) Insight Software Consortium. All rights reserved.
-  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #ifdef _MSC_VER
 #pragma warning(disable:4702)
 #pragma warning(disable:4284)
@@ -111,7 +106,7 @@ MetaTube::
     TubePnt* pnt = *it;
     it++;
     delete pnt;
-  }  
+  }
   m_PointList.clear();
   M_Destroy();
 }
@@ -143,14 +138,14 @@ CopyInfo(const MetaObject * _object)
   MetaObject::CopyInfo(_object);
 }
 
-    
+
 
 void MetaTube::
 PointDim(const char* pointDim)
 {
   strcpy(m_PointDim,pointDim);
 }
-    
+
 const char* MetaTube::
 PointDim(void) const
 {
@@ -174,8 +169,8 @@ Root(bool root)
 {
   m_Root = root;
 }
-    
-bool MetaTube:: 
+
+bool MetaTube::
 Root(void) const
 {
   return m_Root;
@@ -207,7 +202,7 @@ Clear(void)
     TubePnt* pnt = *it;
     it++;
     delete pnt;
-  }  
+  }
   m_PointList.clear();
 
   m_ParentPoint= -1;
@@ -216,7 +211,7 @@ Clear(void)
   strcpy(m_PointDim, "x y z r v1x v1y v1z v2x v2y v2z tx ty tz red green blue alpha id");
   m_ElementType = MET_FLOAT;
 }
-        
+
 /** Destroy tube information */
 void MetaTube::
 M_Destroy(void)
@@ -326,9 +321,9 @@ M_Read(void)
     {
     METAIO_STREAM::cout << "MetaTube: M_Read: Parsing Header" << METAIO_STREAM::endl;
     }
- 
+
   MET_FieldRecordType * mF;
- 
+
   mF = MET_GetFieldRecord("ParentPoint", &m_Fields);
   if(mF->defined)
     {
@@ -339,7 +334,7 @@ M_Read(void)
   mF = MET_GetFieldRecord("Root", &m_Fields);
   if(mF->defined)
     {
-    if(*((char *)(mF->value)) == 'T' 
+    if(*((char *)(mF->value)) == 'T'
       || *((char*)(mF->value)) == 't'
       || *((char*)(mF->value)) == '1')
       {
@@ -388,15 +383,15 @@ M_Read(void)
 
   int pntDim;
   char** pntVal = NULL;
-  MET_StringToWordArray(m_PointDim, &pntDim, &pntVal); 
- 
+  MET_StringToWordArray(m_PointDim, &pntDim, &pntVal);
+
   if(META_DEBUG)
     {
-    METAIO_STREAM::cout << "MetaTube: Parsing point dim" << METAIO_STREAM::endl; 
+    METAIO_STREAM::cout << "MetaTube: Parsing point dim" << METAIO_STREAM::endl;
     }
 
   int j;
-  for(j = 0; j < pntDim; j++) 
+  for(j = 0; j < pntDim; j++)
     {
     if(!strcmp(pntVal[j], "x") || !strcmp(pntVal[j], "X"))
       {
@@ -465,7 +460,7 @@ M_Read(void)
       {
       posGreen = j;
       }
-    
+
     if(!strcmp(pntVal[j], "blue"))
       {
       posBlue = j;
@@ -487,7 +482,7 @@ M_Read(void)
   delete [] pntVal;
 
   float v[30];
-  
+
   if(m_Event)
     {
     m_Event->StartReading(m_NPoints);
@@ -505,20 +500,22 @@ M_Read(void)
     int gc = m_ReadStream->gcount();
     if(gc != readSize)
       {
-      METAIO_STREAM::cout << "MetaLine: m_Read: data not read completely" 
+      METAIO_STREAM::cout << "MetaLine: m_Read: data not read completely"
                 << METAIO_STREAM::endl;
-      METAIO_STREAM::cout << "   ideal = " << readSize 
+      METAIO_STREAM::cout << "   ideal = " << readSize
                 << " : actual = " << gc << METAIO_STREAM::endl;
       return false;
+      delete [] posDim;
+      delete [] _data;
       }
 
     i=0;
     int d;
     unsigned int k;
-    for(j=0; j<(int)m_NPoints; j++) 
+    for(j=0; j<(int)m_NPoints; j++)
       {
       TubePnt* pnt = new TubePnt(m_NDims);
-      
+
       for(d=0; d<m_NDims; d++)
         {
         char* num = new char[sizeof(float)];
@@ -530,7 +527,7 @@ M_Read(void)
         MET_SwapByteIfSystemMSB(&td,MET_FLOAT);
         i+=sizeof(float);
         pnt->m_X[d] = td;
-        delete [] num; 
+        delete [] num;
         }
 
       char* num = new char[sizeof(float)];
@@ -557,7 +554,7 @@ M_Read(void)
         pnt->m_V1[d] = (float)td;
         delete [] num;
         }
-    
+
       if(m_NDims==3)
         {
         for(d = 0; d < m_NDims; d++)
@@ -574,7 +571,7 @@ M_Read(void)
           delete [] num;
           }
         }
-      
+
       for(d = 0; d < m_NDims; d++)
         {
         num = new char[sizeof(float)];
@@ -588,7 +585,7 @@ M_Read(void)
         pnt->m_T[d] = (float)td;
         delete [] num;
         }
-           
+
       for(d=0; d<4; d++)
         {
         num = new char[sizeof(float)];
@@ -610,7 +607,7 @@ M_Read(void)
         }
       int id = (int)((int*)num)[0];
       MET_SwapByteIfSystemMSB(&id,MET_INT);
-      i+=sizeof(int); 
+      i+=sizeof(int);
       pnt->m_ID = id;
       delete [] num;
 
@@ -620,7 +617,7 @@ M_Read(void)
     }
   else
     {
-    for(j=0; j<(int)m_NPoints; j++) 
+    for(j=0; j<(int)m_NPoints; j++)
       {
       if(m_Event)
         {
@@ -644,38 +641,38 @@ M_Read(void)
 
       if(posV1x>=0 && posV1x<pntDim)
         {
-        pnt->m_V1[0] = v[posV1x]; 
-        if(posV1y >= 0 && posV1y<pntDim) 
+        pnt->m_V1[0] = v[posV1x];
+        if(posV1y >= 0 && posV1y<pntDim)
           {
-          pnt->m_V1[1] = v[posV1y]; 
+          pnt->m_V1[1] = v[posV1y];
           }
-        if(posV1z >= 0 && m_NDims>2 && posV1z<pntDim) 
+        if(posV1z >= 0 && m_NDims>2 && posV1z<pntDim)
           {
-          pnt->m_V1[2] = v[posV1z]; 
+          pnt->m_V1[2] = v[posV1z];
           }
         }
-      if(posV2x >= 0 && posV2x<pntDim) 
+      if(posV2x >= 0 && posV2x<pntDim)
         {
-        pnt->m_V2[0] = v[posV2x]; 
-        if(posV2y >= 0 && posV2y<pntDim) 
+        pnt->m_V2[0] = v[posV2x];
+        if(posV2y >= 0 && posV2y<pntDim)
           {
-          pnt->m_V2[1] = v[posV2y]; 
+          pnt->m_V2[1] = v[posV2y];
           }
-        if(posV2z >= 0 && m_NDims>2 && posV2z<pntDim) 
+        if(posV2z >= 0 && m_NDims>2 && posV2z<pntDim)
           {
-          pnt->m_V2[2] = v[posV2z]; 
+          pnt->m_V2[2] = v[posV2z];
           }
         }
-      if(posTx >= 0 && posTx<pntDim) 
+      if(posTx >= 0 && posTx<pntDim)
         {
-        pnt->m_T[0] = v[posTx]; 
-        if(posTy >= 0 && posTy<pntDim) 
+        pnt->m_T[0] = v[posTx];
+        if(posTy >= 0 && posTy<pntDim)
           {
-          pnt->m_T[1] = v[posTy]; 
+          pnt->m_T[1] = v[posTy];
           }
-        if(posTz >= 0 && m_NDims>2 && posTz<pntDim) 
+        if(posTz >= 0 && m_NDims>2 && posTz<pntDim)
           {
-          pnt->m_T[2] = v[posTz]; 
+          pnt->m_T[2] = v[posTz];
           }
         }
 
@@ -688,39 +685,39 @@ M_Read(void)
         {
         pnt->m_Color[1] = v[posGreen];
         }
-    
+
       if(posBlue >= 0 && posBlue < pntDim)
         {
         pnt->m_Color[2] = v[posBlue];
         }
-    
+
       if(posAlpha >= 0 && posAlpha < pntDim)
         {
         pnt->m_Color[3] = v[posAlpha];
         }
- 
+
       if(posID >= 0 && posID < pntDim)
         {
         pnt->m_ID = (int)v[posID];
         }
- 
+
       m_PointList.push_back(pnt);
       }
 
-      
+
     char c = ' ';
     while( (c!='\n') && (!m_ReadStream->eof()))
       {
       c = m_ReadStream->get();// to avoid unrecognize charactere
       }
     }
-  
+
   if(m_Event)
     {
     m_Event->StopReading();
     }
 
-  delete []posDim;
+  delete [] posDim;
   return true;
 }
 
@@ -754,7 +751,12 @@ M_Write(void)
     int elementSize;
     MET_SizeOfType(m_ElementType, &elementSize);
 
-    char* data = new char[(m_NDims*(2+m_NDims)+10)*m_NPoints*elementSize];
+    const unsigned int writeSize = m_NPoints*(m_NDims*(2+m_NDims)+10)*elementSize;
+
+    char* data = new char[writeSize];
+
+    memset( data, '\0', writeSize );
+
     int i=0;
     int d;
     while(it != itEnd)
@@ -769,14 +771,14 @@ M_Write(void)
       float r = (*it)->m_R;
       MET_SwapByteIfSystemMSB(&r,MET_FLOAT);
       MET_DoubleToValue((double)r,m_ElementType,data,i++);
-      
+
       for(d = 0; d < m_NDims; d++)
         {
         float v = (*it)->m_V1[d];
         MET_SwapByteIfSystemMSB(&v,MET_FLOAT);
         MET_DoubleToValue((double)v,m_ElementType,data,i++);
         }
-    
+
       if(m_NDims==3)
         {
         for(d = 0; d < m_NDims; d++)
@@ -786,14 +788,14 @@ M_Write(void)
           MET_DoubleToValue((double)v2,m_ElementType,data,i++);
           }
         }
-      
+
       for(d = 0; d < m_NDims; d++)
         {
         float t = (*it)->m_T[d];
         MET_SwapByteIfSystemMSB(&t,MET_FLOAT);
         MET_DoubleToValue((double)t,m_ElementType,data,i++);
         }
-     
+
       for(d=0; d<4; d++)
         {
         float c = (*it)->m_Color[d];
@@ -807,8 +809,7 @@ M_Write(void)
       it++;
       }
 
-    m_WriteStream->write((char *)data,
-                         (m_NDims*(2+m_NDims)+10)*m_NPoints*elementSize);
+    m_WriteStream->write((char *)data,writeSize);
     m_WriteStream->write("\n",1);
     delete [] data;
     }
@@ -816,7 +817,7 @@ M_Write(void)
     {
     PointListType::const_iterator it = m_PointList.begin();
     PointListType::const_iterator itEnd = m_PointList.end();
-  
+
     int d;
     while(it != itEnd)
       {
@@ -824,14 +825,14 @@ M_Write(void)
         {
         *m_WriteStream << (*it)->m_X[d] << " ";
         }
-      
+
       *m_WriteStream << (*it)->m_R << " ";
 
       for(d = 0; d < m_NDims; d++)
         {
         *m_WriteStream << (*it)->m_V1[d] << " ";
         }
-   
+
       if(m_NDims>=3)
         {
         for(d = 0; d < m_NDims; d++)
@@ -839,7 +840,7 @@ M_Write(void)
           *m_WriteStream << (*it)->m_V2[d] << " ";
           }
         }
-      
+
       for(d = 0; d < m_NDims; d++)
         {
         *m_WriteStream << (*it)->m_T[d] << " ";
